/*
 * dh_am3517.c - board file for DH COM AM3517
 *
 * Copyright (C) 2011 DH electronics GmbH
 *
 * Author:	Ludwig Zenz <lzenz@dh-electronics.de>
 *
 *
 *
 * Based on: am3517evm.c - board file for TI's AM3517 family of devices.
 *
 * Author: Vaibhav Hiremath <hvaibhav@ti.com>
 *
 * Based on ti/evm/evm.c
 *
 * Copyright (C) 2009 Texas Instruments Incorporated
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <config.h>
#include <common.h>
#include <netdev.h>
#include <asm/io.h>
#include <asm/arch/mem.h>
#include <asm/arch/mux.h>
#include <asm/arch/sys_proto.h>
#include <asm/arch/emac_defs.h>
#include <asm/arch/clocks.h>
#include <asm/arch/clocks_omap3.h>
#include <asm/arch/gpio.h>
#include <asm/types.h>
#include <i2c.h>
#include <net.h>
#include <mcp7941x_rtc.h>  /* defines to use the rtc mcp79411 */
#include <asm/mach-types.h>
#include <bmp_layout.h>
#include <nand.h>
#include <dh_settings.h>
#include "dh_am3517.h"

#ifdef CONFIG_DRIVER_SMC91111
#include "../drivers/net/smc91111.h"
#endif
#ifdef CONFIG_DRIVER_LAN91C96
#include "../drivers/net/lan91c96.h"
#endif


#ifdef CONFIG_DRIVER_TI_EMAC
extern void davinci_eth_set_mac_addr (const u_int8_t *addr);
#endif
/*
 * Prototypes
 */
void set_DHCOM_GPIOS_direction_and_state(void);

// TI
#define AM3517_IP_SW_RESET	0x48002598
#define CPGMACSS_SW_RST		(1 << 1)

#ifdef CONFIG_CMD_MMC
#include <asm/arch/mmc_omap3.h>
#include <fsl_esdhc.h>
#endif

/* pointer to global data is kept in Register R8 */
DECLARE_GLOBAL_DATA_PTR;

extern int do_mmcops(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[]);
extern void i2c_init(int speed, int unused);

/*
 * Switch on (bOnOff = 1) or switch off (bOnOff = 0) display backlight
 */
void board_switch_backlight(int bOnOff)
{
   int BacklightPin;
   u8  ulNoBacklightGPIO = 0;

   /* Disable lcd display backlight */
    switch (((gd->dh_board_settings.wLCDConfigFlags & SETTINGS_LCD_BL_EN_GPIO_FLAG) >> 7))
    {
        case 0x0:
            ulNoBacklightGPIO = 1;
            break;
        case 0x1:
            // DHCOM GPIO_A
            BacklightPin = DHCOM_GPIO_A;
            break;
        case 0x2:
            // DHCOM GPIO_B
            BacklightPin = DHCOM_GPIO_B;
            break;
        case 0x3:
            // DHCOM GPIO_C
            BacklightPin = DHCOM_GPIO_C;
            break;
        case 0x4:
            // DHCOM GPIO_D
            BacklightPin = DHCOM_GPIO_D;
            break;
        case 0x5:
            // DHCOM GPIO_E
            BacklightPin = DHCOM_GPIO_E;
            break;
        case 0x6:
            // DHCOM GPIO_F
            BacklightPin = DHCOM_GPIO_F;
            break;
        case 0x7:
            // DHCOM GPIO_G
            BacklightPin = DHCOM_GPIO_G;
            break;
        case 0x8:
            // DHCOM GPIO_H
            BacklightPin = DHCOM_GPIO_H;
            break;
        case 0x9:
            // DHCOM GPIO_I
            BacklightPin = DHCOM_GPIO_I;
            break;
        default:
            ulNoBacklightGPIO = 1;
            break;
    }

    if(ulNoBacklightGPIO == 0) /* if we have a backlight enable gpio ... */
    {
        // Set to output
    	omap_set_gpio_direction(BacklightPin, 0);
        if(gd->dh_board_settings.wLCDConfigFlags & SETTINGS_LCD_IBL_FLAG)
        	omap_set_gpio_dataout(BacklightPin, !bOnOff); /* Set to high */
        else
        	omap_set_gpio_dataout(BacklightPin, bOnOff); /* Set to low */
    }
}

/*
 * Switch off display backlight
 */
void board_disable_backlight(void)
{
	board_switch_backlight(0);
}

/*
 * Switch on display backlight
 */
void board_enable_backlight(void)
{
	board_switch_backlight(1);
}

#ifdef CONFIG_LCD

/*
 * Configure Display Backlight PWM
 */
void board_set_backlight_pwm(int pwm_percent)
{
	struct gptimer *gpt11_base = (struct gptimer *)OMAP34XX_GPT11;
	struct prcm *prcm_base = (struct prcm *)PRCM_BASE;

	/* parse to valid value */
	if (pwm_percent > 100) pwm_percent = 100;
	if (pwm_percent < 0) pwm_percent = 0;

	/* Depending on PWM_POL Flag disable BL */
	if( gd->dh_board_settings.wLCDConfigFlags & SETTINGS_LCD_PWM_POL_FLAG)
		pwm_percent = 100 - pwm_percent;

	/* Handle 0 percent brightness */
	if (pwm_percent == 0)
	{
		/* stop timer and set default state to low */
		writel( 0x00000000 , &gpt11_base->tclr);
		return;
	}

	/* Handle full brightness */
	if (pwm_percent == 100)
	{
		/* stop timer and set default state to high */
		writel( 0x00000080 , &gpt11_base->tclr);
		return;
	}


	/* Select Clock-Source of GPTIMER 11 */
	writel(readl(&prcm_base->clksel_core) | (1 << 7), &prcm_base->clksel_core); /* use system-clock as clock-source */

	/* Enable GPTIMER 11 */
	writel(readl(&prcm_base->iclken1_core) | 1 << 12, &prcm_base->iclken1_core); /* enable Interface Clock GPTIMER 11 */
	writel(readl(&prcm_base->fclken1_core) | 1 << 12, &prcm_base->fclken1_core); /* enable Functional Clock GPTIMER 11 */

	/* stop timer */
	writel( 0 , &gpt11_base->tclr);

	/* load value */
	writel( 0xFFFFFF9B , &gpt11_base->tldr); /* this results in 128 kHz */

	/* load value */
	writel( 0xFFFFFF9B , &gpt11_base->tcrr);

	/* match value */
	writel( 0xFFFFFF9B + pwm_percent, &gpt11_base->tmar); /* configure the pulse with */

	/* bit 0:		start timer */
	/* bit 1:		autoreload mode */
	/* bit 5: 		no prescaler */
	/* bit 6: 		compare enabled */
	/* bit 10-11: 	overflow and match trigger */
	/* bit 12: 		toggle modulation */
	writel( (0x01 << 12) | (0x01 << 6) | (0x02 << 10) | (0x01 << 1)| 0x01 , &gpt11_base->tclr);
}

/*
 * Read splashimage from persistant memory
 */
int board_get_splashimage (void)
{
	char *s;
	int ret = 0;
	ulong addr = 0;
	nand_info_t *nand;
	loff_t off;
	size_t size = 0;
	size_t realsize = 0;
	bmp_image_t *splash_bmp;

	debug("[LCD] board_get_splashimage() called\n");

	nand = &nand_info[nand_curr_device];		/* Get nand-info structure from current device */

	/* Get ddr2 Offset from copy-buffer */
	s = getenv("rambufferaddr");
	addr = (ulong)simple_strtoul(s, NULL, 16);

	/* Set Splash-Image reference */
	splash_bmp = (bmp_image_t *)addr;

	/* Get NAND-Flash Offset from splash-image     */
	s = getenv("splashimage");
	off = (ulong)simple_strtoul(s, NULL, 16);

	/* Set size to read one block from nand */
	size = nand->writesize;

	/* Copy first page of splash-image to SD-RAM */
	DISABLE_PRINTF()
	ret = nand_read_skip_bad(nand, off, &size, (u_char *)addr);
    /* Set printf enable/disable setting */

	SESSION_DEPENDED_PRINTF_ENABLE()
	if(ret)	{ printf("[LCD] Error: reading splash-image failure!\n"); return ret;	}

	/* Check for a valid image signature */
	if (!((splash_bmp->header.signature[0]=='B') && (splash_bmp->header.signature[1]=='M')))
	{
		printf("[LCD] No splash-image found!\n");
		return 1;
	}

	// get size from bmp-header (we do not have to round up to full pages because one block is already loaded)
	realsize = splash_bmp->header.file_size;
	size = realsize / nand->writesize;
	size = size * nand->writesize;

	// debug output: information about the found bmp-image

	debug("[LCD] Splash-Image: size of the bmp-image in bytes: 0x%08lX bytes\n", (unsigned long)realsize);
	debug("[LCD] Splash-Image: additional data to read from nand: 0x%08lX bytes\n", (unsigned long)size);
	debug("[LCD] Splash-Image: bmp dataoffset = %i bytes\n",splash_bmp->header.data_offset);
	debug("[LCD] Splash-Image: bits per pixel: %i\n", splash_bmp->header.bit_count);


	/* Copy the remaining pages to SD-RAM */
	off = off + nand->writesize; // Set off to the next page
	addr = addr + nand->writesize; // Set addr to the next pagearea in ddr2

	DISABLE_PRINTF()
	ret = nand_read_skip_bad(nand, off, &size, (u_char *)addr);
	SESSION_DEPENDED_PRINTF_ENABLE()

	if(ret)	{ printf("[LCD] Error: reading splash-image failure!\n"); return ret;	}

	debug("[LCD] board_get_splashimage() successful\n");

	return 0;
}

/*
 * basic hardware initializations for lcd display
 */
void lcdc_hw_init(void)
{
	/* Request and configure gpio for Backlight and Display power supply */
	board_enable_backlight();

#define PWM_MODE
#if defined(PWM_MODE)

	board_set_backlight_pwm(0);

#else
	/* Request and enable Panel Backlight PWM Pin */
	omap_request_gpio(DEFAULT_BACKLIGHT_PWM);
	omap_set_gpio_direction(DEFAULT_BACKLIGHT_PWM,0);

	/* Depending on PWM_POL Flag disable BL */
	if( gd->dh_board_settings.wLCDConfigFlags & SETTINGS_LCD_PWM_POL_FLAG)
		omap_set_gpio_dataout(DEFAULT_BACKLIGHT_PWM,0);
	else
		omap_set_gpio_dataout(DEFAULT_BACKLIGHT_PWM,1);


#endif

	// FIXME: Framebuffer base hardcoded for now
	gd->fb_base = CONFIG_FRAMEBUFFER_BASE;
}
#endif

/*
 * DHCOM specific
 * Get the hardware-revision from HW-Code Pins
 */
int dh_get_hw_revision(void)
{
    int hw_revision = 0;

	/* request gpios */
    omap_request_gpio(26);
    omap_request_gpio(27);
    omap_request_gpio(186);

    /* Set to input */
	omap_set_gpio_direction(26, 1);
	omap_set_gpio_direction(27, 1);
	omap_set_gpio_direction(186, 1);

    /* read gpios to hw-revision */
	if (omap_get_gpio_datain(26))
		hw_revision |= 0x01;

	if (omap_get_gpio_datain(27))
		hw_revision |= 0x02;

	if (omap_get_gpio_datain(186))
		hw_revision |= 0x04;

	/* calculate hw-revision */
    hw_revision++;
    hw_revision *= 100;

    return hw_revision;
}

/*
 * DHCOM specific
 * Get the SDRAM-Size from SDRAM-Code Pins
 */
int dh_get_sdram_size_mb(void)
{
	int index = 0;
    int sdram_code = 0;
    int sdram_size = 64; /* initialize with minimum size */

	/* request gpios */
    omap_request_gpio(160);
    omap_request_gpio(24);
    omap_request_gpio(25);

    /* Set to input */
	omap_set_gpio_direction(160, 1);
	omap_set_gpio_direction(24, 1);
	omap_set_gpio_direction(25, 1);

    /* read gpios to hw-revision */
	if (omap_get_gpio_datain(160))
		sdram_code |= 0x01;

	if (omap_get_gpio_datain(24))
		sdram_code |= 0x02;

	if (omap_get_gpio_datain(25))
		sdram_code |= 0x04;

	/* calculate sdram size (power of 2) */
    for (index = 0; index < sdram_code; index++)
    {
    	sdram_size *= 2;
    }

    return sdram_size;
}

/*
 * DHCOM specific
 * Get the NAND-Size from NAND-Code Pins
 */
int dh_get_nand_size_mb(void)
{
	int index = 0;
    int nand_code = 0;
    int nand_size = 128; /* initialize with minimum size */

	/* request gpios */
    omap_request_gpio(176);
    omap_request_gpio(154);

    /* Set to input */
	omap_set_gpio_direction(176, 1);
	omap_set_gpio_direction(154, 1);

    /* read gpios to hw-revision */
	if (omap_get_gpio_datain(176))
		nand_code |= 0x01;

	if (omap_get_gpio_datain(154))
		nand_code |= 0x02;


	/* calculate sdram size (power of 2) */
    for (index = 0; index < nand_code; index++)
    {
    	nand_size *= 2;
    }

    return nand_size;
}

/*
 * DHCOM specific
 * Set Input/Output of a gpio and if Output to required default state
 */
void set_gpio_direction_and_state(u32 gpio_settings_mask, u32 omap_gpio_number)
{
	    /* Set GPIO direction and state */
	    if(gd->dh_board_settings.wGPIODir & gpio_settings_mask)
	    {
	    	omap_set_gpio_direction(omap_gpio_number, 1); /* Set to input */
	    }
	    else
	    {
	    	omap_set_gpio_direction(omap_gpio_number, 0); /* Set to output */

	    	/* set default state */
	        if(gd->dh_board_settings.wGPIOState & gpio_settings_mask)
	         	omap_set_gpio_dataout(omap_gpio_number, 1); /* Set to high */
	        else
	        	omap_set_gpio_dataout(omap_gpio_number, 0); /* Set to low */
	    }
}

/*
 * DHCOM specific
 * Set Input/Output and if Output to required default
 * state depending on dh_board_settings for DHCOM GPIO A - W
 */
void set_DHCOM_GPIOS_direction_and_state(void)
{
	u32 index;
	u32 aiDHCOMgpios[23] = {DHCOM_GPIO_A, DHCOM_GPIO_B, DHCOM_GPIO_C, DHCOM_GPIO_D, DHCOM_GPIO_E, DHCOM_GPIO_F,
							DHCOM_GPIO_G, DHCOM_GPIO_H, DHCOM_GPIO_I, DHCOM_GPIO_J, DHCOM_GPIO_K, DHCOM_GPIO_L,
							DHCOM_GPIO_M, DHCOM_GPIO_N, DHCOM_GPIO_O, DHCOM_GPIO_P, DHCOM_GPIO_Q, DHCOM_GPIO_R,
							DHCOM_GPIO_S, DHCOM_GPIO_T, DHCOM_GPIO_U, DHCOM_GPIO_V, DHCOM_GPIO_W};

	for (index = 0; index < 23; index++)
	{

		/* shift 0x01 to generate a mask for DHCOM settings block values */
		set_gpio_direction_and_state(1 << index, aiDHCOMgpios[index]);

	}
	
	/* Enable Ethernet VIO Switched */
	omap_request_gpio(DHCOM_ETH_VIO_GPIO);
	omap_set_gpio_direction(DHCOM_ETH_VIO_GPIO, 0);
	omap_set_gpio_dataout(DHCOM_ETH_VIO_GPIO, 0);

}

/*
 * Routine: board_init
 * Description: Early hardware init.
 */
int board_init(void)
{
    int ret_value = 0;
    uchar ucBuffer[DHCOM_DISPLAY_SETTINGS_SIZE];

	gpmc_init(); /* in SRAM or SDRAM, finish GPMC */
	/* board id for Linux */
	gd->bd->bi_arch_number = MACH_TYPE_DHCM3517;
	/* boot param addr */
	gd->bd->bi_boot_params = (OMAP34XX_SDRC_CS0 + 0x100);

	/* setting up pin mux */
	set_muxconf_regs();

	/* initialize DH Global Data */
	    gd->dh_board_settings.cLength = 		DEFAULT_SETTINGS_BLOCK_LENGTH;
	    gd->dh_board_settings.cDisplayID = 		DEFAULT_SETTINGS_DISPLAY_ID;
	    gd->dh_board_settings.wValidationID = 	0;
	    gd->dh_board_settings.wYResolution = 	DEFAULT_SETTINGS_Y_RESOLUTION;
	    gd->dh_board_settings.wXResolution = 	DEFAULT_SETTINGS_X_RESOLUTION;
	    gd->dh_board_settings.wLCDConfigFlags = DEFAULT_SETTINGS_LCD_CONFIG_FLAGS;
	    gd->dh_board_settings.wPixelClock = 	DEFAULT_SETTINGS_PIXEL_CLOCK;
	    gd->dh_board_settings.wVPulseWidth = 	DEFAULT_SETTINGS_V_PULSE_WIDTH;
	    gd->dh_board_settings.wHPulseWidth = 	DEFAULT_SETTINGS_H_PULSE_WIDTH;
	    gd->dh_board_settings.wHBackPorch = 	DEFAULT_SETTINGS_H_BACK_PORCH;
	    gd->dh_board_settings.wHFrontPorch = 	DEFAULT_SETTINGS_H_FRONT_PORCH;
	    gd->dh_board_settings.wVBackPorch = 	DEFAULT_SETTINGS_V_BACK_PORCH;
	    gd->dh_board_settings.wVFrontPorch = 	DEFAULT_SETTINGS_V_FRONT_PORCH;
	    gd->dh_board_settings.cACBiasTrans = 	DEFAULT_SETTINGS_AC_BIAS_TRANS;
	    gd->dh_board_settings.cACBiasFreq = 	DEFAULT_SETTINGS_AC_BIAS_FREQ;
	    gd->dh_board_settings.cDatalines = 		DEFAULT_SETTINGS_DATALINES;
	    gd->dh_board_settings.wGPIODir = 		DEFAULT_SETTINGS_GPIO_DIRECTION;
	    gd->dh_board_settings.wGPIOState = 		DEFAULT_SETTINGS_GPIO_STATE;
	    gd->dh_board_settings.wHWConfigFlags = 	DEFAULT_SETTINGS_HW_CONFIG_FLAGS;

	    CLEAR_DH_GD_FLAG(	NO_OSIMAGE_FOUND_FLAG		);
	    CLEAR_DH_GD_FLAG(	TRY_TO_CREATE_BBT_FLAG		);
	    CLEAR_DH_GD_FLAG(	DISABLE_PRINTF_FLAG			);
	    CLEAR_DH_GD_FLAG(	UPDATE_ACTIVE_FLAG			);
	    SET_DH_GD_FLAG(		BOOT_AFTER_UPDATE_FLAG		);
	    CLEAR_DH_GD_FLAG(	SESSION_PRINTF_DISABLED_FLAG	);

	    gd->dh_ulSDRAMSize = 0;
	    ulong addr = CONFIG_SETTINGSBIN_TRANSFER_BUFFER;

	    /* copy settingsblock from dram into the dh_board_settings structure */

	    gd->dh_board_settings.wValidationID = ((readl(addr) & 0xFFFF0000) >> 16);

	    // settings.bin file Valid Mask should be "DH" = 0x4844
	    if(gd->dh_board_settings.wValidationID == 0x4844)
	    {
	        gd->dh_board_settings.cLength = (readl(addr) & 0xFF);
	        gd->dh_board_settings.cDisplayID = ((readl(addr) & 0xFF00) >> 8);

	        gd->dh_board_settings.wYResolution = (readl(addr+4) & 0xFFFF);
	        gd->dh_board_settings.wXResolution = ((readl(addr+4) & 0xFFFF0000) >> 16);

	        gd->dh_board_settings.wLCDConfigFlags = (readl(addr+8) & 0xFFFF);
	        gd->dh_board_settings.wPixelClock = ((readl(addr+8) & 0xFFFF0000) >> 16);

	        gd->dh_board_settings.wVPulseWidth = (readl(addr+12) & 0xFFFF);
	        gd->dh_board_settings.wHPulseWidth = ((readl(addr+12) & 0xFFFF0000) >> 16);

	        gd->dh_board_settings.wHBackPorch = (readl(addr+16) & 0xFFFF);
	        gd->dh_board_settings.wHFrontPorch = ((readl(addr+16) & 0xFFFF0000) >> 16);

	        gd->dh_board_settings.wVBackPorch = (readl(addr+20) & 0xFFFF);
	        gd->dh_board_settings.wVFrontPorch = ((readl(addr+20) & 0xFFFF0000) >> 16);

	        gd->dh_board_settings.cACBiasTrans = (readl(addr+24) & 0xFF);
	        gd->dh_board_settings.cACBiasFreq = ((readl(addr+24) & 0xFF00) >> 8);
	        gd->dh_board_settings.cDatalines = ((readl(addr+24) & 0xFFFF0000) >> 16);

	        gd->dh_board_settings.wGPIODir = (readl(addr+32));
	        gd->dh_board_settings.wGPIOState = (readl(addr+36));

	        gd->dh_board_settings.wHWConfigFlags = (readl(addr+40) & 0xFFFF);
	    }
	    else
	    {
	    	// Use default values
	    	// Debug output is not available at this boot stage
	    }

	    /*
	     * setup DHCOM gpios depending on settings.bin file
	     */
	    set_DHCOM_GPIOS_direction_and_state();

	    DISABLE_PRINTF()

	    /* Check and Read Display data from EEPROM if enabled */
	    if((gd->dh_board_settings.wHWConfigFlags & SETTINGS_HW_EN_DISP_ADPT_EE_CHK) != 0)
	    {
	        /* Set i2c driver to use i2c bus 3  */
	    	i2c_set_bus_num(3);

	        i2c_init (CONFIG_SYS_I2C_SPEED, CONFIG_SYS_I2C_SLAVE);
	        ret_value = i2c_read(DISPLAY_ADAPTER_EEPROM_ADDR, 0, 1, &ucBuffer[0], DHCOM_DISPLAY_SETTINGS_SIZE);

	        /* Set i2c driver back to use i2c bus 1 */
	        i2c_set_bus_num(1);

	        if((ret_value == 0) && (ucBuffer[2] == 'D') && (ucBuffer[3] == 'H'))
	        {
	            gd->dh_board_settings.cDisplayID = ucBuffer[1];

	            gd->dh_board_settings.wYResolution = (ucBuffer[5] << 8) | ucBuffer[4];
	            gd->dh_board_settings.wXResolution = (ucBuffer[7] << 8) | ucBuffer[6];

	            gd->dh_board_settings.wLCDConfigFlags = (ucBuffer[9] << 8) | ucBuffer[8];
	            gd->dh_board_settings.wPixelClock = (ucBuffer[11] << 8) | ucBuffer[10];

	            gd->dh_board_settings.wVPulseWidth = (ucBuffer[13] << 8) | ucBuffer[12];
	            gd->dh_board_settings.wHPulseWidth = (ucBuffer[15] << 8) | ucBuffer[14];

	            gd->dh_board_settings.wHBackPorch = (ucBuffer[17] << 8) | ucBuffer[16];
	            gd->dh_board_settings.wHFrontPorch = (ucBuffer[19] << 8) | ucBuffer[18];

	            gd->dh_board_settings.wVBackPorch = (ucBuffer[21] << 8) | ucBuffer[20];
	            gd->dh_board_settings.wVFrontPorch = (ucBuffer[23] << 8) | ucBuffer[22];

	            gd->dh_board_settings.cACBiasTrans = ucBuffer[24];
	            gd->dh_board_settings.cACBiasFreq = ucBuffer[25];
	            gd->dh_board_settings.cDatalines = (ucBuffer[27] << 8) | ucBuffer[26];
	        }
	    }

	    /* Set printf enable/disable setting */
	    if((gd->dh_board_settings.wHWConfigFlags & SETTINGS_HW_SILENT_MODE_FLAG) == 1)
	    	SET_DH_GD_FLAG(SESSION_PRINTF_DISABLED_FLAG); /* disable uart output for this session */
	    else
	    	CLEAR_DH_GD_FLAG(SESSION_PRINTF_DISABLED_FLAG); /* enable uart output for this session */

	    SESSION_DEPENDED_PRINTF_ENABLE()

	/*
	 * basic hardware initialization for lcd-display
	 */
	#ifdef CONFIG_LCD
	lcdc_hw_init();
	#endif

	return 0;
}



#ifdef BOARD_LATE_INIT
int board_late_init(void)
{
	u8 u8MAC[9];
    char cMACAddress[18] = {"00:00:00:00:00:00"};
    char *sEnvMACAddress;
    int status = 0;

	/* ----------------------------------------------------------------------------- */
	/* Save environment to NAND-flash                                                */
	if (getenv("ENVinFlash") == NULL)
	{
		DISABLE_PRINTF() // be quiet during saving env to flash

		run_command(getenv("save_ENV_to_flash"),0);

		SESSION_DEPENDED_PRINTF_ENABLE()
		/* display that operation was successful */
		printf("env:   writing environment to flash/nand-flash was successful!\n");
	}

	/* ----------------------------------------------------------------------------- */
	/* Load MAC Address from EEPROM on first boot                                    */
	 sEnvMACAddress = getenv("ethaddr");

	 if(strcmp (sEnvMACAddress, "00:00:00:00:00:00") == 0)
	 {
		 DISABLE_PRINTF()

		 status = i2c_read(ADDR_EEPROM_READ, ADDR_IEEE_EUI_ID, 1, u8MAC, 8);

		 SESSION_DEPENDED_PRINTF_ENABLE()

	     if (status != 0)
	     {
	    	 /* -> if a error occurred during i2c read of unique ID */
	         printf ("\nERROR: Can't read ethernet MAC address from EEPROM\n");
	     }
	     else
		 {
	    	 /* -> if unique ID of rtc was read successful */
			 sprintf (&cMACAddress[0], "%02x:%02x:%02x:%02x:%02x:%02x", (unsigned int)u8MAC[2], (unsigned int)u8MAC[3], (unsigned int)u8MAC[4], (unsigned int)u8MAC[5], (unsigned int)u8MAC[6], (unsigned int)u8MAC[7]);
			 setenv("ethaddr", &cMACAddress[0]);

			 DISABLE_PRINTF()

			 saveenv(); // save environment with new MAC to flash

			#ifdef CONFIG_ENV_OFFSET_REDUND
			 saveenv(); // save new environment to redundant environment too
			#endif

			 SESSION_DEPENDED_PRINTF_ENABLE()

			 /* display that operation was successful */
			 printf("MAC:   reading unique ID from RTC was successful!\n");
		 }
	}

	// display MAC address
	sEnvMACAddress = getenv("ethaddr");
	printf("MAC:   %s\n", sEnvMACAddress); // display MAC


	/* ----------------------------------------------------------------------------- */
	/* Perform network card initialization                                           */

	#ifdef CONFIG_DRIVER_TI_EMAC
	if (getenv ("ethaddr"))
	{
		uchar enetaddr[6];
		eth_getenv_enetaddr("ethaddr", enetaddr);
		davinci_eth_set_mac_addr(enetaddr);
	}
	#endif

	#if defined(CONFIG_DRIVER_SMC91111) || defined (CONFIG_DRIVER_LAN91C96)
	if (getenv ("ethaddr"))
	{
		uchar enetaddr[6];
		eth_getenv_enetaddr("ethaddr", enetaddr);
		smc_set_mac_addr(enetaddr);
	}
	#endif /* CONFIG_DRIVER_SMC91111 || CONFIG_DRIVER_LAN91C96 */

	/* ----------------------------------------------------------------------------- */
	/* DSS initialization and loading splash-image                                   */
	#ifdef CONFIG_DRIVER_DSS

	/* Request and Activate Backlight Power Supply */
	omap_request_gpio(dh_board_settings_info.ul_backlight_en_gpio);
	omap_set_gpio_direction(dh_board_settings_info.ul_backlight_en_gpio,0);
	omap_set_gpio_dataout(dh_board_settings_info.ul_backlight_en_gpio,1);

	/* Request and Deactivate Panel Backlight PWM Pin */
	omap_request_gpio(dh_board_settings_info.ul_backlight_pwm);
	omap_set_gpio_direction(dh_board_settings_info.ul_backlight_pwm,0);
	omap_set_gpio_dataout(dh_board_settings_info.ul_backlight_pwm,1);

	/* Initialize the Display System */
	omapdss_init(&lcd);

	/* Activate the Backlight PWM Pin */
	omap_set_gpio_dataout(dh_board_settings_info.ul_backlight_pwm,0);
	#endif


    // Set pwm_pol Environemt Variable
    if(gd->dh_board_settings.wLCDConfigFlags & SETTINGS_LCD_PWM_POL_FLAG)
    {
        setenv("pwm_pol", "inverted");
    }
    else
    {
        setenv("pwm_pol", "normal");
    }

	return 0;
}
#endif

/*
 * Routine: misc_init_r
 * Description: Init ethernet (done here so udelay works)
 */
int misc_init_r(void)
{
	volatile unsigned int ctr;
	u32 reset;

	/*
	#ifdef CONFIG_DRIVER_OMAP34XX_I2C
	i2c_init(CONFIG_SYS_I2C_SPEED, CONFIG_SYS_I2C_SLAVE);
	#endif
	 */ /* already done in board_init */

	dieid_num_r();

#if defined(CONFIG_DRIVER_TI_EMAC)

	omap_request_gpio(30);
	omap_set_gpio_direction(30,0);
	omap_set_gpio_dataout(30,0);
	ctr  = 0;
	do
	{
		udelay(1000); /* 1 ms*/
		ctr++;
	}while (ctr <30);  /* LZ: changed from 300 ms to 30 ms*/
	omap_set_gpio_dataout(30,1);
	ctr =0;
	/* allow the PHY to stabilize and settle down */
	do
	{
		udelay(1000); /* 1 ms*/
		ctr++;
	}while (ctr <30); /* LZ: changed from 300 ms to 30 ms*/

	/*ensure that the module is out of reset*/
	reset = readl(AM3517_IP_SW_RESET);
	reset &= (~CPGMACSS_SW_RST);
	writel(reset,AM3517_IP_SW_RESET);
#endif

	return 0;
}


/*
 * Initializes on-chip ethernet controllers.
 * to override, implement board_eth_init()
 */
int cpu_eth_init(bd_t *bis)
{
#if defined(CONFIG_DRIVER_TI_EMAC)
	//printf("davinci_emac_initialize\n");
	davinci_emac_initialize();
#endif
	return 0;
 }

/*
 * Set Write protect pin
 */
void spi_io_set_WP_pin(void)
{
    omap_set_gpio_direction(145,0);
    omap_set_gpio_dataout(145,1);
}

/*
 * Set Write protect pin
 */
void spi_io_clear_WP_pin(void)
{
    omap_set_gpio_direction(145,0);
    omap_set_gpio_dataout(145,0);
}


/*
 * Routine: set_muxconf_regs
 * Description: Setting up the configuration Mux registers specific to the
 *		hardware. Many pins need to be moved from protect to primary
 *		mode.
 */
void set_muxconf_regs(void)
{
	MUX_AM3517(); /* defined in dh_am3517.h */
}
