/*
 * 2011 (c) DH electronics GmbH
 * Author: Andreas Geisreiter <ageisreiter at dh-electronics.de>
 *
 */

#include <common.h>
#include <command.h>
#include <linux/ctype.h>
#include <asm/io.h>
#include <dh_settings.h>

#define NAND_FLASH_PAGE_SIZE 2048

/*
 * global variables
 */
char cBmpSDRAMAddress[9]                = {UPDATE_BMP_SDRAM_ADDRESS};

char cBmpEndSmallResSDRAMAddress[9]     = {UPDATE_BMP_END_SDRAM_ADDRESS_SMALL_RES};
char cBmpErrorSmallResSDRAMAddress[9]   = {UPDATE_BMP_ERROR_SDRAM_ADDRESS_SMALL_RES};
char cBmpEndMiddleResSDRAMAddress[9]    = {UPDATE_BMP_END_SDRAM_ADDRESS_MIDDLE_RES};
char cBmpErrorMiddleResSDRAMAddress[9]  = {UPDATE_BMP_ERROR_SDRAM_ADDRESS_MIDDLE_RES};
char cBmpSDRAMStartAddress[9]           = {UPDATE_BMP_START_SDRAM_ADDRESS};
char cBmpEndSDRAMAddress[9]             = {000000000};
char cBmpErrorSDRAMAddress[9]           = {000000000};


unsigned long ulSDRAMBufferAddress      = 0;

 char *p_cDisplayBmpOnScreen[3]          = {"bmp","display",cBmpSDRAMStartAddress};


char cArgumentsSDRAMAddress[9]          = {UPDATE_ARGUMENTS_SDRAM_ADDRESS};
char cSDRAMBufferAddress[9]             = {UPDATE_SDRAM_BUFFER_ADDRESS};
char cOSImageFlashAddress[9]            = {UPDATE_OS_IMAGE_FLASH_ADDRESS};
char cBootloaderFlashAddress[9]         = {UPDATE_BOOTLOADER_FLASH_ADDRESS};
char cSplashimageFlashAddress[9]        = {UPDATE_SPLASHIMAGE_FLASH_ADDRESS};
char cSettingsBinFlashAddress[9]        = {UPDATE_SETTINGS_FLASH_ADDRESS};
char cArgumentsBinFlashAddress[9]       = {UPDATE_ARGUMENTS_FLASH_ADDRESS};

unsigned long ulNandBlockSize;

DECLARE_GLOBAL_DATA_PTR;

#if defined(CONFIG_CMD_AM35XX_UPDATE)

/* ===============================================================================================================
 * Command line function to update the NAND Flash flash content from MMC/SD - Card. This function is called after 
 * the start of the bootloader with the env variable "bootcmd". The function doesn't need bootline and other parameters. 
 * But two files are needed on the MMC/SD - Card:
 *   1. am35xx_update.bmp
 *   2. nk.bin (Windows Embedded CE 6.0 Image), nk.gz, splash.bmp, settings.bin, ...
 * The am35xx_update.bmp file introduce the update process. The file must have the following layout:
 *
 *   Address:     Hex:                                              ASCII:              Description:
 *   --------------------------------------------------------------------------------------------------------------
 *   0x0000_0000  69 4D 58 32 35 5F 75 70 64 61 74 65 2E 62 6D 70   am35x_update.bmp    Header
 *   0x0000_0010  00 44 48 20 45 6C 65 63 74 72 6F 6E 69 63 73 00   .DH Electronics.
 *   0x0000_0020  31 3D 55 70 64 61 74 65 41 72 67 75 6D 65 6E 74   1=UpdateArgument
 *   0x0000_0030  31 3D 53 69 67 6E 61 6C 47 50 49 4F 00 00 00 00   1=SignalGPIO		
 *   0x0000_0040  38 30 30 78 34 38 30 ?? free ??                   800x480				Display-Aufloesung
 * a) Resolution 1 (640x480 or 800x480)
 * ------------------------------------
 *   0x0000_0050  42 4D 36 ?? Bitmap ??                             BM6                 Bitmap Update in progress
 *   ...
 *   0x0006_4050  42 4D 36 ?? Bitmap ??                             BM6                 Bitmap Update end
 *   ...
 *   0x000C_8050  42 4D 36 ?? Bitmap ??                             BM6                 Bitmap Update error
 *   ...
 * b) Resolution 2 (800x600)
 * -------------------------
 *   0x0000_0050  42 4D 36 ?? Bitmap ??                             BM6                 Bitmap Update in progress
 *   ...
 *   0x0007_D050  42 4D 36 ?? Bitmap ??                             BM6                 Bitmap Update end
 *   ...
 *   0x000F_A050  42 4D 36 ?? Bitmap ??                             BM6                 Bitmap Update error
 *   ...
 *
 * Possible Argument Flags: 
 * Argument = '1' ==> Update only WinCE Flash Image (nk.bin or nk.gz)
 * Argument = '2' ==> Update only Linux Flash Image
 * Argument = '3' ==> Update complete Flash Image
 * Argument = '4' ==> Update Bootloader Flash Image
 * Argument = '5' ==> Update splash.bmp (Start Screen Picture)
 * Argument = '6' ==> Update settings.bin file
 * Argument = '7' ==> Update OS boot Argumente
 */
 
extern int do_mmc (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[]);
extern int do_fat_fsload (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[]);
extern int do_bmp (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[]);
extern int do_nand(cmd_tbl_t * cmdtp, int flag, int argc, char *argv[]);
// extern int do_mmu(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[]);
extern int do_mem_mw (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[]);

/*------------------------------------------------------------------------------
 *
 *  Function:  update_nand_flash
 *
 *  Update NAND Flash content on the specified address.
 *
 *  Commit values:  - ulNANDFlashAddress    = NAND FLash Block address
 *                  - ulSDRAMBufferAddress  = SDRAM Buffer address
 *                  - ulBlocks              = Number of block that should be updated
 *                  - ulNandBlockSize       = NAND Flash block size
 *
 *  Return value:   0 = No error
 *                  1 = NAND Flash erase error (the error block number is specified from bit 3 to bit 31)
 *                  2 = NAND FLash write error (the error block number is specified from bit 3 to bit 31)
 */
int update_nand_flash (unsigned long ulNANDFlashAddress, unsigned long ulSDRAMBufferAddress, unsigned long ulBlocks, unsigned long ulNandBlockSize)
{
    char *p_cEraseNANDFlashBlock[4]     = {"nand", "erase", "12345671", "12345672"};  				/* address values are place holders */
    char *p_cWriteNANDFlashBlock[5]     = {"nand", "write", "12345673", "12345674", "12345675"};  	/* address values are place holders */
    //char *mmu_on[2]                     = {"mmu","on"};
    //char *mmu_off[2]                    = {"mmu","off"};
    
    unsigned long i,j;
    int ret_value = 0;
    
    /* Set NAND FLash block size / length */
    sprintf (p_cEraseNANDFlashBlock[3], "%08x", (unsigned int)ulNandBlockSize);
    sprintf (p_cWriteNANDFlashBlock[4], "%08x", (unsigned int)ulNandBlockSize);     
    
    // Enable MMU
    // ToDo: do_mmu(NULL, 0, 2, mmu_on);
    
    /*  */
    for(j = 1, i = 0; i < ulBlocks; j++, i++)
    {
        if(j == 51)
        {
            printf ("\n                          ");
            j = 1;
        }
        
        /* Set the start address of the nand-flash area */
        sprintf (p_cEraseNANDFlashBlock[2], "%08x", (unsigned int)(ulNANDFlashAddress + i * ulNandBlockSize));
        sprintf (p_cWriteNANDFlashBlock[3], "%08x", (unsigned int)(ulNANDFlashAddress + i * ulNandBlockSize));
        
        /* Erase nand flash block */
        printf ("e");
        
        disable_printf;
        ret_value = do_nand(NULL, 0, 4, p_cEraseNANDFlashBlock); /* execute nand erase operation */
        if(dh_board_settings_info.ul_disable_printf == 0) {	enable_printf; }
        
        /* Exit if an erase error occured */
        if(ret_value != 0)
        {
            // ToDo: do_mmu(NULL, 0, 2, mmu_off);
            ret_value = (i << 2) | 1;
            return ret_value;  
        }

        /* Set sdram buffer address */
        sprintf (p_cWriteNANDFlashBlock[2], "%08x", (unsigned int)(ulSDRAMBufferAddress + i * ulNandBlockSize));
        
        /* Write nand flash block */
        printf ("\bw");
        disable_printf;
        ret_value = do_nand(NULL, 0, 5, p_cWriteNANDFlashBlock);
        
        if(dh_board_settings_info.ul_disable_printf == 0) { enable_printf; }

        /* Exit if a write error occured */
        if(ret_value != 0)
        {
            // ToDo: do_mmu(NULL, 0, 2, mmu_off);
            ret_value = (i << 2) | 2;
            return ret_value;    
        }
        
        printf ("\bd");
    }
    // Disable MMU
    // ToDo: do_mmu(NULL, 0, 2, mmu_off);
    
    printf ("]\n");      
    return 0;
}


/*------------------------------------------------------------------------------
 * Function: 	wince_img_update
 */
int wince_img_update(int argc, char *argv[], unsigned char FileType)
{
	int ret_value = 0;
    unsigned long ulFilesize;
    unsigned long ulBlocks;

    char *p_cLoadNkBinToSDRAM[5]            = {"fatload","mmc","0",cSDRAMBufferAddress,"nk.bin"};
    char *p_cLoadNkGzToSDRAM[5]             = {"fatload","mmc","0",cSDRAMBufferAddress,"nk.gz"};

    char cImageFileSize[9]                  = {"12345677\0"};

    unsigned long ulOSImageFlashAddress     = simple_strtoul (cOSImageFlashAddress, NULL, 16);

	printf("\n==> Update: Start to Update WinCE 6.0 Flash Image");

	// Load nk.bin or nk.gz file from MMC/SD - Card to SDRAM.
	if ((argc < 3) || (FileType == WINCE_FILE_TYPE_BIN))
	{
		// Try to load nk.bin or command line specified *.bin file
		disable_printf;
		ret_value = do_fat_fsload(NULL, 0, 5, p_cLoadNkBinToSDRAM);

		if (dh_board_settings_info.ul_disable_printf == 0) { enable_printf; }
	}
	else
	{
		// Can't load *.bin file or File Type is WINCE_FILE_TYPE_GZ
		ret_value = 1;
	}

	if (ret_value == 1) {
		if ((argc < 3) || (FileType == WINCE_FILE_TYPE_GZ)) {
			// Try to load nk.gz or command line specified *.gz file
			disable_printf;
			ret_value = do_fat_fsload(NULL, 0, 5, p_cLoadNkGzToSDRAM);
			if (dh_board_settings_info.ul_disable_printf == 0) {
				enable_printf;
			}
		} else {
			// Can't load *.gz file
			ret_value = 1;
		}

		if (ret_value == 0) {
			printf("\n--> Update: Load %s to SDRAM (%d bytes)",
					(char*) p_cLoadNkGzToSDRAM[4], (int) simple_strtoul(getenv(
							"filesize"), NULL, 16));
			// Save NAND Flash Image Type for do_bootwince command
			disable_printf;
			setenv("winceimagetype", "nk.gz");
			saveenv();
			if (dh_board_settings_info.ul_disable_printf == 0) {
				enable_printf;
			}
		}
	} else {
		printf("\n--> Update: Load %s to SDRAM (%d bytes)",
				(char*) p_cLoadNkBinToSDRAM[4], (int) simple_strtoul(getenv(
						"filesize"), NULL, 16));
		// Save NAND Flash Image Type for do_bootwince command
		disable_printf;
		setenv("winceimagetype", "nk.bin");
		saveenv();
		if (dh_board_settings_info.ul_disable_printf == 0) {
			enable_printf;
		}
	}

	if (ret_value == 0) {
		// Calculate necessary sectors in flash for the nk.bin file.
		ulFilesize = simple_strtoul(getenv("filesize"), NULL, 16);
		ulBlocks = ulFilesize / ulNandBlockSize + 0x1;

		printf(
				"\n--> Update: The new WinCE image File needs %lu Flash blocks\n",
				ulBlocks);
		printf("    e = Erase Block\n");
		printf("    w = Write to Block\n");
		printf("    d = Done\n");
		printf("    Write File to Flash:[");

		ret_value = update_nand_flash(ulOSImageFlashAddress, ulSDRAMBufferAddress, ulBlocks, ulNandBlockSize);

		if ((ret_value & 0x3) == 1) {
			printf("\n--> Update ERROR: Erase error on block 0x%08x\n",
					(unsigned int) (ulOSImageFlashAddress + (ret_value >> 2)
							* ulNandBlockSize));
			p_cDisplayBmpOnScreen[2] = cBmpErrorSDRAMAddress;
			if (get_update_via_bmpfile() == 0)
			{
				do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);

				// Don't Start OS after Update error
				set_stop_after_update();
			}
			// Save NAND Flash Image Size for do_bootwince command
			disable_printf;
			setenv("winceimagesize", "00000000");
			setenv("winceimagetype", "no_wince");
			saveenv();
			if (dh_board_settings_info.ul_disable_printf == 0) {
				enable_printf;
			}
			return 1;
		} else if ((ret_value & 0x3) == 2) {
			printf("\n--> Update ERROR: Write error on block 0x%08x\n",
					(unsigned int) (ulOSImageFlashAddress + (ret_value >> 2)
							* ulNandBlockSize));
			p_cDisplayBmpOnScreen[2] = cBmpErrorSDRAMAddress;
			if (get_update_via_bmpfile() == 0) {
				do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
				// Don't Start OS after Update error
				set_stop_after_update();
			}
			// Save NAND Flash Image Size for do_bootwince command
			disable_printf;
			setenv("winceimagesize", "00000000");
			setenv("winceimagetype", "no_wince");
			saveenv();
			if (dh_board_settings_info.ul_disable_printf == 0) {
				enable_printf;
			}
			return 1;
		}

		// Windows Embedded CE 6.0 Flash Update done.
		printf("\n--> Update: WinCE 6.0 Update done");
		p_cDisplayBmpOnScreen[2] = cBmpEndSDRAMAddress;
		if (get_update_via_bmpfile() == 0) {
			do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
		}

		// Save NAND Flash Image Size for do_bootwince command
		disable_printf;
		sprintf(&cImageFileSize[0], "%08x", (unsigned int) ulFilesize);
		setenv("winceimagesize", &cImageFileSize[0]);
		if (dh_board_settings_info.ul_disable_printf == 0) {
			enable_printf;
		}

		// Setup U-Boot ENV for WinCE
		printf("\n--> Update: Setup ENV for WinCE\n");
		setenv("loadaddr", "00200000");
		setenv("bootcmd", "run nandbootwince");
		disable_printf;
		saveenv();
		if (dh_board_settings_info.ul_disable_printf == 0) {
			enable_printf;
		}
	} else {
		if (argc < 3) {
			// Can't load nk.bin or nk.gz file from MMC/SD - Card to SDRAM.
			printf(
					"\n--> Update ERROR: No nk.bin or nk.gz found on MMC/SD-Card\n");
			p_cDisplayBmpOnScreen[2] = cBmpErrorSDRAMAddress;
			if (get_update_via_bmpfile() == 0) {
				do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
				// Don't Start OS after Update error
				set_stop_after_update();
			}
			return 1;
		} else {
			if (FileType == WINCE_FILE_TYPE_GZ) {
				// Can't load *.gz file from MMC/SD - Card to SDRAM.
				printf("\n--> Update ERROR: No %s found on MMC/SD-Card\n",
						(char*) p_cLoadNkGzToSDRAM[4]);
				return 1;
			} else {
				// Can't load *.bin file from MMC/SD - Card to SDRAM.
				printf("\n--> Update ERROR: No %s found on MMC/SD-Card\n",
						(char*) p_cLoadNkBinToSDRAM[4]);
				return 1;
			}
		}
	}
	return 0;
}


/*------------------------------------------------------------------------------
 * Function: 	linux_img_update
 */
int linux_img_update(int argc, char *argv[])
{
	int ret_value = 0;
    unsigned long ulFilesize;
    unsigned long ulBlocks;

    char *p_cENVCompareString[1]            = {"Set_ENV"};
    char *p_cSDRAMBufferAddress             = (char*)ulSDRAMBufferAddress;
    char *p_cLoaduImageToSDRAM[5]           = {"fatload","mmc","1",cSDRAMBufferAddress,"uImage"};
    char *p_cLoaduImageENVToSDRAM[5]        = {"fatload","mmc","1",cSDRAMBufferAddress,"uImage.env"};
    char *p_cRunLinuxENVScript[2]           = {"source",cSDRAMBufferAddress};

    unsigned long ulOSImageFlashAddress     = simple_strtoul (cOSImageFlashAddress, NULL, 16);

	printf("\n==> Update: Start to Update Linux Flash Image");
	// Load Linux image file from MMC/SD - Card to SDRAM.
	// Try to load Linux image
	disable_printf;
	ret_value = do_fat_fsload(NULL, 0, 5, p_cLoaduImageToSDRAM);
	if (dh_board_settings_info.ul_disable_printf == 0) {
		enable_printf;
	}
	if (ret_value == 0) {
		// Get linux image size and save it to ENV
		ulFilesize = simple_strtoul(getenv("filesize"), NULL, 16);
		setenv("linuxfilesize", getenv("filesize"));
		disable_printf;
		saveenv();
		if (dh_board_settings_info.ul_disable_printf == 0) {
			enable_printf;
		}

		printf("\n--> Update: Load %s to SDRAM (%d bytes)",
				(char*) p_cLoaduImageToSDRAM[4], ulFilesize);

		// Calculate necessary sectors in flash for the Linux image file.
		ulBlocks = ulFilesize / ulNandBlockSize + 0x1;

		printf(
				"\n--> Update: The new Linux image File needs %lu Flash blocks\n",
				ulBlocks);
		printf("    e = Erase Block\n");
		printf("    w = Write to Block\n");
		printf("    d = Done\n");
		printf("    Write File to Flash:[");

		ret_value = update_nand_flash(ulOSImageFlashAddress,
				ulSDRAMBufferAddress, ulBlocks, ulNandBlockSize);

		if ((ret_value & 0x3) == 1) {
			printf("\n--> Update ERROR: Erase error on block 0x%08x\n",
					(unsigned int) (ulOSImageFlashAddress + (ret_value >> 2)
							* ulNandBlockSize));
			p_cDisplayBmpOnScreen[2] = cBmpErrorSDRAMAddress;
			if (get_update_via_bmpfile() == 0) {
				do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
				// Don't Start OS after Update error
				set_stop_after_update();
			}
			return 1;
		} else if ((ret_value & 0x3) == 2) {
			printf("\n--> Update ERROR: Write error on block 0x%08x\n",
					(unsigned int) (ulOSImageFlashAddress + (ret_value >> 2)
							* ulNandBlockSize));
			p_cDisplayBmpOnScreen[2] = cBmpErrorSDRAMAddress;
			if (get_update_via_bmpfile() == 0) {
				do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
				// Don't Start OS after Update error
				set_stop_after_update();
			}
			return 1;
		}

		// Try to load linux ENV setup file
		disable_printf;
		ret_value = do_fat_fsload(NULL, 0, 5, p_cLoaduImageENVToSDRAM);
		if (dh_board_settings_info.ul_disable_printf == 0) {
			enable_printf;
		}

		if (ret_value == 0) {
			printf("\n--> Update: Load ENV setup file %s to SDRAM (%d bytes)",
					(char*) p_cLoaduImageENVToSDRAM[4], (int) simple_strtoul(
							getenv("filesize"), NULL, 16));

			// Setup U-Boot ENV for Linux
			printf("\n--> Update: Setup ENV for Linux");

			// Check if *.env file contains the script name "Set_ENV"
			if (!(memcmp((char*) (p_cSDRAMBufferAddress + 0x20), p_cENVCompareString[0], 7))) {
				printf(
						"\n\n----------------- Print ENV Script Info: -----------------\n");
				ret_value = do_source(NULL, 0, 2, p_cRunLinuxENVScript);
				printf(
						"\n----------------------------------------------------------\n");
			} else {
				printf(
						"\n--> Update ERROR: No valid %s file found on MMC/SD-Card\n",
						(char*) p_cLoaduImageENVToSDRAM[4]);
				p_cDisplayBmpOnScreen[2] = cBmpErrorSDRAMAddress;
				if (get_update_via_bmpfile() == 0) {
					do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
					// Don't Start OS after Update error
					set_stop_after_update();
				}
				return 1;
			}

			disable_printf;
			saveenv();
			if (dh_board_settings_info.ul_disable_printf == 0) {
				enable_printf;
			}
		} else {
			printf(
					"\n--> Update INFO: No ENV setup file %s found on MMC/SD-Card",
					(char*) p_cLoaduImageENVToSDRAM[4]);
		}

		// Linux Flash Update done.
		printf("\n--> Update: Linux Update done\n");
		p_cDisplayBmpOnScreen[2] = cBmpEndSDRAMAddress;
		if (get_update_via_bmpfile() == 0) {
			do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
		}
	} else {
		// Can't load Linux image file from MMC/SD - Card to SDRAM.
		printf("\n--> Update ERROR: No %s file found on MMC/SD-Card\n",
				(char*) p_cLoaduImageToSDRAM[4]);
		p_cDisplayBmpOnScreen[2] = cBmpErrorSDRAMAddress;
		if (get_update_via_bmpfile() == 0) {
			do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
			// Don't Start OS after Update error
			set_stop_after_update();
		}
		return 1;
	}
	return 0;
}


/*------------------------------------------------------------------------------
 * Function: 	bootloader_update
 */
int bootloader_update(int argc, char *argv[])
{
	int ret_value = 0;
    unsigned long ulFilesize;
    unsigned long ulBlocks;

    char *p_cLoadUBootBinToSDRAM[5]         = {"fatload","mmc","0",cSDRAMBufferAddress,"u-boot.bin"};

    unsigned long ulBootloaderFlashAddress  = simple_strtoul (cBootloaderFlashAddress, NULL, 16);

    printf ("\n==> Update: Start to Update Bootloader Flash Image");
    // Load u-boot image file  from MMC/SD - Card to SDRAM.
    disable_printf;
    ret_value = do_fat_fsload(NULL, 0, 5, p_cLoadUBootBinToSDRAM);
    if(dh_board_settings_info.ul_disable_printf == 0)
    {
        enable_printf;
    }

    if(ret_value == 0)
    {
        printf ("\n--> Update: Load %s to SDRAM (%d bytes)", (char*)p_cLoadUBootBinToSDRAM[4], (int)simple_strtoul (getenv("filesize"), NULL, 16));

        // Calculate necessary sectors in flash for the u-boot image file.
        ulFilesize = simple_strtoul (getenv("filesize"), NULL, 16);
        ulBlocks = ulFilesize / ulNandBlockSize + 0x1;

        printf ("\n--> Update: The new u-boot image File needs %lu Flash blocks\n", ulBlocks);
        printf ("    e = Erase Block\n");
        printf ("    w = Write to Block\n");
        printf ("    d = Done\n");
        printf ("    Write File to Flash:[");

        ret_value = update_nand_flash (ulBootloaderFlashAddress, ulSDRAMBufferAddress, ulBlocks, ulNandBlockSize);

        if((ret_value & 0x3) == 1)
        {
            printf ("\n--> Update ERROR: Erase error on block 0x%08x\n", (unsigned int)(ulBootloaderFlashAddress + (ret_value >> 2) * ulNandBlockSize));
            p_cDisplayBmpOnScreen[2] = cBmpErrorSDRAMAddress;
            if(get_update_via_bmpfile() == 0)
            {
                do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
                // Don't Start OS after Update error
                set_stop_after_update();
            }
            return 1;
        }
        else if((ret_value & 0x3) == 2)
        {
            printf ("\n--> Update ERROR: Write error on block 0x%08x\n", (unsigned int)(ulBootloaderFlashAddress + (ret_value >> 2) * ulNandBlockSize));
            p_cDisplayBmpOnScreen[2] = cBmpErrorSDRAMAddress;
            if(get_update_via_bmpfile() == 0)
            {
                do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
                // Don't Start OS after Update error
                set_stop_after_update();
            }
            return 1;
        }

        // Bootloader Flash Update done.
        printf ("\n--> Update: U-Boot Update done\n");
        p_cDisplayBmpOnScreen[2] = cBmpEndSDRAMAddress;
        if(get_update_via_bmpfile() == 0)
        {
            do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
        }
    }
    else
    {
        // Can't load u-boot image file from MMC/SD - Card to SDRAM.
        printf ("\n--> Update ERROR: No %s file found on MMC/SD-Card\n", (char*)p_cLoadUBootBinToSDRAM[4]);
        p_cDisplayBmpOnScreen[2] = cBmpErrorSDRAMAddress;
        if(get_update_via_bmpfile() == 0)
        {
            do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
            // Don't Start OS after Update error
            set_stop_after_update();
        }
        return 1;
    }
	return 0;
}


/*------------------------------------------------------------------------------
 * Function: 	settings_bin_update
 */
int splashimage_update(int argc, char *argv[])
{
	int ret_value = 0;
    unsigned long ulFilesize;
    unsigned long ulBlocks;

    unsigned long ulSplashimageFlashAddress = simple_strtoul (cSplashimageFlashAddress, NULL, 16);

    char *p_cLoadSplashBmpToSDRAM[5]        = {"fatload","mmc","0",cSDRAMBufferAddress,"splash.bmp"};

    printf ("\n==> Update: Start to Update Splashimage Bitmap");
    // Load splash bitmap file from MMC/SD - Card to SDRAM.
    disable_printf;
    ret_value = do_fat_fsload(NULL, 0, 5, p_cLoadSplashBmpToSDRAM);
    if(dh_board_settings_info.ul_disable_printf == 0)
    {
        enable_printf;
    }

    if(ret_value == 0)
    {
        printf ("\n--> Update: Load %s to SDRAM (%d bytes)", (char*)p_cLoadSplashBmpToSDRAM[4], (int)simple_strtoul (getenv("filesize"), NULL, 16));

        // Calculate necessary sectors in flash for the splash bitmap file.
        ulFilesize = simple_strtoul (getenv("filesize"), NULL, 16);
        ulBlocks = ulFilesize / ulNandBlockSize + 0x1;

        printf ("\n--> Update: The new splash bitmap File needs %lu Flash blocks\n", ulBlocks);
        printf ("    e = Erase Block\n");
        printf ("    w = Write to Block\n");
        printf ("    d = Done\n");
        printf ("    Write File to Flash:[");

        ret_value = update_nand_flash (ulSplashimageFlashAddress, ulSDRAMBufferAddress, ulBlocks, ulNandBlockSize);

        if((ret_value & 0x3) == 1)
        {
            printf ("\n--> Update ERROR: Erase error on block 0x%08x\n", (unsigned int)(ulSplashimageFlashAddress + (ret_value >> 2) * ulNandBlockSize));
            p_cDisplayBmpOnScreen[2] = cBmpErrorSDRAMAddress;
            if(get_update_via_bmpfile() == 0)
            {
                do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
                // Don't Start OS after Update error
                set_stop_after_update();
            }
            return 1;
        }
        else if((ret_value & 0x3) == 2)
        {
            printf ("\n--> Update ERROR: Write error on block 0x%08x\n", (unsigned int)(ulSplashimageFlashAddress + (ret_value >> 2) * ulNandBlockSize));
            p_cDisplayBmpOnScreen[2] = cBmpErrorSDRAMAddress;
            if(get_update_via_bmpfile() == 0)
            {
                do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
                // Don't Start OS after Update error
                set_stop_after_update();
            }
            return 1;
        }

        // Splashimage Flash Update done.
        printf ("\n--> Update: Splashimage Update done\n");
        p_cDisplayBmpOnScreen[2] = cBmpEndSDRAMAddress;
        if(get_update_via_bmpfile() == 0)
        {
            do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
        }
    }
    else
    {
        // Can't load splash bitmap file from MMC/SD - Card to SDRAM.
        printf ("\n--> Update ERROR: No %s file found on MMC/SD-Card\n", (char*)p_cLoadSplashBmpToSDRAM[4]);
        p_cDisplayBmpOnScreen[2] = cBmpErrorSDRAMAddress;
        if(get_update_via_bmpfile() == 0)
        {
            do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
            // Don't Start OS after Update error
            set_stop_after_update();
        }
        return 1;
    }
	return 0;
}

/*------------------------------------------------------------------------------
 * Function: 	settings_bin_update
 */
int settings_bin_update(int argc, char *argv[])
{
	int ret_value = 0;
    unsigned long ulFilesize;
    unsigned long ulBlocks;

    unsigned long ulSettingsBinFlashAddress = simple_strtoul (cSettingsBinFlashAddress, NULL, 16);

    char *p_cLoadSettingsBinToSDRAM[5]      = {"fatload","mmc","0",cSDRAMBufferAddress,"settings.bin"};

    printf ("\n==> Update: Start to Update settings file");
   // Load settings file from MMC/SD - Card to SDRAM.
   disable_printf;
   ret_value = do_fat_fsload(NULL, 0, 5, p_cLoadSettingsBinToSDRAM);
   if(dh_board_settings_info.ul_disable_printf == 0)
   {
       enable_printf;
   }

   if(ret_value == 0)
   {
       printf ("\n--> Update: Load %s to SDRAM (%d bytes)", (char*)p_cLoadSettingsBinToSDRAM[4], (int)simple_strtoul (getenv("filesize"), NULL, 16));

       // Calculate necessary sectors in flash for the settings file.
       ulFilesize = simple_strtoul (getenv("filesize"), NULL, 16);
       ulBlocks = ulFilesize / ulNandBlockSize + 0x1;

       printf ("\n--> Update: The new settings File needs %lu Flash blocks\n", ulBlocks);
       printf ("    e = Erase Block\n");
       printf ("    w = Write to Block\n");
       printf ("    d = Done\n");
       printf ("    Write File to Flash:[");

       ret_value = update_nand_flash (ulSettingsBinFlashAddress, ulSDRAMBufferAddress, ulBlocks, ulNandBlockSize);

       if((ret_value & 0x3) == 1)
       {
           printf ("\n--> Update ERROR: Erase error on block 0x%08x\n", (unsigned int)(ulSettingsBinFlashAddress + (ret_value >> 2) * ulNandBlockSize));
           p_cDisplayBmpOnScreen[2] = cBmpErrorSDRAMAddress;
           if(get_update_via_bmpfile() == 0)
           {
               do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
               // Don't Start OS after Update error
               set_stop_after_update();
           }
           return 1;
       }
       else if((ret_value & 0x3) == 2)
       {
           printf ("\n--> Update ERROR: Write error on block 0x%08x\n", (unsigned int)(ulSettingsBinFlashAddress + (ret_value >> 2) * ulNandBlockSize));
           p_cDisplayBmpOnScreen[2] = cBmpErrorSDRAMAddress;
           if(get_update_via_bmpfile() == 0)
           {
               do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
               // Don't Start OS after Update error
               set_stop_after_update();
           }
           return 1;
       }

       // Settings Flash Update done.
       printf ("\n--> Update: Settings Update done\n");
       p_cDisplayBmpOnScreen[2] = cBmpEndSDRAMAddress;
       if(get_update_via_bmpfile() == 0)
       {
           do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
       }
   }
   else
   {
       // Can't load settings file from MMC/SD - Card to SDRAM.
       printf ("\n--> Update ERROR: No %s file found on MMC/SD-Card\n", (char*)p_cLoadSettingsBinToSDRAM[4]);
       p_cDisplayBmpOnScreen[2] = cBmpErrorSDRAMAddress;
       if(get_update_via_bmpfile() == 0)
       {
           do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
           // Don't Start OS after Update error
           set_stop_after_update();
       }
       return 1;
   }
	return 0;
}

/*------------------------------------------------------------------------------
 * Function: 	os_arguments_update
 */
int os_arguments_update(int argc, char *argv[])
{
	int ret_value = 0;
	unsigned long ulFilesize;
	unsigned long ulBlocks;

	unsigned long ulArgumentsBinFlashAddress = simple_strtoul(cArgumentsBinFlashAddress, NULL, 16);

	char *p_cLoadArgumentsBinToSDRAM[5] = { "fatload", "mmc", "0", cSDRAMBufferAddress, "arguments.bin" };

	printf("\n==> Update: Start to Update arguments.bin file");
	// Load arguments.bin file from MMC/SD - Card to SDRAM.
	disable_printf;
	ret_value = do_fat_fsload(NULL, 0, 5, p_cLoadArgumentsBinToSDRAM);
	if (dh_board_settings_info.ul_disable_printf == 0) {
		enable_printf;
	}

	if (ret_value == 0) {
		printf("\n--> Update: Load arguments.bin to SDRAM (%d bytes)",
				(int) simple_strtoul(getenv("filesize"), NULL, 16));

		// Calculate necessary sectors in flash for the arguments.bin file.
		ulFilesize = simple_strtoul(getenv("filesize"), NULL, 16);
		ulBlocks = ulFilesize / ulNandBlockSize + 0x1;

		printf("\n--> Update: The new arguments.bin File needs %lu Flash blocks\n", ulBlocks);
		printf("    e = Erase Block\n");
		printf("    w = Write to Block\n");
		printf("    d = Done\n");
		printf("    Write File to Flash:[");

		ret_value = update_nand_flash(ulArgumentsBinFlashAddress,
				ulSDRAMBufferAddress, ulBlocks, ulNandBlockSize);

		if ((ret_value & 0x3) == 1)
		{
			printf("\n--> Update ERROR: Erase error on block 0x%08x\n",
					(unsigned int) (ulArgumentsBinFlashAddress + (ret_value	>> 2) * ulNandBlockSize));

			p_cDisplayBmpOnScreen[2] = cBmpErrorSDRAMAddress;

			if (get_update_via_bmpfile() == 0)
			{
				do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
				// Don't Start OS after Update error
				set_stop_after_update();
			}
			return 1;
		}
		else if ((ret_value & 0x3) == 2)
		{
			printf("\n--> Update ERROR: Write error on block 0x%08x\n",
					(unsigned int) (ulArgumentsBinFlashAddress + (ret_value
							>> 2) * ulNandBlockSize));
			p_cDisplayBmpOnScreen[2] = cBmpErrorSDRAMAddress;
			if (get_update_via_bmpfile() == 0) {
				do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
				// Don't Start OS after Update error
				set_stop_after_update();
			}
			return 1;
		}

		// Arguments.bin Flash Update done.
		printf("\n--> Update: arguments.bin Update done\n");
		p_cDisplayBmpOnScreen[2] = cBmpEndSDRAMAddress;
		if (get_update_via_bmpfile() == 0) {
			do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
		}
	} else {
		// Can't load arguments.bin file from MMC/SD - Card to SDRAM.
		printf(
				"\n--> Update ERROR: No arguments.bin file found on MMC/SD-Card\n");
		p_cDisplayBmpOnScreen[2] = cBmpErrorSDRAMAddress;
		if (get_update_via_bmpfile() == 0) {
			do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
			// Don't Start OS after Update error
			set_stop_after_update();
		}
		return 1;
	}

	return 0;
}

/*------------------------------------------------------------------------------
 *
 *  Function:  do_am35xxupdate
 *
 *  Update command
 */
int do_am35xxupdate (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	unsigned char  UpdateModeArg = INVALID_MODE;
	unsigned char  FileTypeWinCE = INVALID_TYPE;

    ulSDRAMBufferAddress      				= simple_strtoul (cSDRAMBufferAddress, NULL, 16);

    void *p_vUpdateArgument;
    void *p_vCheckString;    
    char *p_cCheckString[1]                 = {"00000000000000000"};
    char *p_cCompareString[1]               = {"am35x_update.bmp\0"};
    char *p_cBmpCompareString[1]            = {"BM6\0"};
    char *p_cMMCCardInit[3]                 = {"mmc", "init", "0"};
    char *p_cLoadUpdateBmpToSDRAM[5]        = {"fatload","mmc","0",cBmpSDRAMAddress,"am35x_update.bmp"};


    char *p_cLoadNkBinToSDRAM[5]            = {"fatload","mmc","0",cSDRAMBufferAddress,"nk.bin"};
    char *p_cLoadNkGzToSDRAM[5]             = {"fatload","mmc","0",cSDRAMBufferAddress,"nk.gz"};
    
    char *p_cLoaduImageToSDRAM[5]           = {"fatload","mmc","1",cSDRAMBufferAddress,"uImage"};
    char *p_cLoaduImageENVToSDRAM[5]        = {"fatload","mmc","1",cSDRAMBufferAddress,"uImage.env"};

    char *p_cLoadUBootBinToSDRAM[5]         = {"fatload","mmc","1",cSDRAMBufferAddress,"u-boot.bin"};
    char *p_cLoadSplashBmpToSDRAM[5]        = {"fatload","mmc","1",cSDRAMBufferAddress,"splash.bmp"};
    char *p_cLoadSettingsBinToSDRAM[5]      = {"fatload","mmc","1",cSDRAMBufferAddress,"settings.bin"};


    char cuImageScriptFileName[50];

    int ret_value = 0;
    int iStringCharacters = 0;
    int i;
    char *cmd, *file_name, *image_type;

    unsigned long ulNandPagesPerBlock = 0;

    // Set Flag to update with "am35x_update.bmp" file
    set_update_via_bmpfile();
    
    // Determine NAND Flash pages per block
    switch((readw(0xBB001E1A) >> 9) & 0x3)
    {
        case 0:
            ulNandPagesPerBlock = 32;
        break;
        case 1:
            ulNandPagesPerBlock = 64;        
        break;
        case 2:
            ulNandPagesPerBlock = 128;       
        break;
        case 3:
            ulNandPagesPerBlock = 256;       
        break;
        default:
            printf ("\n--> Error: Can't determine NAND Flash pages per block\n");
    }
    
    ulNandBlockSize = (NAND_FLASH_PAGE_SIZE * ulNandPagesPerBlock);
    
    // Initialize MMC/SD -Card.
    printf ("\n--> Try to initialize MMC/SD - Card:\n");
    ret_value = do_mmc(NULL, 0, 3, p_cMMCCardInit);

    // Can't Initialize MMC/SD -Card.
    if(ret_value != 0)
    {
        printf ("\n--> No MMC/SD - Card detected! Try to start Operating System\n");
        return 1;
    }


    /* Load am35x_update.bmp from MMC/SD - Card to SDRAM */
  	disable_printf;

    ret_value = do_fat_fsload(NULL, 0, 5, p_cLoadUpdateBmpToSDRAM);

    if(dh_board_settings_info.ul_disable_printf == 0) {	enable_printf; }

    /* The am35x_update.bmp is not a valid update file */
    if(ret_value != 0)
    {
        printf ("\n--> No valid bitmap am35x_update.bmp found on MMC/SD-Card\n");
        return 1;
    }

    // Check if am35x_update.bmp is valid. There must be the string "am35x_update.bmp" on the beginning of the bmp File.
    p_vCheckString = (void*)simple_strtoul (cBmpSDRAMAddress, NULL, 16);
        
    /* count string characters */
    for(iStringCharacters = 0; *(char*)(p_vCheckString + iStringCharacters) != '\0'; iStringCharacters++);

    iStringCharacters++;

    // Update without am35x_update.bmp file
    if (argc > 1)
    {
		cmd = argv[1];

		if (strcmp(cmd, "wince") == 0)
		{
			UpdateModeArg = WINCE_IMAGE_UPDATE;
			p_vUpdateArgument = &UpdateModeArg;
			set_update_via_console() ;
			// Load File name from command line
			if (argc > 2) {
				file_name = argv[2];

				// Search for image type (*.gz or *.bin)
				for (i = 0; *(file_name + i) != '.'; i++)
					;

				image_type = file_name + i + 1;

				if (strcmp(image_type, "gz") == 0) {
					FileTypeWinCE = WINCE_FILE_TYPE_GZ;
					p_cLoadNkGzToSDRAM[4] = file_name;
				} else if (strcmp(image_type, "bin") == 0) {
					FileTypeWinCE = WINCE_FILE_TYPE_BIN;
					p_cLoadNkBinToSDRAM[4] = file_name;
				} else {
					printf(
							"\n--> Update ERROR: Wrong WinCE image type (Filename must end with .gz or .bin) \n");
					return 1;
				}
			}
		}
		else if (strcmp(cmd, "linux") == 0)
		{
			UpdateModeArg = LINUX_IMAGE_UPDATE;
			p_vUpdateArgument = &UpdateModeArg;
			set_update_via_console() ;
			// Load File name from command line
			if (argc > 2) {
				file_name = argv[2];
				p_cLoaduImageToSDRAM[4] = file_name;

				// Set ENV Script File name
				for (i = 0; (i < 50) && (*(file_name + i) != '.')
						&& (*(file_name + i) != '\0'); i++) {
					cuImageScriptFileName[i] = *(file_name + i);
				}
				cuImageScriptFileName[i] = '.';
				cuImageScriptFileName[i + 1] = 'e';
				cuImageScriptFileName[i + 2] = 'n';
				cuImageScriptFileName[i + 3] = 'v';
				cuImageScriptFileName[i + 4] = '\0';

				p_cLoaduImageENVToSDRAM[4] = &cuImageScriptFileName[0];
			}
		}
		else if (strcmp(cmd, "flash") == 0)
		{
			UpdateModeArg = COMPLETE_FLASH_UPDATE;
			p_vUpdateArgument = &UpdateModeArg;
			set_update_via_console() ;
		}
		else if (strcmp(cmd, "bootloader") == 0)
		{
			UpdateModeArg = BOOTLOADER_FLASH_UPDATE;
			p_vUpdateArgument = &UpdateModeArg;
			set_update_via_console() ;

			// Load File name from command line
			if (argc > 2)
			{
				file_name = argv[2];
				p_cLoadUBootBinToSDRAM[4] = file_name;
			}
		}
		else if (strcmp(cmd, "splash") == 0)
		{
			UpdateModeArg = SPLASHIMAGE_UPDATE;
			p_vUpdateArgument = &UpdateModeArg;
			set_update_via_console() ;

			// Load File name from command line
			if (argc > 2)
			{
				file_name = argv[2];
				p_cLoadSplashBmpToSDRAM[4] = file_name;
			}
		}
		else if (strcmp(cmd, "settings") == 0)
		{
			UpdateModeArg = SETTINGS_BIN_UPDATE;
			p_vUpdateArgument = &UpdateModeArg;
			set_update_via_console() ;

			// Load File name from command line
			if (argc > 2)
			{
				file_name = argv[2];
				p_cLoadSettingsBinToSDRAM[4] = file_name;
			}
		}
		else
		{
			// Wrong argument
			set_update_via_bmpfile();
		}
	}

	if (get_update_via_bmpfile() == 0)
	{
		// Load am35x_update.bmp from MMC/SD - Card to SDRAM.
		disable_printf;
		ret_value = do_fat_fsload(NULL, 0, 5, p_cLoadUpdateBmpToSDRAM);
		if (dh_board_settings_info.ul_disable_printf == 0) {
			enable_printf;
		}
	}

	if ((ret_value == 0) || (get_update_via_bmpfile() != 0)) {
		if (get_update_via_bmpfile() == 0) {
			// Check if am35x_update.bmp is valid. There must be the string "am35x_update.bmp" on the beginning
			// of the bmp File.
			p_vCheckString = (void*) simple_strtoul(cBmpSDRAMAddress, NULL, 16);
			for (iStringCharacters = 0; *(char*) (p_vCheckString
					+ iStringCharacters) != '\0'; iStringCharacters++)
				;
			iStringCharacters++;
			memcpy(p_cCheckString[0], p_vCheckString, iStringCharacters);
		}
	}





    memcpy ( p_cCheckString[0], p_vCheckString, iStringCharacters);
      
    ret_value = memcmp ( p_cCheckString[0], p_cCompareString[0], iStringCharacters);

    if(	ret_value )
    {
        printf ("\n--> am35x_update.bmp is corrupted\n");
        return 1;
    }

	if (get_update_via_bmpfile() == 0)
	{
		printf(	"\n--> Update: am35x_update.bmp (%d bytes) found on MMC/SD - Card",
				(int) simple_strtoul(getenv("filesize"), NULL, 16));

		// Don't enable printf after update, if DIS_PRINTF in settings.bin file is set.
		set_update_is_active();

		// Check am35x_update.bmp display resolution
		// Try if it is small resolution
		p_vCheckString = (void*) simple_strtoul(cBmpEndSmallResSDRAMAddress,
				NULL, 16);
		iStringCharacters = 3;

		memset(p_cCheckString[0], '\0', 17);
		memcpy(p_cCheckString[0], p_vCheckString, iStringCharacters);

		// Small Resolution
		if (!(memcmp(p_cCheckString[0], p_cBmpCompareString[0],
				iStringCharacters))) {
			memcpy(&cBmpEndSDRAMAddress[0], &cBmpEndSmallResSDRAMAddress[0], 9);
			memcpy(&cBmpErrorSDRAMAddress[0],
					&cBmpErrorSmallResSDRAMAddress[0], 9);
		}
		// Middle Resolution
		else {
			memcpy(&cBmpEndSDRAMAddress[0], &cBmpEndMiddleResSDRAMAddress[0], 9);
			memcpy(&cBmpErrorSDRAMAddress[0],
					&cBmpErrorMiddleResSDRAMAddress[0], 9);
		}

		do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);

		// Read the Update Argument in the am35x_update.bmp file
		p_vUpdateArgument = (void*) simple_strtoul(cArgumentsSDRAMAddress,
				NULL, 16);
	}

	printf(
			"\n--> am35x_update.bmp (%d bytes) found on MMC/SD - Card >> Flash Update started\n",
			(int) simple_strtoul(getenv("filesize"), NULL, 16));

	// Don't enable printf after update, if DIS_PRINTF in settings.bin file is set.
	set_update_is_active();


		// Check am35x_update.bmp display resolution
		// Try if it is small resolution
		p_vCheckString = (void*) simple_strtoul(cBmpEndSmallResSDRAMAddress, NULL,
				16);
		iStringCharacters = 3;

		memset(p_cCheckString[0], '\0', 17);
		memcpy(p_cCheckString[0], p_vCheckString, iStringCharacters);

		// Small Resolution
		if (!(memcmp(p_cCheckString[0], p_cBmpCompareString[0], iStringCharacters))) {
			memcpy(&cBmpEndSDRAMAddress[0], &cBmpEndSmallResSDRAMAddress[0], 9);
			memcpy(&cBmpErrorSDRAMAddress[0], &cBmpErrorSmallResSDRAMAddress[0], 9);
		}
		// Middle Resolution
		else {
			memcpy(&cBmpEndSDRAMAddress[0], &cBmpEndMiddleResSDRAMAddress[0], 9);
			memcpy(&cBmpErrorSDRAMAddress[0], &cBmpErrorMiddleResSDRAMAddress[0], 9);
		}

		do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);


	p_vUpdateArgument = (void*) simple_strtoul(cArgumentsSDRAMAddress, NULL, 16);
        
    switch (*(char*) p_vUpdateArgument)
    {
		// ***************************************************************************************
		// ******************* Update only WinCE Image in the Flash memory ***********************
		// ***************************************************************************************
	case WINCE_IMAGE_UPDATE:
		if (wince_img_update(argc, argv, FileTypeWinCE) != 0) { return 1; }
		break;

		// ***************************************************************************************
		// ******************* Update only Linux Image in the Flash memory ***********************
		// ***************************************************************************************
	case LINUX_IMAGE_UPDATE:
		if (linux_img_update(argc, argv) != 0) { return 1; }
		break;

		// ***************************************************************************************
		// ************************** Update complete Flash Image ********************************
		// ***************************************************************************************
	case COMPLETE_FLASH_UPDATE:

		break;

		// ***************************************************************************************
		// **************************** Bootloader Flash Image ***********************************
		// ***************************************************************************************
	case BOOTLOADER_FLASH_UPDATE:
		if (bootloader_update(argc, argv) != 0) {	return 1; }
		break;

		// ***************************************************************************************
		// ****************************** Update SplashImage *************************************
		// ***************************************************************************************
	case SPLASHIMAGE_UPDATE:
		if (splashimage_update(argc, argv) != 0) { return 1; }
		break;

		// ***************************************************************************************
		// **************************** Update settings.bin File *********************************
		// ***************************************************************************************
	case SETTINGS_BIN_UPDATE:
		if (settings_bin_update(argc, argv) != 0) { return 1; }
		break;

		// ***************************************************************************************
		// ****************************** Update OS Arguments ************************************
		// ***************************************************************************************
	case OS_ARGUMENTS_UPDATE:
		if (os_arguments_update(argc, argv) != 0) { return 1;	}
		break;

		// ***************************************************************************************
		// *** No Update Event is defined for the Update Argument in the am35x_update.bmp file ***
		// ***************************************************************************************
	default:
		printf("\n--> No Update Event defined for the Argument: %s\n",	(char*) p_vUpdateArgument);

		p_cDisplayBmpOnScreen[2] = cBmpErrorSDRAMAddress;

		do_bmp(NULL, 0, 3, p_cDisplayBmpOnScreen);
		return 1;
	}
   
    return 0;
}

/* ====================================================================== */

U_BOOT_CMD(
  update,      1,      0,      do_am35xxupdate,
  "Update NAND Flash content from MMC/SD - Card.",
  "- Starts update with am35x_update.bmp file\n"
  "         Necessary files on SD/MMC Card:\n"
  "           - am35x_update.bmp File\n"
  "           - nk.bin or flash.bin File, ...\n"
  "update <type> [filename] - Starts update without am35x_update.bmp file\n"
  "         Types:\n"
  "           - wince = WinCE image update (default file names nk.gz or nk.bin)\n"
  "           - linux = Linux image update (default file name uImage)\n"
  "           - flash = Complete flash image update \n"
  "           - bootloader = Bootloader update (default file name u-boot.bin)\n"
  "           - splash = Bootbitmap update (default file name splash.bmp)\n"
  "           - settings = Settings block update (default file name settings.bin)\n"
);
#endif/* CONFIG_CMD_UPDATECE */
