//
// Copyright (c) DH electronics GmbH.  All rights reserved.
//
//------------------------------------------------------------------------------
//
//  File: cmd_board_info.c       
//           
//  Version:  1.1
//
//  Abstract: This file contains the source code for the "boardinfo" command line function.
//            This function lists the act. am35x Core module hardware settings.
//
//  Notes:
//		Created by: 	Andreas Geisreiter (ageisreiter@dh-electronics.de), Ludwig Zenz (lzenz@dh-electronics.de)
//  	Created: 		February 14, 2012
//  	Modified: 		
//		
 
#include <common.h>
#include <command.h>
#include <asm/io.h>
#include <asm/errno.h>
#include <asm/errno.h>
#include <nand.h>   

#if defined(CONFIG_CMD_BOARD_INFO)

DECLARE_GLOBAL_DATA_PTR;

extern int dh_get_hw_revision(void);
extern int dh_get_sdram_size_mb(void);

int do_board_info ( cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{             
#ifdef DH_NAND_FLASH                        
    u32 uFlashSize = (u32)nand_info[0].size;
#endif /* DH_NAND_FLASH  */

    printf("[info_start]\n");
    printf("CPU: %s\n", CPU_TYPE_STRING);
    printf("HW: %u\n", dh_get_hw_revision());
    printf("DRAM Size: %u MB\n", dh_get_sdram_size_mb());
#ifdef DH_NAND_FLASH     
    printf("FLASH Size: %u MB\n", uFlashSize/1024/1024);
#else /* DH_SPI_FLASH */ 
    printf("FLASH Size: %u MB\n", SPI_FLASH_SIZE_MB);
#endif /* DH_NAND_FLASH  */   
    printf("FLASH Type: %s\n", FLASH_TYPE_STRING);    
    printf("[info_end]\n");
    return 0;
}

U_BOOT_CMD(
	boardinfo,   1,   1,     do_board_info,
	"Shows the Core Module hardware information",
	"\n"
);

#endif
