//
// Copyright (c) DH electronics GmbH.  All rights reserved.
//
//------------------------------------------------------------------------------
//
//  File: cmd_disp_info.c          
//           
//  Version:  1.0
//
//  Abstract: This file contains the source code for the "lcdinfo" command line function.
//            This function lists the act. i.MX25 display controller settings.
//
//  Notes:
//		Created by: 	Andreas Geisreiter (ageisreiter@dh-electronics.de)
//  	Created: 		May 10, 2011
//  	Modified: 		
//		
 
#include <common.h>
#include <command.h>
#include <lcd.h>
#include <asm/io.h>
#include <asm/errno.h>

DECLARE_GLOBAL_DATA_PTR;

#if defined(CONFIG_CMD_DISPLAY_INFO)

int do_lcd_info ( cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    printf("  Display Resolution: %d x %d pixels\n", (int)gd->dh_board_settings.wXResolution, (int)gd->dh_board_settings.wYResolution);
    printf("  Display Type:       %s Display\n", (gd->dh_board_settings.wLCDConfigFlags & SETTINGS_LCD_ACT_PAS_FLAG) ? "Active" : "Passiv");
    printf("  Pixel Clock:        %d kHz\n",(int)gd->dh_board_settings.wPixelClock);
#ifndef CONFIG_RGB24_FRAMEBUFFER
	printf("  Bits per Pixel:     16 Bits/Pixel\n");
#else
	printf("  Bits per Pixel:     24 Bits/Pixel\n");
#endif

    printf("  Pixel clk Polarity: %s \n", (gd->dh_board_settings.wLCDConfigFlags & SETTINGS_LCD_IPC_FLAG) ? "Active Low" : "Active High");
    printf("  VSYNC Polarity:     %s \n", (gd->dh_board_settings.wLCDConfigFlags & SETTINGS_LCD_IVS_FLAG) ? "Active Low" : "Active High");
    printf("  HSYNC Polarity:     %s \n", (gd->dh_board_settings.wLCDConfigFlags & SETTINGS_LCD_IHS_FLAG) ? "Active Low" : "Active High");

    printf("  Output En Polarity: %s \n", (gd->dh_board_settings.wLCDConfigFlags & SETTINGS_LCD_IOE_FLAG) ? "Active Low" : "Active High");
    printf("  HSYNC Puls Width:   %d Clock Cycles\n",(int) gd->dh_board_settings.wHPulseWidth);
    printf("  HSYNC Back Porch:   %d Clock Cycles\n",(int)gd->dh_board_settings.wHBackPorch);
    printf("  HSYNC Front Porch:  %d Clock Cycles\n",(int)gd->dh_board_settings.wHFrontPorch);
    printf("  VSYNC Puls Width:   %d Line Cycles\n",(int)gd->dh_board_settings.wVPulseWidth);
    printf("  VSYNC Back Porch:   %d Line Cycles\n",(int)gd->dh_board_settings.wVBackPorch);
    printf("  VSYNC Front Porch:  %d Line Cycles\n",(int)gd->dh_board_settings.wVFrontPorch);
    printf("  LCD Data Lines:     %d \n", (int)gd->dh_board_settings.cDatalines);
    return 0;
}

U_BOOT_CMD(
	lcdinfo,   1,   1,     do_lcd_info,
	"shows the display controller settings",
	"\n"
);

#endif
