/*
 * File: cmd_eraseenv.c
 *
 * (C) Copyright 2011-2012
 * Andreas Geisreiter , DH electronics GmbH , ageisreiter@dh-electronics.de
 * Ludwig Zenz, DH electronics GmbH , lzenz@dh-electronics.de
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 *  Version:  1.1
 *
 *  Abstract: This file contains the source code for the "eraseenv" command line function.
 *            This function removes the act. env settings from NAND and SPI flash.
 *
 *  Notes:
 *  	Created: 		March 23, 2012 by Andreas Geisreiter (ageisreiter@dh-electronics.de)
 *  	Modified: 		March 28, 2012, Customized for am35x by Ludwig Zenz (lzenz@dh-electronics.de)
 */
 
#include <common.h>
#include <command.h>

#include <asm/io.h>

#ifdef DH_NAND_FLASH    
extern int do_nand(cmd_tbl_t * cmdtp, int flag, int argc, char *argv[]);
#else
extern int do_spi_flash(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[]);
#endif /* DH_NAND_FLASH  */

/*
 * command to erase the u-boot environment from the persistent memory
 */
int do_eraseenv ( cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{            
    int ret_value = 0;

#ifdef DH_NAND_FLASH   

    char *p_cEraseFlash[4] = {"nand", "erase", "12345671", "12345672"};

    sprintf (p_cEraseFlash[2], "%08x", (unsigned int)(CONFIG_ENV_OFFSET));
    sprintf (p_cEraseFlash[3], "%08x", (unsigned int)(CONFIG_ENV_SIZE));

    ret_value = do_nand(NULL, 0, 4, p_cEraseFlash);


	#ifdef CONFIG_ENV_OFFSET_REDUND
		if (ret_value != 0) /* check for erase errors before erasing redundant environment*/
			return 1; /* error occured -> exit */

		sprintf (p_cEraseFlash[2], "%08x", (unsigned int)(CONFIG_ENV_OFFSET_REDUND));
		sprintf (p_cEraseFlash[3], "%08x", (unsigned int)(CONFIG_ENV_SIZE_REDUND));

		ret_value = do_nand(NULL, 0, 4, p_cEraseFlash);
	#endif

#else /* DH_SPI_FLASH */
    char *p_cEraseFlash[4] = {"sf", "erase", "12345671", "12345672"};  

    sprintf (p_cEraseFlash[2], "%08x", (unsigned int)(CONFIG_ENV_OFFSET)); 
    sprintf (p_cEraseFlash[3], "%08x", (unsigned int)(CONFIG_ENV_SIZE));

    ret_value = do_spi_flash(NULL, 0, 4, p_cEraseFlash);
#endif /* DH_NAND_FLASH  */
    
    return ret_value;
}

U_BOOT_CMD(
	eraseenv,   1,   1,     do_eraseenv,
	"Delete the environment in persistent memory",
	"\n"
);
