//
// Copyright (c) DH electronics GmbH.  All rights reserved.
//
//------------------------------------------------------------------------------
//
// File: cmd_lcd_debug.c
//           
// Abstract: Command line function to debug lcd interface problems
//
// Notes:
//     Created by: 	Ludwig Zenz / lzenz@dh-electronics.de
//     Created:     June 22, 2012
//
//------------------------------------------------------------------------------

#include <config.h>
#include <common.h>
#include <command.h>


#if defined(CONFIG_CMD_LCD_DEBUG)

DECLARE_GLOBAL_DATA_PTR;

// ===============================================================================================================
// Defines
// ===============================================================================================================

#define REG_DSS_SYSCONFIG	0x48050010
#define DSS_SOFTRESET		1

#define REG_DSS_STATUS 		0x48050014
#define DSS_RESETDONE		0

#define REG_DSS_IRQSTATUS 	0x48050018
#define DSI_IRQ 			1
#define DISPC_IRQ			0

#define REG_DISPC_SYSCONFIG 0x48050410
#define DISPC_SOFTRESET		1

#define REG_DISPC_SYSSTATUS 0x48050414
#define DISPC_RESETDONE		0

#define REG_DISPC_IRQSTATUS 0x48050418
#define IRQ_WAKEUP			  	16
#define IRQ_SYNCLOSTDIGITAL	 	15
#define IRQ_SYNCLOST		 	14
#define IRQ_VID2ENDWINDOW	  	13
#define IRQ_VID2FIFOUNDERFL   	12
#define IRQ_VID1ENDWINDOW	  	11
#define IRQ_VID1FIFOUNDERFLOW 	10
#define IRQ_OCPERROR		  	9
#define IRQ_PALETTEGAMMALOADING	8
#define IRQ_GFXENDWINDOW		7
#define IRQ_GFXFIFOUNDERFLOW	6
#define IRQ_PROGRAMEDLINENUMBER	5
#define IRQ_ACBIASCOUNTSTATUS	4
#define IRQ_EVSYNC_ODD			3
#define IRQ_EVSYNC_EVEN			2
#define IRQ_VSYNC				1
#define IRQ_FRAMEDONE			0

#define CHECK_REG_BIT(addr, bitnumber) \
		*((uint*) addr) & (0x01 << bitnumber)

// ===============================================================================================================
// Variables
// ===============================================================================================================

// ===============================================================================================================
// Extern defined variables
// ===============================================================================================================

// ===============================================================================================================
// Extern defined functions
// ===============================================================================================================
extern void lcd_ctrl_init (void *lcdbase);
extern void lcd_enable (void);

// ===============================================================================================================
// Functions
// ===============================================================================================================


//------------------------------------------------------------------------------
//
//  Function:  lcd_debug_reinit
//
//  Reinit Display Subsystem after doing a reset.
//
//  Return value:   0 = No error
//                  1 = error
//
int lcd_debug_reinit (void)
{
	/* Execute Reset of DSS */
	printf("reseting dss ... \n");
	*((uint*) REG_DSS_SYSCONFIG)		= 0x01 << DSS_SOFTRESET;

	while (!CHECK_REG_BIT(REG_DSS_STATUS, DSS_RESETDONE));

	/* Execute Reset of DISPC */
	printf("reseting display controller ... \n");
	*((uint*) REG_DISPC_SYSCONFIG)		= 0x01 << DISPC_SOFTRESET;

	while (!CHECK_REG_BIT(REG_DISPC_SYSSTATUS, DISPC_RESETDONE));

	/* reinitialize display */
	printf("initialize lcd ... \n");
	lcd_ctrl_init ((void *)(gd->fb_base));

	/* enable lcd */
	printf("enable lcd ... \n");
	lcd_enable();

	return 0;
}

//------------------------------------------------------------------------------
//
//  Function:  lcd_debug_errorcheck
//
//  Check for error interrupts at Display Subsystem
//
//  Return value:   0 = No error
//                  1 = error
//
int lcd_debug_errorcheck (void)
{
	printf("check for occured interrupts ... \n");

	if (CHECK_REG_BIT(REG_DSS_IRQSTATUS, DSI_IRQ))
		printf ("DSI_IRQ occured!\n");

	if (CHECK_REG_BIT(REG_DSS_IRQSTATUS, DISPC_IRQ))
		printf ("DISPC_IRQ occured!\n");

	if (CHECK_REG_BIT(REG_DISPC_IRQSTATUS, IRQ_WAKEUP))
		printf ("IRQ_WAKEUP occured!\n");

	//if (CHECK_REG_BIT(REG_DISPC_IRQSTATUS, IRQ_SYNCLOSTDIGITAL))
	//	printf ("IRQ_SYNCLOSTDIGITAL occured!\n");


	if (CHECK_REG_BIT(REG_DISPC_IRQSTATUS, IRQ_SYNCLOST))
		printf ("IRQ_SYNCLOST occured!\n");

	if (CHECK_REG_BIT(REG_DISPC_IRQSTATUS, IRQ_VID2ENDWINDOW))
		printf ("IRQ_VID2ENDWINDOW occured!\n");

	if (CHECK_REG_BIT(REG_DISPC_IRQSTATUS, IRQ_VID2FIFOUNDERFL))
		printf ("IRQ_VID2FIFOUNDERFL occured!\n");

	if (CHECK_REG_BIT(REG_DISPC_IRQSTATUS, IRQ_VID1ENDWINDOW))
		printf ("IRQ_VID1ENDWINDOW occured!\n");

	if (CHECK_REG_BIT(REG_DISPC_IRQSTATUS, IRQ_VID1FIFOUNDERFLOW))
		printf ("IRQ_VID1FIFOUNDERFLOW occured!\n");

	if (CHECK_REG_BIT(REG_DISPC_IRQSTATUS, IRQ_OCPERROR))
		printf ("IRQ_OCPERROR occured!\n");

	if (CHECK_REG_BIT(REG_DISPC_IRQSTATUS, IRQ_PALETTEGAMMALOADING))
		printf ("IRQ_PALETTEGAMMALOADING occured!\n");

	//if (CHECK_REG_BIT(REG_DISPC_IRQSTATUS, IRQ_GFXENDWINDOW))
	//	printf ("IRQ_GFXENDWINDOW occured!\n");

	if (CHECK_REG_BIT(REG_DISPC_IRQSTATUS, IRQ_GFXFIFOUNDERFLOW))
		printf ("IRQ_GFXFIFOUNDERFLOW occured!\n");

	//if (CHECK_REG_BIT(REG_DISPC_IRQSTATUS, IRQ_PROGRAMEDLINENUMBER))
	//	printf ("IRQ_PROGRAMEDLINENUMBER occured!\n");

	if (CHECK_REG_BIT(REG_DISPC_IRQSTATUS, IRQ_ACBIASCOUNTSTATUS))
		printf ("IRQ_ACBIASCOUNTSTATUS occured!\n");

	//if (CHECK_REG_BIT(REG_DISPC_IRQSTATUS, IRQ_EVSYNC_ODD))
	//	printf ("IRQ_EVSYNC_ODD occured!\n");

	//if (CHECK_REG_BIT(REG_DISPC_IRQSTATUS, IRQ_EVSYNC_EVEN))
	//	printf ("IRQ_EVSYNC_EVEN occured!\n");

	//if (CHECK_REG_BIT(REG_DISPC_IRQSTATUS, IRQ_VSYNC))
	//	printf ("IRQ_VSYNC occured!\n");

	if (CHECK_REG_BIT(REG_DISPC_IRQSTATUS, IRQ_FRAMEDONE))
		printf ("IRQ_FRAMEDONE occured!\n");

	return 0;
}


//------------------------------------------------------------------------------
//
//  Function:  do_lcd_debug
//
//  DHCOM U-Boot Update function
//  Description:	Scan for available storage devices
//
//  Return value:   0 = No error
//                  1 = error
//
int do_lcd_debug (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	char *cmd;

	/* at least two arguments please */
	if (argc < 2)
		goto usage;

	cmd = argv[1];

	/* reinit display subsystem */
	if (strcmp(cmd, "reset") == 0)
	{
		if (argc != 2)
			goto usage;

		printf("reset and reinitialize display-subsystem  ...\n\n");
		return lcd_debug_reinit();
	}

	/* check for error interrupts */
	if (strcmp(cmd, "error") == 0)
	{
		if (argc != 2)
			goto usage;

		printf("check if error interrupts occurred  ...\n\n");
		return lcd_debug_errorcheck();
	}

	usage:
		cmd_usage(cmdtp);
		return 1;
}

/* ====================================================================== */

U_BOOT_CMD(
  lcd,      3,      0,      do_lcd_debug,
  "debug am3517 lcd interface.",
  "reset   - reinitialize the lcd interface\n"
  "lcd error   - check error interrupt flags\n"
);
#endif/* CONFIG_CMD_LCD_DEBUG */
