/*
 * (C) Copyright 2003
 * Kyle Harris, kharris@nexus-tech.net
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>
#include <asm/arch/mmc_omap3.h>
static int curr_device = -1;

/*
 * Print info about current mmc-device
 */
static void print_mmcinfo(void)
{
	printf("\n\n");
	printf("Current Device:   mmc%d\n", curr_device);
	printf("Manufacturer ID:  %x\n", cur_card_data.mid);
	printf("OEM ID:           %d\n", cur_card_data.oid);

	printf("Product Name:     %c%c%c%c%c%c \n", cur_card_data.pnm[5], cur_card_data.pnm[4],
			cur_card_data.pnm[3], cur_card_data.pnm[2],	cur_card_data.pnm[1], cur_card_data.pnm[0]);

	printf("Serial number:    %d\n", cur_card_data.psn);

	printf("Transfer Speed:   %d kbit\\s\n", cur_card_data.tran_speed);
	printf("Read Block Len:   %d\n", cur_card_data.read_bl_len);

	printf("Production Date:  %d.%d\n", cur_card_data.month, cur_card_data.year);
	printf("Product revision: %i.%i\n", cur_card_data.prv[1], cur_card_data.prv[0]);

	printf("SDHC-Card:        %s\n", cur_card_data.high_capacity ? "Yes" : "No");
	printf("Capacity:         %d bytes (%d MB)\n", cur_card_data.size, (cur_card_data.size >> 20));
	printf("\n");
}

/*
 * This function is called if you fun a mmc-command
 * - here we decide which mmc-command to run
 * - check if the command-arguments are right
 */
int do_mmc (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	int dev;
	int hsnum;

	if (argc < 2) 		/* Check minimum number of arguments*/
	{
		cmd_usage(cmdtp);
		return 1;
	}

	/* mmc init */
	if (strcmp(argv[1], "init") == 0)
	{
		if (argc == 2) {
			if (curr_device < 0)
				dev = 0;
			else
				dev = curr_device;
		} else if (argc == 3) {
			dev = (int)simple_strtoul(argv[2], NULL, 10);
		} else {
			cmd_usage(cmdtp);
			return 1;
		}

		if (mmc_legacy_init(dev) != 0) {
			puts("No MMC card found\n");
			return 1;
		}
		// initialize mmcx was successfull
		curr_device = dev;
		printf("mmc%d is available\n", curr_device);
		return 0;
	}

	/* mmc device [dev] - set or show current device */
	if (strcmp(argv[1], "device") == 0)
	{
		if (argc == 2) {
			if (curr_device < 0) {
				puts("No MMC device available\n");
				return 1;
			}
		} else if (argc == 3) {
			dev = (int)simple_strtoul(argv[2], NULL, 10);

#ifdef CONFIG_SYS_MMC_SET_DEV
			if (mmc_set_dev(dev) != 0)
				return 1;
#endif
			curr_device = dev;
		} else {
			cmd_usage(cmdtp);
			return 1;
		}

		printf("mmc%d is current device\n", curr_device);
		return 0;
	}

	/* print mmc info */
	if (strcmp(argv[1], "info") == 0)
	{
		if (argc == 2)
		{
			if (curr_device < 0)
				dev = 0;
			else
				dev = curr_device;
		} else if (argc == 3)				/* set device from input string */
		{
			dev = (int)simple_strtoul(argv[2], NULL, 10);
		} else								/* too many arguments */
		{
			cmd_usage(cmdtp);
			return 1;
		}

		if (mmc_legacy_init(dev) != 0)		/* Run the initialization of the mmc */
		{
			puts("No MMC card found\n");	/* initialization was not successful */
			return 1;
		}
		curr_device = dev;		/* initialize a mmc- of sd-card was successful */

		print_mmcinfo();		/* Print the available card information */
		return 0;
	}

	/* mmc switch host controller */
	if (strcmp(argv[1], "switch") == 0)
	{
		if (argc == 2) /* if only 2 arguments use MMCHS1 as default */
		{
			hsnum = 1;
		}
		else if (argc == 3) /* if 3 arguments: compute the third argument */
		{
			hsnum = (int)simple_strtoul(argv[2], NULL, 10); /* String to number */
			if (!((1==hsnum)||(2==hsnum)||(3==hsnum)))
			{
				cmd_usage(cmdtp);							/* Number out of range */
				return 1;
			}

		}
		else /* Too many arguments - print command usage information */
		{
			cmd_usage(cmdtp);
			return 1;
		}
		return (mmc_set_host_controller_base(hsnum));
	}
	else /* Wrong command input - print command usage information */
	{
		cmd_usage(cmdtp);
		return 1;
	}

	return 0;
}

U_BOOT_CMD(
	mmc, 3, 1, do_mmc,
	"MMC sub-system",
	"init [dev]     - init MMC sub system\n"
	"mmc device [dev]   - show or set current device\n"
	"mmc info [dev]     - display MMC info\n"
	"mmc switch [dev]   - switch between MMC Host Controller 1, 2 or 3\n"
);

/*
 * This is called via DHCOM update routines!
 */
int do_mmcinit (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	int dev_num = 1;

	if (strcmp(argv[0], "mmcinit") == 0)
	{
		if (argc == 2)	{
			/* get host controller id from string */
			dev_num = (int)simple_strtoul(argv[1], NULL, 10);

			/* set the correct base address */
			mmc_set_host_controller_base(dev_num);

		}
		else
		{
			printf("wrong length of argc! %d\n", argc);
			return 1; /* wrong count of arguments, exit with error code */
		}

		if (mmc_legacy_init(dev_num) != 0) {
		puts("No MMC card found\n");
		return 1;
		}

		// initialize mmcx was successfull
		curr_device = dev_num;
		printf("mmc%d is available\n", curr_device);

		return 0;
	}


	/* Command call is only valid with "mmcinit" in argc[0] */
	return 1;
}
