/*
 * 2012 (c) DH electronics GmbH
 * Author: Andreas Geisreiter <ageisreiter at dh-electronics.de>
 *
 */

#include <common.h>
#include <command.h>
#include <linux/ctype.h>
#include <asm/io.h>
#ifdef CONFIG_MX25
#include <asm/arch/mx25.h>
#include <asm/arch/mx25-regs.h>
#include <asm/arch/mx25_pins.h>
#include <asm/arch/iomux.h>
#endif
#include <asm/arch/gpio.h>
#include <dh_settings.h>
#include <dh_update.h>

extern void udelay(unsigned long usec);

#ifdef CONFIG_MX25
iomux_pin_name_t iMXUpdateLEDPin;
#elif defined (CONFIG_AM3517)
int iUpdateLEDPin;
#endif
enum UpdateGPIOEnum eDHCOMUpdateGPIO = GPIO_NOT_DEFINED;

//------------------------------------------------------------------------------
//
//  Function:  DHCOMUpdateLED_Init
//
//  This function initialize the DHCOM update LED GPIO
//
//  Commit values:  - *p_stDHupdateINI = Pointer to DHupdateINI struct
//
//  Return value:   0 = No error
//                  1 = error
//
int DHCOMUpdateLED_Init(updateinfo_t *p_stDHupdateINI)
{
   char cCompareStringLEDGPIO[7] = {"GPIO_A\0"};
   
   for(eDHCOMUpdateGPIO = GPIO_A; eDHCOMUpdateGPIO != GPIO_NOT_DEFINED; eDHCOMUpdateGPIO++, cCompareStringLEDGPIO[5]++)
   {
        if(!(memcmp ( p_stDHupdateINI->p_cUpdateGpioName, (char*)&cCompareStringLEDGPIO[0], 7)))
        {
            break;
        }
   }
   
    switch (eDHCOMUpdateGPIO) {
        case GPIO_A:
			#if defined(CONFIG_MX25)
            // DHCOM GPIO_A --> gpio2_GPIO[12]           
            iMXUpdateLEDPin = MX25_PIN_EB0;
			#elif defined (CONFIG_AM3517)
            iUpdateLEDPin = DHCOM_GPIO_A;
			#endif
            break;
        case GPIO_B:
			#if defined(CONFIG_MX25)
            // DHCOM GPIO_B --> gpio3_GPIO[21]
            iMXUpdateLEDPin = MX25_PIN_CS5;
			#elif defined (CONFIG_AM3517)
			iUpdateLEDPin = DHCOM_GPIO_B;
			#endif
            break;
        case GPIO_C:
			#if defined(CONFIG_MX25)
            // DHCOM GPIO_C --> gpio2_GPIO[4]
            iMXUpdateLEDPin = MX25_PIN_A18;
			#elif defined (CONFIG_AM3517)
			iUpdateLEDPin = DHCOM_GPIO_C;
			#endif
            break;
        case GPIO_D:
			#if defined(CONFIG_MX25)
            // DHCOM GPIO_D --> gpio2_GPIO[5]
            iMXUpdateLEDPin = MX25_PIN_A19;
			#elif defined (CONFIG_AM3517)
			iUpdateLEDPin = DHCOM_GPIO_D;
			#endif
            break;
        case GPIO_E:
			#if defined(CONFIG_MX25)
            // DHCOM GPIO_E --> gpio2_GPIO[6]
            iMXUpdateLEDPin = MX25_PIN_A20;
			#elif defined (CONFIG_AM3517)
			iUpdateLEDPin = DHCOM_GPIO_E;
			#endif
            break;
        case GPIO_F:
			#if defined(CONFIG_MX25)
            // DHCOM GPIO_F --> gpio2_GPIO[7]
            iMXUpdateLEDPin = MX25_PIN_A21;
			#elif defined (CONFIG_AM3517)
			iUpdateLEDPin = DHCOM_GPIO_F;
			#endif
            break;
        case GPIO_G:
			#if defined(CONFIG_MX25)
            // DHCOM GPIO_G --> gpio2_GPIO[8]
            iMXUpdateLEDPin = MX25_PIN_A22;
			#elif defined (CONFIG_AM3517)
			iUpdateLEDPin = DHCOM_GPIO_G;
			#endif
            break;
        case GPIO_H:
			#ifdef CONFIG_MX25
            // DHCOM GPIO_H --> gpio2_GPIO[9]
            iMXUpdateLEDPin = MX25_PIN_A23;
			#elif defined (CONFIG_AM3517)
			iUpdateLEDPin = DHCOM_GPIO_H;
			#endif
            break;
        case GPIO_I:
			#if defined(CONFIG_MX25)
            // DHCOM GPIO_I --> gpio3_GPIO[17]   
            iMXUpdateLEDPin = MX25_PIN_VSTBY_REQ;
			#elif defined (CONFIG_AM3517)
			iUpdateLEDPin = DHCOM_GPIO_I;
			#endif
            break;
        default:
            return 1;
            break;
    }

    // Set to output
#if defined(CONFIG_MX25)
    mxc_set_gpio_direction(iMXUpdateLEDPin, 0);
#elif defined (CONFIG_AM3517)
    omap_set_gpio_direction(iUpdateLEDPin, 0);
#endif
    
    return 0;
}

//------------------------------------------------------------------------------
//
//  Function:  DHCOMUpdateLED_SetHigh
//
//  This function sets the DHCOM update LED GPIO to high
//
//  Commit values:  - 
//
//  Return value:   -
//
void DHCOMUpdateLED_SetHigh(void)
{
    if(eDHCOMUpdateGPIO != GPIO_NOT_DEFINED)
    {
    	// Set to high
		#if defined(CONFIG_MX25)
			mxc_set_gpio_dataout(iMXUpdateLEDPin, 1);
		#elif defined (CONFIG_AM3517)
			omap_set_gpio_dataout(iUpdateLEDPin, 1);
		#endif
    }
}

//------------------------------------------------------------------------------
//
//  Function:  DHCOMUpdateLED_SetLow
//
//  This function sets the DHCOM update LED GPIO to low
//
//  Commit values:  -
//
//  Return value:   -
//
void DHCOMUpdateLED_SetLow(void)
{
    if(eDHCOMUpdateGPIO != GPIO_NOT_DEFINED)
    {
        // Set to low
		#if defined(CONFIG_MX25)
			mxc_set_gpio_dataout(iMXUpdateLEDPin, 0);
		#elif defined (CONFIG_AM3517)
			omap_set_gpio_dataout(iUpdateLEDPin, 0);
		#endif
    }
}

//------------------------------------------------------------------------------
//
//  Function:  get_DHCOMUpdateLED
//
//  This function is to read the update-led gpio number
//
//  Commit values:  -
//
//  Return value:   -
//
int get_DHCOMUpdateLED(void)
{
	#if defined(CONFIG_MX25)
		unsigned int tmpGPIO;
		tmpGPIO = (unsigned int)iMXUpdateLEDPin;
		return (int)((tmpGPIO >> 24) & 0x000000FF);
	#elif defined (CONFIG_AM3517)
		return iUpdateLEDPin;
	#endif
}

//------------------------------------------------------------------------------
//
//  Function:  DHCOMUpdateDelayMs
//
//  This function calls the CPU specified uDelay function
//
//  Commit values:  - msec = delay time in msec
//
//  Return value:   -
//
void DHCOMUpdateDelayMs(unsigned long msec)
{

    unsigned long usec;
    
    usec = msec * 1000;
    
    udelay(usec);
}
