/*
 * Copyright (C) 2009 Texas Instruments Incorporated - http://www.ti.com/
 * Vaibhav Hiremath <hvaibhav@ti.com>
 *
 * Based on mem.c and sdrc.c
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <asm/io.h>
#include <asm/arch/mem.h>
#include <asm/arch/sys_proto.h>
#include <asm/arch/emif4.h>

extern omap3_sysinfo sysinfo;
extern int dh_get_sdram_size_mb(void);

static emif4_t *emif4_base = (emif4_t *)OMAP34XX_SDRC_BASE;

/**
 * is_mem_sdr() - return 1 if mem type in use is SDR
 */
u32 is_mem_sdr(void)
{
	return 0;
}

/**
 * get_sdr_cs_size() - get size of chip select 0/1
 */
u32 get_sdr_cs_size(u32 cs)
{
	/* u32 size;

	// TODO: Calculate the size based on EMIF4 configuration
	size = PHYS_SDRAM_1_SIZE;

	#ifdef  PHYS_SDRAM_2_SIZE
	size += PHYS_SDRAM_2_SIZE;
	#endif

	return size;
	*/

	/* LZ: generate output depending ohne HW-Code */
	return (dh_get_sdram_size_mb() << 20);
}

/**
 * get_sdr_cs_offset() - get offset of cs from cs0 start
 */
u32 get_sdr_cs_offset(u32 cs)
{
	u32 offset=0;

	return offset;
}

/**
 * do_emif4_init() - init the emif4 module for DDR access
 *  - early init routines, called from flash or
 *  SRAM.
 *
 * 2011-10-13:	Customized for DHCOM Cortex A8
 */
void do_emif4_init(void)
{
	unsigned int regval;

	/* Set the DDR PHY parameters in PHY ctrl registers */
	regval = 	(EMIF4_DDR1_RD_LAT |
				(EMIF4_DDR1_PWRDN_DIS << 6) |
				(EMIF4_DDR1_STRBEN_EXT << 7) |
				(EMIF4_DDR1_DLL_MODE << 12) |
				(EMIF4_DDR1_VTP_DYN_DIS << 15) |
				(EMIF4_DDR1_LB_CK_SEL << 23));
	writel(regval, &emif4_base->ddr_phyctrl1);
	writel(regval, &emif4_base->ddr_phyctrl1_shdw);

	regval = (EMIF4_DDR2_TX_DATA_ALIGN | (EMIF4_DDR2_RX_DLL_BYPASS << 1));
	writel(regval, &emif4_base->ddr_phyctrl2);

	/* Reset the DDR PHY and wait till completed */
	regval = readl(&emif4_base->sdram_iodft_tlgc);
	regval |= (1<<10);
	writel(regval, &emif4_base->sdram_iodft_tlgc);
	/*Wait till that bit clears*/
	while ((readl(&emif4_base->sdram_iodft_tlgc) & (1<<10)) == 0x1);

	/*Re-verify the DDR PHY status*/
	while ((readl(&emif4_base->sdram_sts) & (1<<2)) == 0x0);

	/* Set bit0 of "Test Logic Global Control" Register */
	/* Functional mode enable */
	regval |= (1<<0);
	writel(regval, &emif4_base->sdram_iodft_tlgc);

	/* VTP Dynamic Update */
	/* Voltage, Temperature, Process update is recommended to achive a more robust system */
	/* Set the DDR PHY parameters in PHY ctrl registers */
	regval = 	(EMIF4_DDR1_RD_LAT |
				(EMIF4_DDR1_PWRDN_DIS << 6) |
				(EMIF4_DDR1_STRBEN_EXT << 7) |
				(EMIF4_DDR1_DLL_MODE << 12) |
				(EMIF4_DDR1_VTP_DYN_EN << 15) |
				(EMIF4_DDR1_LB_CK_SEL << 23));

	writel(regval, &emif4_base->ddr_phyctrl1);
	writel(regval, &emif4_base->ddr_phyctrl1_shdw);

	/*Wait till VTP-Ready bit is set*/
	while ((readl(0x48002584) & (1<<5)) == (1<<5));

	/* Clear the CONFIG_VTP_DYNAMIC_UPDATE bit */
	regval = 	(EMIF4_DDR1_RD_LAT |
				(EMIF4_DDR1_PWRDN_DIS << 6) |
				(EMIF4_DDR1_STRBEN_EXT << 7) |
				(EMIF4_DDR1_DLL_MODE << 12) |
				(EMIF4_DDR1_VTP_DYN_DIS << 15) |
				(EMIF4_DDR1_LB_CK_SEL << 23));

	writel(regval, &emif4_base->ddr_phyctrl1);
	writel(regval, &emif4_base->ddr_phyctrl1_shdw);

	/* Set SDR timing registers */
	regval = (EMIF4_TIM1_T_WTR | (EMIF4_TIM1_T_RRD << 3) |
		(EMIF4_TIM1_T_RC << 6) | (EMIF4_TIM1_T_RAS << 12) |
		(EMIF4_TIM1_T_WR << 17) | (EMIF4_TIM1_T_RCD << 21) |
		(EMIF4_TIM1_T_RP << 25));
	writel(regval, &emif4_base->sdram_time1);
	writel(regval, &emif4_base->sdram_time1_shdw);

	regval = (EMIF4_TIM2_T_CKE | (EMIF4_TIM2_T_RTP << 3) |
		(EMIF4_TIM2_T_XSRD << 6) | (EMIF4_TIM2_T_XSNR << 16) |
		(EMIF4_TIM2_T_ODT << 25) | (EMIF4_TIM2_T_XP << 28));
	writel(regval, &emif4_base->sdram_time2);
	writel(regval, &emif4_base->sdram_time2_shdw);

	regval = (EMIF4_TIM3_T_RAS_MAX | (EMIF4_TIM3_T_RFC << 4) |
		(EMIF4_TIM3_T_TDQSCKMAX << 13));
	writel(regval, &emif4_base->sdram_time3);
	writel(regval, &emif4_base->sdram_time3_shdw);

	/* Set the PWR control register */
	regval = (EMIF4_PWR_PM_TIM | (EMIF4_PWR_PM_EN << 8) |
		(EMIF4_PWR_DPD_EN << 10) | (EMIF4_PWR_IDLE << 30));
	writel(regval, &emif4_base->sdram_pwr_mgmt);
	writel(regval, &emif4_base->sdram_pwr_mgmt_shdw);

	/* Set the DDR refresh rate control register */
	regval = (EMIF4_REFRESH_RATE | (EMIF4_PASR << 24) |
		(EMIF4_INITREF_DIS << 31));
	writel(regval, &emif4_base->sdram_refresh_ctrl);
	writel(regval, &emif4_base->sdram_refresh_ctrl_shdw);

	/* set the SDRAM configuration register */
	regval = (EMIF4_CFG_PGSIZE | (EMIF4_CFG_EBANK << 3) |
		(EMIF4_CFG_IBANK << 4) | (EMIF4_CFG_ROWSIZE << 7) |
		(EMIF4_CFG_CL << 10) | (EMIF4_CFG_NARROW_MD << 14) |
		(EMIF4_CFG_CWL << 16) | (EMIF4_CFG_SDR_DRV << 18) |
		(EMIF4_CFG_DDR_DIS_DLL << 20) | (EMIF4_CFG_DYN_ODT << 21) |
		(EMIF4_CFG_DDR2_DDQS << 23) | (EMIF4_CFG_DDR_TERM << 24) |
		(EMIF4_CFG_IBANK_POS << 27) | (EMIF4_CFG_SDRAM_TYP << 29));
	writel(regval, &emif4_base->sdram_config);
}

/**
 * dram_init - Sets uboots idea of sdram size
 */
int dram_init(void)
{
	DECLARE_GLOBAL_DATA_PTR;
	unsigned int size0 = 0, size1 = 0;

	size0 = get_sdr_cs_size(CS0);
	/*
	 * If a second bank of DDR is attached to CS1 this is
	 * where it can be started.  Early init code will init
	 * memory on CS0.
	 */
	if ((sysinfo.mtype == DDR_COMBO) || (sysinfo.mtype == DDR_STACKED))
	{
		size1 = get_sdr_cs_size(CS1);
	}

	gd->bd->bi_dram[0].start = PHYS_SDRAM_1;
	gd->bd->bi_dram[0].size = size0;

	if ((sysinfo.mtype == DDR_COMBO) || (sysinfo.mtype == DDR_STACKED))
	{
		gd->bd->bi_dram[1].start = PHYS_SDRAM_1 + get_sdr_cs_offset(CS1);
		gd->bd->bi_dram[1].size = size1;
	}


	return 0;
}

/**
 * mem_init() - Initialize memory subsystem
 */
void mem_init(void)
{
	do_emif4_init();
}
