/*
 * (C) Copyright 2008
 * Texas Instruments, <www.ti.com>
 * Syed Mohammed Khasim <khasim@ti.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation's version 2 of
 * the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <common.h>
#include <fat.h>
#include <part.h>
#include <i2c.h>
#include <twl4030.h>
#include <asm/io.h>
#include <asm/arch/mmc_omap3.h>

const unsigned short mmc_transspeed_val[15][4] = {
	{CLKD(10, 1), CLKD(10, 10), CLKD(10, 100), CLKD(10, 1000)},
	{CLKD(12, 1), CLKD(12, 10), CLKD(12, 100), CLKD(12, 1000)},
	{CLKD(13, 1), CLKD(13, 10), CLKD(13, 100), CLKD(13, 1000)},
	{CLKD(15, 1), CLKD(15, 10), CLKD(15, 100), CLKD(15, 1000)},
	{CLKD(20, 1), CLKD(20, 10), CLKD(20, 100), CLKD(20, 1000)},
	{CLKD(26, 1), CLKD(26, 10), CLKD(26, 100), CLKD(26, 1000)},
	{CLKD(30, 1), CLKD(30, 10), CLKD(30, 100), CLKD(30, 1000)},
	{CLKD(35, 1), CLKD(35, 10), CLKD(35, 100), CLKD(35, 1000)},
	{CLKD(40, 1), CLKD(40, 10), CLKD(40, 100), CLKD(40, 1000)},
	{CLKD(45, 1), CLKD(45, 10), CLKD(45, 100), CLKD(45, 1000)},
	{CLKD(52, 1), CLKD(52, 10), CLKD(52, 100), CLKD(52, 1000)},
	{CLKD(55, 1), CLKD(55, 10), CLKD(55, 100), CLKD(55, 1000)},
	{CLKD(60, 1), CLKD(60, 10), CLKD(60, 100), CLKD(60, 1000)},
	{CLKD(70, 1), CLKD(70, 10), CLKD(70, 100), CLKD(70, 1000)},
	{CLKD(80, 1), CLKD(80, 10), CLKD(80, 100), CLKD(80, 1000)}
};

mmc_card_data cur_card_data;
static block_dev_desc_t mmc_blk_dev;
static hsmmc_t *mmc_base = (hsmmc_t *)OMAP_HSMMC_BASE1; /* Base-address of MMCHS1 is default */

/*
 * Switch between the different Base-Addresses of the MMCHSx
 */
unsigned char mmc_set_host_controller_base(int HSnum)
{
	switch(HSnum)
	{
		case 2: /* Onboard microSD - Card */
			mmc_base = (hsmmc_t *)OMAP_HSMMC_BASE1; /* Work with MMCHS1 */
			break;
		case 1: /* External SD - Card interface */
			mmc_base = (hsmmc_t *)OMAP_HSMMC_BASE2; /* Work with MMCHS2 */
			break;
		case 3:
			mmc_base = (hsmmc_t *)OMAP_HSMMC_BASE3; /* Work with MMCHS3 */
			break;
		default:
			return 0; // HSnum out of range
	}
	return 1; // If successful return 1
}

block_dev_desc_t *mmc_get_dev(int dev)
{
	return (block_dev_desc_t *) &mmc_blk_dev;
}

/*
 * Never called! - Because these registers are not available at am35xx controllers
 */
unsigned char mmc_board_init(void)
{
	t2_t *t2_base = (t2_t *)T2_BASE; // Reference to the Base of the System Control Module

#if defined(CONFIG_TWL4030_POWER) // the TWL4030 is a integrated power-management/audio CODEC device - we don't have it
	twl4030_power_mmc_init();
#endif

	writel(readl(&t2_base->pbias_lite) | PBIASLITEPWRDNZ1 | // This Register is removed in the am35xx series!!!
		PBIASSPEEDCTRL0 | PBIASLITEPWRDNZ0,
		&t2_base->pbias_lite);

	writel(readl(&t2_base->devconf0) | MMCSDIO1ADPCLKISEL,	&t2_base->devconf0);
	writel(readl(&t2_base->devconf1) | MMCSDIO2ADPCLKISEL,	&t2_base->devconf1);

	return 1;
}

/*
 * Card IDentification register values to info-structure
 */
void mmc_cid_info(mmc_card_data *mmc_dev_data, mmc_cid_reg_t *cur_cid)
{
	mmc_dev_data->month = cur_cid->mdt & 0x00F;						/* Manufacturing Date */
	mmc_dev_data->year = (((cur_cid->mdt) >> 4) & 0x0FF) + 2000;

	mmc_dev_data->psn = cur_cid->lowpsn | (cur_cid->highpsn << 16);	/* Serial Number of the SD-Card */

	mmc_dev_data->mid = cur_cid->mid;								/* Manufacturer ID */

	mmc_dev_data->oid = cur_cid->oid; 								/* OID */


	mmc_dev_data->prv[0] = (cur_cid->prv) & 0x0000000F;				/* Product revision */
	mmc_dev_data->prv[1] = ((cur_cid->prv) >> 4) & 0x0000000F;

	mmc_dev_data->pnm[0] = ((cur_cid->lowpnm)) & 0x00FF; 			/* Product Name*/
	mmc_dev_data->pnm[1] = ((cur_cid->middlepnm)) & 0x00FF;
	mmc_dev_data->pnm[2] = ((cur_cid->middlepnm) >> 8) & 0x00FF;
	mmc_dev_data->pnm[3] = ((cur_cid->middlepnm) >> 16) & 0x00FF;
	mmc_dev_data->pnm[4] = ((cur_cid->middlepnm) >> 24) & 0x00FF;
	mmc_dev_data->pnm[5] = ((cur_cid->highpnm)) & 0x00FF;
}

/*
 * Send Card initialization stream to SD Card
 */
void mmc_init_stream(void)
{
	// MMCHS Controller INIT Procedure Start
	writel(readl(&mmc_base->con) | INIT_INITSTREAM, &mmc_base->con);		/* set MMCHS_CON[1] INIT to 1*/
	writel(MMC_CMD0, &mmc_base->cmd);										/* send dummy command to start initialize sequence */

	while (!(readl(&mmc_base->stat) & CC_MASK));							/* Wait until command complete */

	writel(CC_MASK, &mmc_base->stat);										/* Repeat procedure */

	writel(MMC_CMD0, &mmc_base->cmd);
	while (!(readl(&mmc_base->stat) & CC_MASK));							/* Wait for command complete bit */

	writel(readl(&mmc_base->con) & ~INIT_INITSTREAM, &mmc_base->con);		/* Disable the Init-Stream bit */
}

/*
 * Configure the clock settings
 */
unsigned char mmc_clock_config(unsigned int iclk, unsigned short clk_div)
{
	unsigned int val;

	mmc_reg_out(&mmc_base->sysctl, (ICE_MASK | DTO_MASK | CEN_MASK),		/* Internal clock enable: Stop internal clock */
			(ICE_STOP | DTO_15THDTO | CEN_DISABLE));						/* Data time counter: TCF x 2²⁷
																	           Disable the Clock, so it can be changed */

	switch (iclk) {															/* Choose clock-rate */
	case CLK_INITSEQ:
		val = MMC_INIT_SEQ_CLK / 2;
		break;
	case CLK_400KHZ:
		val = MMC_400kHz_CLK;
		break;
	case CLK_MISC:
		val = clk_div;
		break;
	default:
		return 0;
	}
	mmc_reg_out(&mmc_base->sysctl, ICE_MASK | CLKD_MASK,					/* Set Clock divider */
			(val << CLKD_OFFSET) | ICE_OSCILLATE);

	while ((readl(&mmc_base->sysctl) & ICS_MASK) == ICS_NOTREADY);

	writel(readl(&mmc_base->sysctl) | CEN_ENABLE, &mmc_base->sysctl);		/* Set Clock enable */
	return 1;
}

/*
 * Do the initial configuration of the MMCHS controller
 */
unsigned char mmc_init_setup(void)
{
	unsigned int reg_val;

	// MMCHS Controller Interface and Functional Clocks Enabling
	writel(readl(CM_ICLKEN1_CORE) | (0x1 << 18) | (0x1 << 19) | (0x1 << 24) | (0x1 << 25) ,	CM_ICLKEN1_CORE); 		/* MMCHS1 + 2 interface clock enabled */
	writel(readl(CM_FCLKEN1_CORE) | (0x1 << 18) | (0x1 << 19) | (0x1 << 24) | (0x1 << 25),	CM_FCLKEN1_CORE); 		/* MMCHS1 + 2 functional clock enabled */

	// MMCH Controller Software Reset
	writel(readl(&mmc_base->sysconfig) | MMC_SOFTRESET,	&mmc_base->sysconfig); /* initiate a soft reset */
	while ((readl(&mmc_base->sysstatus) & RESETDONE) == 0); 				/* wait until softreset is done */

	writel(readl(&mmc_base->sysctl) | SOFTRESETALL, &mmc_base->sysctl); 	/* Software reset for all. This affects
																			   the entire host controller
																			   except card detection unit */
	while ((readl(&mmc_base->sysctl) & SOFTRESETALL) != 0x0);				/* Wait until reset is completed */

	writel(DTW_1_BITMODE | SDBP_PWROFF | SDVS_3V0, &mmc_base->hctl);		/* ToDo */

	// MMCHS Controller Voltage Capailitiers Initialization
	writel(readl(&mmc_base->capa) | VS30_3V0SUP | VS18_1V8SUP, 				/* - initialize voltage-support 1.8 Volt */
			&mmc_base->capa);												/* - initialize voltage-support 3.0 Volt */

	mmc_clock_config(CLK_INITSEQ, 0); 										/* Call Clock initialization method */

	// MMCHS Controller Default Initialization
	writel(DTW_1_BITMODE | SDBP_PWRON | SDVS_3V0, &mmc_base->hctl);			/* - Set Data-Tranfer-Width Bit: a set-bus-width command must follow
																			   - Switch on the bus power / not the card's power
																			   - Set bus voltage to 3.0 Volts */

	reg_val = readl(&mmc_base->con) & RESERVED_MASK;						/* read debounce filter value */

	writel(CTPL_MMC_SD | reg_val | WPP_ACTIVEHIGH | CDP_ACTIVEHIGH |		/* - Disable all the input buffers outside of a transaction */
		MIT_CTO | DW8_1_4BITMODE | MODE_FUNC | STR_BLOCK |					/* - Set write protect polarity to active high */
		HR_NOHOSTRESP | INIT_NOINIT | NOOPENDRAIN, &mmc_base->con);			/* - Set card detect polartiy to active high */
																			/* - Enable command timeout detection
																			 * - Set 1-bit or 4-bit datawidth (not 8-bit)
																			 * - Set functional mode
																			 * - Set Block oriented transfer
																			 * - Disable broadcast host response
																			 * - Disable the host to not send a initialisation sequence
																			 * - Set to Open Drain mode */


	writel(IE_BADA | IE_CERR | IE_DEB | IE_DCRC | IE_DTO | IE_CIE |			/* Configure the interrupt enable register */
		IE_CEB | IE_CCRC | IE_CTO | IE_BRR | IE_BWR | IE_TC | IE_CC,
		&mmc_base->ie);

	mmc_init_stream();														/* Send a initialize stream */

	return 1;																/* Return to caller */
}


/*--------------------------------------------------------------------------
 * Send a command to the memory card and receive response
 */
unsigned char mmc_send_cmd(unsigned int cmd, unsigned int arg,
				unsigned int *response)
{
	unsigned int mmc_stat;

	while ((readl(&mmc_base->pstate) & DATI_MASK) == DATI_CMDDIS);  		/* Wait until issuing of command using mmci_dat lines is allowed */

	writel(BLEN_512BYTESLEN | NBLK_STPCNT, &mmc_base->blk);					/* Configure block size */
	writel(0xFFFFFFFF, &mmc_base->stat);									/* Clear stat - register */
	writel(arg, &mmc_base->arg);											/* Write the command argument to arg-register */
	writel(cmd | CMD_TYPE_NORMAL | CICE_NOCHECK | CCCE_NOCHECK |			/* Write the command to command-register */
		MSBS_SGLEBLK | ACEN_DISABLE | BCE_DISABLE | DE_DISABLE,
		&mmc_base->cmd);

	while (1) {																/* Get Response from Card*/
		do {
			mmc_stat = readl(&mmc_base->stat);								/* Read until a valid stat-value is available */
		} while (mmc_stat == 0);

		if ((mmc_stat & ERRI_MASK) != 0)									/* A error occurred */
			return (unsigned char) mmc_stat;

		if (mmc_stat & CC_MASK) {											/* if command complete bit is set ... */
			writel(CC_MASK, &mmc_base->stat);
			response[0] = readl(&mmc_base->rsp10);							/* read response RSP10 register RSP0 [0:15], RSP1 [16:31] */
			if ((cmd & RSP_TYPE_MASK) == RSP_TYPE_LGHT136) {
				response[1] = readl(&mmc_base->rsp32);						/* read response RSP32 register RSP2 [0:15], RSP3 [16:31] */
				response[2] = readl(&mmc_base->rsp54);						/* read response RSP54 register RSP4 [0:15], RSP5 [16:31] */
				response[3] = readl(&mmc_base->rsp76);						/* read response RSP76 register RSP6 [0:15], RSP7 [16:31] */
			}
			break;
		}
	}
	return 1;																/* return to caller */
}


unsigned char mmc_read_data(unsigned int *output_buf)
{
	unsigned int mmc_stat;
	unsigned int read_count = 0;

	/*
	 * Start Polled Read
	 */
	while (1) {
		do {
			mmc_stat = readl(&mmc_base->stat);
		} while (mmc_stat == 0);

		if ((mmc_stat & ERRI_MASK) != 0)
			return (unsigned char) mmc_stat;

		if (mmc_stat & BRR_MASK) {
			unsigned int k;

			writel(readl(&mmc_base->stat) | BRR_MASK,
				&mmc_base->stat);
			for (k = 0; k < MMCSD_SECTOR_SIZE / 4; k++) {
				*output_buf = readl(&mmc_base->data);
				output_buf++;
				read_count += 4;
			}
		}

		if (mmc_stat & BWR_MASK)
			writel(readl(&mmc_base->stat) | BWR_MASK,
				&mmc_base->stat);

		if (mmc_stat & TC_MASK) {
			writel(readl(&mmc_base->stat) | TC_MASK,
				&mmc_base->stat);
			break;
		}
	}
	return 1;
}

/*
 * Detect the kind of Card to deal with
 */
unsigned char mmc_detect_card(mmc_card_data *mmc_card_cur)
{
	unsigned char err;
	unsigned int argument = 0;
	unsigned int ocr_value, ocr_recvd, ret_cmd41, hcs_val;
	unsigned short retry_cnt = 2000;
	mmc_resp_t mmc_resp;


	// MMCHS Controller Pre-card Identification Configuration
	writel(readl(&mmc_base->hctl) | SDBP_PWRON, &mmc_base->hctl);	/* bus power on*/
	err = mmc_clock_config(CLK_400KHZ, 0);							/* Set to Initialization Clock, that means 400 kHz */
	if (err != 1)
		return err;
	writel(readl(&mmc_base->con) | OPENDRAIN, &mmc_base->con);		/* Set MMC bus mode to open drain */


	mmc_card_cur->RCA = MMC_RELATIVE_CARD_ADDRESS;					/* Set Relative Card Address*/
	argument = 0x00000000;

	ocr_value = (0x1FF << 15);
	err = mmc_send_cmd(MMC_CMD0, argument, mmc_resp.resp);			/* Reset the MMC-Card */
	if (err != 1)
		return err;

	argument = SD_CMD8_CHECK_PATTERN | SD_CMD8_2_7_3_6_V_RANGE;
	err = mmc_send_cmd(MMC_SDCMD8, argument, mmc_resp.resp);		/* Ask SD-Card version 2.x to send its operation conditions */
	hcs_val = (err == 1) ?
		MMC_OCR_REG_HOST_CAPACITY_SUPPORT_SECTOR :
		MMC_OCR_REG_HOST_CAPACITY_SUPPORT_BYTE;						/* Remember if support sector or support byte access */

	argument = 0x0000 << 16;
	err = mmc_send_cmd(MMC_CMD55, argument, mmc_resp.resp);			/* Prevent the card that the following command is going to be an application one*/
	if (err == 1) {													/* This causes an error if you have an SD-Card */
		mmc_card_cur->card_type = SD_CARD;
		ocr_value |= hcs_val;
		ret_cmd41 = MMC_ACMD41;		/* If sended: Asks the accessed SD-Card to send its operating condition register (OCR) content*/
	} else {
		mmc_card_cur->card_type = MMC_CARD;
		ocr_value |= MMC_OCR_REG_ACCESS_MODE_SECTOR;
		ret_cmd41 = MMC_CMD1;
		writel(readl(&mmc_base->con) & ~OD, &mmc_base->con);
		writel(readl(&mmc_base->con) | OPENDRAIN, &mmc_base->con);
	}

	argument = ocr_value;
	err = mmc_send_cmd(ret_cmd41, argument, mmc_resp.resp);			/* Ask SD-Card version to send its operating conditions - the OCR register */
	if (err != 1)
		return err;

	ocr_recvd = mmc_resp.r3.ocr;									/* Copy the receive OCR-value */

	while (!(ocr_recvd & (0x1 << 31)) && (retry_cnt > 0)) 			/* Check if value is valid: Control the Card power status bit, if not repeat ... */
	{
		retry_cnt--;
		if (mmc_card_cur->card_type == SD_CARD) 					/* If we use a SD-Card send CMD55 to indicate ... */
		{															/* ... to the card next command is app specific   */
			argument = 0x0000 << 16;
			err = mmc_send_cmd(MMC_CMD55, argument, mmc_resp.resp);
		}

		argument = ocr_value;
		err = mmc_send_cmd(ret_cmd41, argument, mmc_resp.resp);
		if (err != 1)
			return err;
		ocr_recvd = mmc_resp.r3.ocr;
	}

	if (!(ocr_recvd & (0x1 << 31)))
		return 0;													/* Abort mmc_card_detect() because Card is always busy */

	/* Now we have a valid OCR response */

	if (mmc_card_cur->card_type == MMC_CARD) /* If we use MMC-Cards */
	{
		if ((ocr_recvd & MMC_OCR_REG_ACCESS_MODE_MASK) == MMC_OCR_REG_ACCESS_MODE_SECTOR)
		{
			mmc_card_cur->mode = SECTOR_MODE;
			mmc_card_cur->high_capacity = 1;
		}
		else
		{
			mmc_card_cur->mode = BYTE_MODE;
			mmc_card_cur->high_capacity = 0;
		}

		ocr_recvd &= ~MMC_OCR_REG_ACCESS_MODE_MASK;
	}
	else /* If we use SD-Cards ... */
	{	/* detect if SDSC of SDHC card in card-slot	*/
		if ((ocr_recvd & MMC_OCR_REG_HOST_CAPACITY_SUPPORT_MASK) == MMC_OCR_REG_HOST_CAPACITY_SUPPORT_SECTOR)
		{
			mmc_card_cur->mode = SECTOR_MODE;
			mmc_card_cur->high_capacity = 1;
		} else {
			mmc_card_cur->mode = BYTE_MODE;
			mmc_card_cur->high_capacity = 0;
		}
		ocr_recvd &= ~MMC_OCR_REG_HOST_CAPACITY_SUPPORT_MASK;       /* delete Card Capacity Status in variable */
	}

	ocr_recvd &= ~(0x1 << 31);										/* delete "Card power up status bit" in variable */
	if (!(ocr_recvd & ocr_value))									/* check if we got support voltage range right */
		return 0;

	err = mmc_send_cmd(MMC_CMD2, argument, mmc_resp.resp);			/* request the CID (Card IDentification) register content from card ... */
	if (err != 1)													/* ... and switch to identification state */
		return err;

	mmc_cid_info(mmc_card_cur, (mmc_cid_reg_t *) mmc_resp.resp);						/* Write CID-Info into current device structure */

	if (mmc_card_cur->card_type == MMC_CARD) {
		argument = mmc_card_cur->RCA << 16;
		err = mmc_send_cmd(MMC_CMD3, argument, mmc_resp.resp);
		if (err != 1)
			return err;
	} else {
		argument = 0x00000000;
		err = mmc_send_cmd(MMC_SDCMD3, argument, mmc_resp.resp);	/* request new RCA-address */
		if (err != 1)
			return err;

		mmc_card_cur->RCA = mmc_resp.r6.newpublishedrca;
	}

	writel(readl(&mmc_base->con) & ~OD, &mmc_base->con);
	writel(readl(&mmc_base->con) | NOOPENDRAIN, &mmc_base->con);
	return 1;
}

/*
 * Read cardsize out of the CSD-register information
 */
unsigned char mmc_read_cardsize(mmc_card_data *mmc_dev_data, mmc_csd_reg_t *cur_csd)
{
	mmc_extended_csd_reg_t ext_csd;
	unsigned int csize, cmult, blk_len, blk_no, card_size, argument;
	unsigned char err;
	unsigned int resp[4];



	if (mmc_dev_data->mode == SECTOR_MODE)
	{
		if (mmc_dev_data->card_type == SD_CARD)
		{
			printf("Get high capacity SD-Card size.\n");
			card_size =
				(((mmc_sd2_csd_reg_t *) cur_csd)->c_size_lsb & MMC_SD2_CSD_C_SIZE_LSB_MASK) |
				((((mmc_sd2_csd_reg_t *) cur_csd)->c_size_msb & MMC_SD2_CSD_C_SIZE_MSB_MASK)
				<< MMC_SD2_CSD_C_SIZE_MSB_OFFSET);
			mmc_dev_data->size = card_size * 1024;
			if (mmc_dev_data->size == 0)
				return 0;
		}
		else
		{
			printf("Get high capacity MMC-Card size.\n");
			argument = 0x00000000;
			err = mmc_send_cmd(MMC_CMD8, argument, resp);
			if (err != 1)
				return err;
			err = mmc_read_data((unsigned int *) &ext_csd);
			if (err != 1)
				return err;
			mmc_dev_data->size = ext_csd.sectorcount;

			if (mmc_dev_data->size == 0)
				mmc_dev_data->size = 8388608;
		}
	}
	else
	{
		if (cur_csd->c_size_mult > 7)				/* c_size_mult must be smaller than 8 */
		{
			printf("Error: c_size_mult must be smaller than 8!\n");
			return 0;
		}


		if (cur_csd->read_bl_len > 11)				/* read_bl_len must be smaller then 12 */
		{
			printf("Error: read_bl_len must be smaller then 12!\n");
			return 0;
		}

		/* Compute size */
		cmult = cur_csd->c_size_mult + 2;			/* mult = 2 ^ (c_size_mult + 2) */
		card_size = ((cur_csd->c_size_msb)<< 2)	| (cur_csd->c_size_lsb);
		blk_no = (card_size + 1) << cmult;
		blk_len = 1 << cur_csd->read_bl_len;
		csize = blk_no * blk_len;

		mmc_dev_data->read_bl_len = blk_len;
		mmc_dev_data->size = csize;

		if (mmc_dev_data->size == 0)
		{
			printf("Card size 0 detected");
			return 0;
		}
	}
	return 1;
}

unsigned char omap_mmc_read_sect(unsigned int start_sec, unsigned int num_bytes,
				 mmc_card_data *mmc_c,
				 unsigned long *output_buf)
{
	unsigned char err;
	unsigned int argument;
	unsigned int resp[4];
	unsigned int num_sec_val =
		(num_bytes + (MMCSD_SECTOR_SIZE - 1)) / MMCSD_SECTOR_SIZE;
	unsigned int sec_inc_val;

	if (num_sec_val == 0)
		return 1;

	if (mmc_c->mode == SECTOR_MODE) {
		argument = start_sec;
		sec_inc_val = 1;
	} else {
		argument = start_sec * MMCSD_SECTOR_SIZE;
		sec_inc_val = MMCSD_SECTOR_SIZE;
	}

	while (num_sec_val) {
		err = mmc_send_cmd(MMC_CMD17, argument, resp);
		if (err != 1)
			return err;

		err = mmc_read_data((unsigned int *) output_buf);
		if (err != 1)
			return err;

		output_buf += (MMCSD_SECTOR_SIZE / 4);
		argument += sec_inc_val;
		num_sec_val--;
	}
	return 1;
}

unsigned char configure_mmc(mmc_card_data *mmc_card_cur)
{
	unsigned char ret_val;
	unsigned int argument;
	unsigned int trans_clk, trans_fact, trans_unit, trans_tmp, i, retries = 2;
	unsigned char trans_speed;
	mmc_resp_t mmc_resp;

	ret_val = mmc_init_setup(); 					/* call mmc_init_setup() for initial configuration */

	if (ret_val != 1)
		return ret_val;								/* Only proceed if no errors have been occurred */

	do {
		ret_val = mmc_detect_card(mmc_card_cur); 	/* Try to detect the Card */
		retries--;
	} while ((retries > 0) && (ret_val != 1));

	argument = mmc_card_cur->RCA << 16;
	ret_val = mmc_send_cmd(MMC_CMD9, argument, mmc_resp.resp);		/* Ask the Card to send its CSD register content */
	if (ret_val != 1)
		return ret_val;

	ret_val = mmc_read_cardsize(mmc_card_cur, &mmc_resp.Card_CSD); 	/* Read Card size */
	if (ret_val != 1)
		return ret_val;

	if (mmc_card_cur->card_type == MMC_CARD)
		mmc_card_cur->version = mmc_resp.Card_CSD.spec_vers;  			/* Readout the version of the card */

	trans_speed = mmc_resp.Card_CSD.tran_speed;

	ret_val = mmc_send_cmd(MMC_CMD4, MMC_DSR_DEFAULT << 16, mmc_resp.resp);
	if (ret_val != 1)
		return ret_val;

	trans_unit = trans_speed & MMC_CSD_TRAN_SPEED_UNIT_MASK;
	trans_fact = trans_speed & MMC_CSD_TRAN_SPEED_FACTOR_MASK;

	if (trans_unit > MMC_CSD_TRAN_SPEED_UNIT_100MHZ) 		/* Check for a valid Transfer-Speed-Unit */
	{
		printf("Error: Bad transfer-unit value!\n");
				return 0;
	}

															/* Check for a valid Transfer-Speed-Factor */
	if ((trans_fact < MMC_CSD_TRAN_SPEED_FACTOR_1_0) || (trans_fact > MMC_CSD_TRAN_SPEED_FACTOR_8_0))
	{
		printf("Error: Bad transfer-factor value!\n");
		return 0;
	}

	trans_unit >>= 0;
	trans_fact >>= 3; /* shift right (because of unit-bits) */

		/* calculate transfer-speed for info */
		trans_tmp = 1;				/* Use variable trans_speed because we do not need it anymore */
		for (i=0;i<trans_unit;i++)
		{
			trans_tmp *= 10; 			/* calculate 10 to the power of trans_unit */
		}


		switch (trans_fact)				/* multiply with trans_factor */
		{
		case 1:
			trans_tmp *= 100;
			break;
		case 2:
			trans_tmp *= 120;
			break;
		case 3:
			trans_tmp *= 130;
			break;
		case 4:
			trans_tmp *= 150;
			break;
		case 5:
			trans_tmp *= 200;
			break;
		case 6:
			trans_tmp *= 260;
			break;
		case 7:
			trans_tmp *= 300;
			break;
		case 8:
			trans_tmp *= 350;
			break;
		case 9:
			trans_tmp *= 400;
			break;
		case 10:
			trans_tmp *= 450;
			break;
		case 11:
			trans_tmp *= 520;
			break;
		case 12:
			trans_tmp *= 550;
			break;
		case 13:
			trans_tmp *= 600;
			break;
		case 14:
			trans_tmp *= 700;
			break;
		case 15:
			trans_tmp *= 800;
			break;
		default:
			trans_tmp *= 100;				/* unit is 1 kHz */
		}

		mmc_card_cur->tran_speed = trans_tmp;


	trans_clk = mmc_transspeed_val[trans_fact - 1][trans_unit] * 2;
	ret_val = mmc_clock_config(CLK_MISC, trans_clk); 		/* Set clock frequency according to CSD-value */
	if (ret_val != 1)										/* Check for errors while clock-configuration */
	{
		printf("Error while clock config!\n");
		return ret_val;
	}


	argument = mmc_card_cur->RCA << 16;						/* Select programming state of with RCA addressed card */
	ret_val = mmc_send_cmd(MMC_CMD7_SELECT, argument, mmc_resp.resp);
	if (ret_val != 1)
	{
		printf("Error while selecting programming state of SD-Card!\n");
		return ret_val;
	}


	argument = MMCSD_SECTOR_SIZE;							/* Configure the block length to 512 bytes */
	ret_val = mmc_send_cmd(MMC_CMD16, argument, mmc_resp.resp);
	if (ret_val != 1)										/* Comment: By the use of SDHC-Cards a block-length ... */
	{
		printf("Error: Set sector size does not work!\n");
		return ret_val;										/* ... of 512 bytes is standard and it is not possible to change */
	}

	return 1;
}

/*
 * Block read
 */
unsigned long mmc_bread(int dev_num, unsigned long blknr, lbaint_t blkcnt, void *dst)
{
	omap_mmc_read_sect(blknr, (blkcnt * MMCSD_SECTOR_SIZE), &cur_card_data,
				(unsigned long *) dst);
	return 1;
}

/*
 * initialize and configure a MMC/SD-Card and set properties
 */
int mmc_legacy_init(int verbose) // called if you run: mmc init [dev]
{
	/* Do board initialization */
	if (mmc_board_init() != 1)
		return 1;

	/* Configure mmc/sd card */
	if (configure_mmc(&cur_card_data) != 1)
		return 1;

	mmc_blk_dev.if_type = IF_TYPE_MMC;
	mmc_blk_dev.part_type = PART_TYPE_DOS;
	mmc_blk_dev.dev = 0;
	mmc_blk_dev.lun = 0;
	mmc_blk_dev.type = DEV_TYPE_HARDDISK;
	memcpy(mmc_blk_dev.product, cur_card_data.pnm, 6);

	/* fill in the correct size (is set to 32MByte) */
	mmc_blk_dev.blksz = cur_card_data.read_bl_len;
	mmc_blk_dev.lba = (cur_card_data.size / cur_card_data.read_bl_len);
	mmc_blk_dev.removable = 0;
	mmc_blk_dev.block_read = mmc_bread; 		/* Link to block-read function via function-pointer */

	fat_register_device(&mmc_blk_dev, 1);		/* Register MMC/SD-Card to fat-system */
	return 0;
}
