/*
 * File:	 ehci-omap.c
 *
 * (C) Copyright 2011-2012
 * Ludwig Zenz, DH electronics GmbH , lzenz@dh-electronics.de
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <common.h>
#include <usb.h>
#include <asm/io.h>
#include <asm/arch/gpio.h>
#include <asm/arch/clocks.h>
#include <asm/arch/clocks_omap3.h>
#include <asm/arch/ehci_omap3.h>
#include <asm/arch/sys_proto.h>
#include "ehci-core.h"

/* EHCI Register Set */
#define EHCI_INSNREG04					(0xA0)
#define EHCI_INSNREG04_DISABLE_UNSUSPEND		(1 << 5)
#define	EHCI_INSNREG05_ULPI				(0xA4)
#define	EHCI_INSNREG05_ULPI_CONTROL_SHIFT		31
#define	EHCI_INSNREG05_ULPI_PORTSEL_SHIFT		24
#define	EHCI_INSNREG05_ULPI_OPSEL_SHIFT			22
#define	EHCI_INSNREG05_ULPI_REGADD_SHIFT		16
#define	EHCI_INSNREG05_ULPI_EXTREGADD_SHIFT		8
#define	EHCI_INSNREG05_ULPI_WRDATA_SHIFT		0

#define ULPI_FUNC_CTRL				0x04
#define ULPI_FUNC_CTRL_RESET			(1 << 5)
/*
 * Macros for Set and Clear
 * See ULPI 1.1 specification to find the registers with Set and Clear offsets
 */
#define ULPI_SET(a)				(a + 1)
#define ULPI_CLR(a)				(a + 2)

inline int __board_usb_init(void)
{
       return 0;
}
int board_usb_init(void) __attribute__((weak, alias("__board_usb_init")));

#if defined(CONFIG_OMAP_EHCI_PHY1_RESET_GPIO) || defined(CONFIG_OMAP_EHCI_PHY2_RESET_GPIO)
/* controls PHY(s) reset signal(s) */
static inline void omap_ehci_phy_reset(int on, int delay)
{
       /*
        * Refer ISSUE1:
        * Hold the PHY in RESET for enough time till
        * PHY is settled and ready
        */
       if (delay && !on)
               udelay(delay);
#ifdef CONFIG_OMAP_EHCI_PHY1_RESET_GPIO
       omap_request_gpio(CONFIG_OMAP_EHCI_PHY1_RESET_GPIO);
       omap_set_gpio_direction(CONFIG_OMAP_EHCI_PHY1_RESET_GPIO, 0);
       omap_set_gpio_dataout(CONFIG_OMAP_EHCI_PHY1_RESET_GPIO, !on);
#endif
#ifdef CONFIG_OMAP_EHCI_PHY2_RESET_GPIO
       omap_request_gpio(CONFIG_OMAP_EHCI_PHY2_RESET_GPIO);
       omap_set_gpio_direction(CONFIG_OMAP_EHCI_PHY2_RESET_GPIO, 0);
       omap_set_gpio_dataout(CONFIG_OMAP_EHCI_PHY2_RESET_GPIO, !on);
#endif

       /* Hold the PHY in RESET for enough time till DIR is high */
       /* Refer: ISSUE1 */
       if (delay && on)
               udelay(delay);
}
#else
#define omap_ehci_phy_reset(on, delay) do {} while (0)
#endif

/*
 * Reset is needed otherwise the kernel-driver will throw an error.
 */
int ehci_hcd_stop(void)
{
       debug("Resetting OMAP3 EHCI\n");
       omap_ehci_phy_reset(1, 0);
       writel(OMAP_UHH_SYSCONFIG_SOFTRESET,
                       OMAP3_UHH_BASE + OMAP_UHH_SYSCONFIG);
       /* disable USB clocks */
       struct prcm *prcm_base = (struct prcm *)PRCM_BASE;
       sr32(&prcm_base->iclken_usbhost, 0, 1, 0);
       sr32(&prcm_base->fclken_usbhost, 0, 2, 0);
       sr32(&prcm_base->iclken3_core, 2, 1, 0);
       sr32(&prcm_base->fclken3_core, 2, 1, 0);
       return 0;
}

/*
 * Enable Clock and Power
 */
void ehci_omap_clock_power(void)
{
       struct prcm *prcm_base = (struct prcm *)PRCM_BASE;

       /* Set PLL5 Multiplier and Divider */
       sr32(&prcm_base->clksel4_pll, 0, 7, 0x0C); /* Set Divider */
       sr32(&prcm_base->clksel4_pll, 8, 11, 0x0078); /* Set Multiplier */

       udelay(100);

       /* Frequency selection DPLL5 */
       sr32(&prcm_base->clken2_pll, 4, 4, 0x3);

       udelay(100);

       /* Enable DPLL5 in Lock mode */
       sr32(&prcm_base->clken2_pll, 0, 3, 0x7);

       udelay(100);

       /* Enable USBHOST_L3_ICLK (USBHOST_MICLK) */
       sr32(&prcm_base->iclken_usbhost, 0, 1, 1);
       /*
        * Enable USBHOST_48M_FCLK (USBHOST_FCLK1)
        * and USBHOST_120M_FCLK (USBHOST_FCLK2)
        */
       sr32(&prcm_base->fclken_usbhost, 0, 2, 3); /* ok */

       /* Enable USBTTL_ICLK */
       sr32(&prcm_base->iclken3_core, 2, 1, 1);
       /* Enable USBTTL_FCLK */
       sr32(&prcm_base->fclken3_core, 2, 1, 1);

       /* Enable ST_USBTTL */
       sr32(&prcm_base->idlest3_core, 2, 1, 0);

       udelay(100);

       debug("[USB] clocks enabled\n");
}

/*
 * Perform a reset of the UHH Subsystem
 */
static int omap_uhh_reset(void)
{
	unsigned long init = get_timer(0);

	/* perform UHH soft reset, and wait until reset is complete */
	writel(OMAP_UHH_SYSCONFIG_SOFTRESET, OMAP3_UHH_BASE + OMAP_UHH_HOSTCONFIG);

	/* Wait for UHH reset to complete */
	while (!(readl(OMAP3_UHH_BASE + OMAP_UHH__SYSSTATUS) & OMAP_UHH_SYSSTATUS_EHCI_RESETDONE))
	{
		if (get_timer(init) > CONFIG_SYS_HZ)
		{
			printf("OMAP UHH error: timeout resetting ehci\n");
			return -1;
		}
	}

    debug("[USB] UHH reset done\n");
	return 0;
}

/*
 * Perform a reset of the USBTLL Subsystem
 */
static int omap_ehci_tll_reset(void)
{
	unsigned long init = get_timer(0);

	/* perform TLL soft reset, and wait until reset is complete */
    writel(OMAP_USBTLL_SYSCONFIG_SOFTRESET, OMAP3_USBTLL_BASE + OMAP_USBTLL_SYSCONFIG);

    /* Wait for TLL reset to complete */
    while (!(readl(OMAP3_USBTLL_BASE + OMAP_USBTLL_SYSSTATUS) & OMAP_USBTLL_SYSSTATUS_RESETDONE))
    	if (get_timer(init) > CONFIG_SYS_HZ)
    	{
    		printf("OMAP EHCI error: timeout resetting TLL\n");
    		return -1;
    	}

    debug("[USB] TLL reset done\n");
	return 0;
}



/*
 * Initialize the OMAP3 EHCI controller and PHY.
 * Based on "drivers/usb/host/ehci-omap.c" from Linux 2.6.37.
 */
int ehci_hcd_init(void)
{
       int ret;
       unsigned int reg = 0;

       debug("[USB] Initializing OMAP3 EHCI\n");

       /* Put the PHY in RESET */
       omap_ehci_phy_reset(1, 10);

       udelay(100); /* DIR must be high after reset */

       /* Do usb board initialization */
       ret = board_usb_init();
       if (ret < 0) return ret;

       /* Enable Clock and Power */
       ehci_omap_clock_power();

       reg = ULPI_FUNC_CTRL_RESET
       		// FUNCTION_CTRL_SET register
       		| (ULPI_SET(ULPI_FUNC_CTRL) << EHCI_INSNREG05_ULPI_REGADD_SHIFT)
       		// Write
       		| (2 << EHCI_INSNREG05_ULPI_OPSEL_SHIFT)
       		// PORTn
       		| ((1) << EHCI_INSNREG05_ULPI_PORTSEL_SHIFT)
       		// start ULPI access
       		| (1 << EHCI_INSNREG05_ULPI_CONTROL_SHIFT);

       writel(reg, 0x480648A4);

       // Wait for ULPI access completion
       while (!(readl(0x480648A4) & (1 << EHCI_INSNREG05_ULPI_CONTROL_SHIFT)))
    	   udelay(100);

       /* perform TLL soft reset, and wait until reset is complete */
       omap_ehci_tll_reset();

       writel(OMAP_USBTLL_SYSCONFIG_ENAWAKEUP | OMAP_USBTLL_SYSCONFIG_SIDLEMODE | OMAP_USBTLL_SYSCONFIG_CACTIVITY, OMAP3_USBTLL_BASE + OMAP_USBTLL_SYSCONFIG);

       debug("[USB] USBTLL config done\n");

       /* perform UHH reset, and wait until reset is complete */
       omap_uhh_reset();

       /* Put UHH in NoIdle/NoStandby mode */
       writel(OMAP_UHH_SYSCONFIG_ENAWAKEUP | OMAP_UHH_SYSCONFIG_SIDLEMODE | OMAP_UHH_SYSCONFIG_CACTIVITY | OMAP_UHH_SYSCONFIG_MIDLEMODE | OMAP_UHH_SYSCONFIG_CACTIVITY,
               OMAP3_UHH_BASE + OMAP_UHH_SYSCONFIG);

       debug("UHH sys_config done\n");

       /* setup burst configurations */
       writel(OMAP_UHH_HOSTCONFIG_INCR4_BURST_EN
               | OMAP_UHH_HOSTCONFIG_INCR8_BURST_EN
               | OMAP_UHH_HOSTCONFIG_INCR16_BURST_EN
               | OMAP_UHH_HOSTCONFIG_BYPASS
               | 1 << 8,
               OMAP3_UHH_BASE + OMAP_UHH_HOSTCONFIG);

       debug("UHH setup burst config done\n");

       hccr = (struct ehci_hccr *)(OMAP3_EHCI_BASE);
       hcor = (struct ehci_hcor *)(OMAP3_EHCI_BASE + 0x10);

       /* Get PHY back from RESET */
       omap_ehci_phy_reset(0, 10);

       debug("OMAP3 EHCI init done\n");
       return 0;
}

