/*
 * (C) Copyright 2012
 *
 * Ludwig Zenz , <lzenz@dh-electronics.de>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <config.h>
#include <command.h>
#include <lcd.h>
#include <asm/arch/mux.h>
#include <asm/arch/am35xfb.h>
#include <asm/arch/gpio.h>
#include <asm/io.h>
#include <asm/errno.h>
#include <bmp_layout.h>
#include <nand.h>
#include <dh_settings.h>

DECLARE_GLOBAL_DATA_PTR;

void *lcd_base;				/* Start of framebuffer memory	*/

int lcd_line_length;
int lcd_color_fg;
int lcd_color_bg;

short console_col;
short console_row;


extern void board_set_backlight_pwm(int pwm);

#if  defined (CONFIG_DISPLAY_DATAIMAGE_FG040360DSSWBG03)
	struct omap_panel lcd =
	{	/* DataImage FG040360DSSWBG03 4.3" Display */
		.acbi = 0,
		.acb = 0,
		.is_tft = 1,
		.config = OMAP_DSS_LCD_TFT | OMAP_DSS_LCD_IVS | OMAP_DSS_LCD_IHS | OMAP_DSS_LCD_IPC,
		.timings = {
			.x_res = 480,
			.y_res = 272,

			.pixel_clock	= 9000,

			.hsw		= 42,
			.hfp		= 3,
			.hbp		= 2,

			/* Note: The vertical timings were copied from the Linux sources,
			 *	but they appear to be incorrect.  The first line of data
			 *	is not shown on the display.  Maybe I have a newer LCD?
			 */
			.vsw		= 11,
			.vfp		= 3,
			.vbp		= 2,
		},

		.data_lines = 24,

		.fb_address1 = 0x805CB000, /* Default value which has to be replace with real buffer reference */
		.fb_address2 = 0x805CB000, /* Default value which has to be replace with real buffer reference  */
	};
#else
	/* No display is defined - data from dh_settings_block is used */
	struct omap_panel lcd;
#endif


/*
 *  initialize panel_info structure for u-boot lcd system with default values
 */
vidinfo_t panel_info = {
		.vl_col		= 480,
		.vl_row		= 272,
#if defined(CONFIG_RGB16_FRAMEBUFFER)
		.vl_bpix	= 16,
#elif  defined(CONFIG_RGB24_FRAMEBUFFER)
		.vl_bpix	= 24,
#endif
	};

/* -------------------------------------------------------------------
 * Load Values from dh_board_settings into omap_panel struct
 */
void lcd_read_dh_board_settings(struct omap_panel *lcd)
{

	lcd->timings.y_res = gd->dh_board_settings.wYResolution;
	panel_info.vl_row = gd->dh_board_settings.wYResolution;

	lcd->timings.x_res = gd->dh_board_settings.wXResolution;
	panel_info.vl_col = gd->dh_board_settings.wXResolution;

    lcd->timings.pixel_clock = gd->dh_board_settings.wPixelClock;
    lcd->timings.vsw = gd->dh_board_settings.wVPulseWidth;
    lcd->timings.hsw = gd->dh_board_settings.wHPulseWidth;
    lcd->timings.hbp = gd->dh_board_settings.wHBackPorch;
    lcd->timings.hfp = gd->dh_board_settings.wHFrontPorch;
    lcd->timings.vbp = gd->dh_board_settings.wVBackPorch;
    lcd->timings.vfp = gd->dh_board_settings.wVFrontPorch;

    lcd->acbi = gd->dh_board_settings.cACBiasTrans;
    lcd->acb = gd->dh_board_settings.cACBiasFreq;

    lcd->data_lines = gd->dh_board_settings.cDatalines;

    /* Clear all configuration flags*/
    lcd->config = 0;

    /* Set invert vertical sync flag */
    if(gd->dh_board_settings.wLCDConfigFlags & SETTINGS_LCD_IVS_FLAG) { lcd->config |= OMAP_DSS_LCD_IVS; }

    /* Set invert horizontal sync flag */
    if(gd->dh_board_settings.wLCDConfigFlags & SETTINGS_LCD_IHS_FLAG) { lcd->config |= OMAP_DSS_LCD_IHS; }

    /* Set invert pixel-clock flag */
    if(gd->dh_board_settings.wLCDConfigFlags & SETTINGS_LCD_IPC_FLAG)
    {
    	/* do nothing */
    }
    else { lcd->config |= OMAP_DSS_LCD_IPC; }

    /* Set invert enable output flag */
    if(gd->dh_board_settings.wLCDConfigFlags & SETTINGS_LCD_IOE_FLAG) { lcd->config |= OMAP_DSS_LCD_IEO; }

    /* Set invert data flag */
    if(gd->dh_board_settings.wLCDConfigFlags & SETTINGS_LCD_IDATA_FLAG) { lcd->config |= OMAP_DSS_LCD_IDATA; }

    /* Set active matrix flag */
    if(gd->dh_board_settings.wLCDConfigFlags & SETTINGS_LCD_ACT_PAS_FLAG)
    {
    	lcd->is_tft = 1;
    	lcd->config |= OMAP_DSS_LCD_TFT;
    }
    else { lcd->is_tft = 0; } /* set passive matrix */

    /* Set polarity of pwm */
    if(gd->dh_board_settings.wLCDConfigFlags & SETTINGS_LCD_PWM_POL_FLAG) { lcd->config |= OMAP_DSS_LCD_PWM_POL; }

    /* Set inert backlight enable flag */
    if(gd->dh_board_settings.wLCDConfigFlags & SETTINGS_LCD_IBL_FLAG) { lcd->config |= OMAP_DSS_LCD_IBL; }


}

/* -------------------------------------------------------------------
 * Enable LCD display controller module
 */
void omapdss_lcd_enable(struct omap_panel *lcd)
{
	struct dispc *dispc_base 	= (struct dispc*)DISPC_BASE;
	enum omap_tft_datalines tft_datalines = TFTDATALINES_24;

	udelay(2000);

	// LZ: 29.05.2012
	// Use always a 24 datalines configuration
	// If not all datalines are available/used the least significant bits run open.

	// Set correct count of datalines
	//if (lcd->data_lines == 12) {tft_datalines = TFTDATALINES_12;}
	//if (lcd->data_lines == 16) {tft_datalines = TFTDATALINES_16;}
	//if (lcd->data_lines == 18) {tft_datalines = TFTDATALINES_18;}
	//if (lcd->data_lines == 24) {tft_datalines = TFTDATALINES_24;}

	// Enable the LCD Display
	dispc_base->control		= 0x00018068 | tft_datalines | OMAP_LCDENABLE;		// TFTDATALINES | GOLCD | TFT | LCDENABLE

	/* Disable the unused Datalines */
	/* Note: The MSB of the datalines are always used but the cpu is disabling this lines *
	 *		 So we're building modes with less datalines ourself when we configure the unused
	 * 		 datalines to gpio inputs */
	if (lcd->data_lines == 18)
	{
		MUX_VAL(CP(DSS_DATA0),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_B0     */
		MUX_VAL(CP(DSS_DATA1),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_B1     */
		MUX_VAL(CP(DSS_DATA8),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_G0     */
		MUX_VAL(CP(DSS_DATA9),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_G1     */
		MUX_VAL(CP(DSS_DATA16),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_R0     */
		MUX_VAL(CP(DSS_DATA17),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_R1     */
	}

	if (lcd->data_lines == 16)
	{
		MUX_VAL(CP(DSS_DATA0),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_B0     */
		MUX_VAL(CP(DSS_DATA1),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_B1     */
		MUX_VAL(CP(DSS_DATA2),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_B2     */
		MUX_VAL(CP(DSS_DATA8),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_G0     */
		MUX_VAL(CP(DSS_DATA9),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_G1     */
		MUX_VAL(CP(DSS_DATA16),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_R0     */
		MUX_VAL(CP(DSS_DATA17),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_R1     */
		MUX_VAL(CP(DSS_DATA18),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_R2     */
	}

	if (lcd->data_lines == 12)
	{
		MUX_VAL(CP(DSS_DATA0),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_B0     */
		MUX_VAL(CP(DSS_DATA1),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_B1     */
		MUX_VAL(CP(DSS_DATA2),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_B2     */
		MUX_VAL(CP(DSS_DATA3),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_B3     */
		MUX_VAL(CP(DSS_DATA8),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_G0     */
		MUX_VAL(CP(DSS_DATA9),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_G1     */
		MUX_VAL(CP(DSS_DATA10),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_G2     */
		MUX_VAL(CP(DSS_DATA11),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_G3     */
		MUX_VAL(CP(DSS_DATA16),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_R0     */
		MUX_VAL(CP(DSS_DATA17),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_R1     */
		MUX_VAL(CP(DSS_DATA18),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_R2     */
		MUX_VAL(CP(DSS_DATA18),		(IDIS | PTD | DIS | M4)); /* DHCOM: LC_R3     */
	}

	/* Try to detect GFX FIFO UNDERFLOW */
	if (dispc_base->irqstatus & GFX_FIFO_UNDERFLOW)
	{
		printf("[LCD] ERROR: GFX Fifo underflow occured!!\n");
	}

}

void omapdss_set_pol_freq(enum omap_panel_config config, unsigned char acbi, unsigned char acb)
{
	struct dispc *dispc_base = (struct dispc*)DISPC_BASE;
	unsigned int l = 0;

	debug("[LCD] set_pol_freq() called\n");

	l |= (config & OMAP_DSS_LCD_ONOFF) ? (1<<17) : 0;
	l |= (config & OMAP_DSS_LCD_RF) ? (1<<16) : 0;
	l |= (config & OMAP_DSS_LCD_IEO) ? (1<<15) : 0;
	l |= (config & OMAP_DSS_LCD_IPC) ? (1<<14) : 0;
	l |= (config & OMAP_DSS_LCD_IHS) ? (1<<13) : 0;
	l |= (config & OMAP_DSS_LCD_IVS) ? (1<<12) : 0;
	l |= ((acbi & 0x0F) << 8);
	l |= acb;

	dispc_base->pol_freq = l;
	udelay(1000);
	dispc_base->pol_freq = l;
	udelay(1000);
}

void omapdss_set_lcd_timings(struct omap_video_timings *timings, unsigned char is_tft)
{
	struct dispc *dispc_base = (struct dispc*)DISPC_BASE;


	debug("[LCD] set_lcd_timings() called\n");


	// Calculate the DSS clocks in kHz (using DSS1_ALWON_FCL)
	unsigned int sys_clock = 13;
	unsigned int dpll4_mult 	= (*((unsigned int*)0x48004D44) >> 8) & 0x7FF;		// CM_CLKSEL2_PLL - 0x1B0 (432)
	unsigned int dpll4_div 		= *((unsigned int*)0x48004D44) & 0x7F;				// CM_CLKSEL2_PLL - 0xC (12)
	unsigned int dpll4_clock 	= sys_clock * 2 * dpll4_mult / (dpll4_div + 1);		// AM3517: 864 MHz

	// Adjust DPLL4 clock to kHz so we get better precision for the LCD Clock
	dpll4_clock *= 1000;

	unsigned char found = 0;
	unsigned int dss1_div = 5;
	unsigned int p_div = 1;
	unsigned int l_div = is_tft ? 2 : 3;

	while((dss1_div < 0x10) && !found)
	{
		p_div = dpll4_clock / dss1_div / l_div / timings->pixel_clock;

		if(p_div <= 255)
		{
			found = 1;
			break;
		}

		dss1_div++;
	}

	// Write the DSS Clock Divider, Pixel Clock Divider, and Logic Clock Divider
	*((unsigned int *)0x48004E40) = dss1_div;
	dispc_base->divisor = (l_div << 16) | (p_div);

	// Calculate and set the panel H/V register values
	dispc_base->timing_h = FLD_VAL(timings->hsw, 5, 0) | FLD_VAL(timings->hfp-1, 15, 8) | FLD_VAL(timings->hbp-1, 27, 20);
	dispc_base->timing_v = FLD_VAL(timings->vsw, 5, 0) | FLD_VAL(timings->vfp, 15, 8) | FLD_VAL(timings->vbp, 27, 20);

	/* Set the LCD size values */
	dispc_base->size_lcd = FLD_VAL(timings->y_res - 1, 26, 16) | FLD_VAL(timings->x_res - 1, 10, 0);
}


void lcd_initcolregs(void)
{
}

void lcd_setcolreg(ushort regno, ushort red, ushort green, ushort blue)
{
}

/*
 * Enable backlight power supply and backlight pwm
 */
void lcd_enable (void)
{

	debug("[LCD] lcd_enable() called\n");

	/* if we work with LG 4.3" Display LG043WV2-SD01 execute SPI initialization */
	if (gd->dh_board_settings.cDisplayID == 0x05)
	{
		DISABLE_PRINTF()

	#ifdef UNPROGRAMMED_PIC  /* normaly PIC should execute the power-up sequence */
		run_command("testdh congpio 116",0); /*GPIO A - RESET */
		run_command("testdh congpio 119",0); /*GPIO D - 2.8V - VCI/VCC*/
		run_command("testdh congpio 140",0); /*GPIO F - 1.8V - IOVCC*/

		run_command("testdh setgpio 116 0",0); /*GPIO A - RESET */
		run_command("testdh setgpio 119 0",0); /*GPIO D - 2.8V - VCI/VCC*/
		run_command("testdh setgpio 140 0",0); /*GPIO F - 1.8V - IOVCC*/
	#endif

		/* configure spi interface */
		run_command("spiconf 1 500000 3",0);

	#ifdef UNPROGRAMMED_PIC /* normaly PIC should execute the power-up sequence */
		udelay(30000);

		run_command("testdh setgpio 140 1",1); /*GPIO F - 1.8V - IOVCC*/

		udelay(20000);

		run_command("testdh setgpio 119 1",1); /*GPIO D - 2.8V - VCI/VCC*/

		udelay(30000);

		run_command("testdh setgpio 119 0",1); /*GPIO D - 2.8V - VCI/VCC*/

		udelay(30000);

		run_command("testdh setgpio 119 1",1); /*GPIO D - 2.8V - VCI/VCC*/

		udelay(20000);

		/* set reset to high */
		run_command("testdh setgpio 116 1",0); /*GPIO A - RESET */

		//udelay(20000);
		udelay(20000);
	#endif

		/*------------------------------------------------------------*/
		/* display softreset ... */
		run_command("spi 1 16 7000",0);
		run_command("spi 1 16 7001",0);
		udelay(1000);

		/*------------------------------------------------------------*/
		printf("display inversion off ...\n");
		run_command("spi 1 16 7020",0);

		/*------------------------------------------------------------*/
		printf("set interface pixel format (COLMOD) ...\n");
		run_command("spi 1 16 703a",0);
		run_command("spi 1 16 7270",0);

		/*------------------------------------------------------------*/
		printf("RGB interface setting ...\n");
		run_command("spi 1 16 70b1",0);
		run_command("spi 1 32 72011E0C",0);

		/*------------------------------------------------------------*/
		printf("panel characteristics settings\n");
		run_command("spi 1 16 70b2",0);
		run_command("spi 1 24 7200c8",0);

		/*------------------------------------------------------------*/
		printf("panel drive settings\n");
		run_command("spi 1 16 70b3",0);
		run_command("spi 1 16 7200",0);

		/*------------------------------------------------------------*/
		printf("display mode control\n");
		run_command("spi 1 16 70b4",0);
		run_command("spi 1 16 7204",0);

		/*------------------------------------------------------------*/
		printf("display control 1\n");
		run_command("spi 1 16 70b5",0);
		run_command("spi 1 48 720110100020",0);

		/*------------------------------------------------------------*/
		printf("display control 2\n");
		run_command("spi 1 16 70b6",0);
		run_command("spi 1 56 720B0F3C1313e8",0);

		/*------------------------------------------------------------*/
		printf("display control 3\n");
		run_command("spi 1 16 70b7",0);
		run_command("spi 1 48 7246060c0000",0);

		/*------------------------------------------------------------*/
		printf("internal oscillator setting\n");
		run_command("spi 1 16 70c0",0);
		run_command("spi 1 24 720111",0);

		/*------------------------------------------------------------*/
		printf("power control 3\n");
		run_command("spi 1 16 70c3",0);
		run_command("spi 1 56 720703040404",0);

		/*------------------------------------------------------------*/
		printf("power control 4\n");
		run_command("spi 1 16 70c4",0);
		run_command("spi 1 56 72122418180249",0);

		/*------------------------------------------------------------*/
		printf("power control 5\n");
		run_command("spi 1 16 70c5",0);
		run_command("spi 1 16 726F",0);

		/*------------------------------------------------------------*/
		printf("power control 6\n");
		run_command("spi 1 16 70c6",0);
		run_command("spi 1 24 724163",0);

		/*------------------------------------------------------------*/
		/*Gamma Setting */
		printf("gamma setting \n");
		run_command("spi 1 16 70d0",0);
		run_command("spi 1 80 72030773350001200003",0);

		run_command("spi 1 16 70d2",0);
		run_command("spi 1 80 72030773350001200003",0);

		run_command("spi 1 16 70d4",0);
		run_command("spi 1 80 72030773350001200003",0);

		run_command("spi 1 16 70d1",0);
		run_command("spi 1 80 72030773350001200003",0);

		run_command("spi 1 16 70d3",0);
		run_command("spi 1 80 72030773350001200003",0);

		run_command("spi 1 16 70d5",0);
		run_command("spi 1 80 72030773350001200003",0);
		SESSION_DEPENDED_PRINTF_ENABLE()
	}

	/* Activate Backlight PWM */
	board_set_backlight_pwm(100);

	/* Enable the LCD Display */
	omapdss_lcd_enable(&lcd);

	/* if we work with LG 4.3" Display LG043WV2-SD01 execute SPI initialization */
	if (gd->dh_board_settings.cDisplayID == 0x05)
	{
		DISABLE_PRINTF()

		/* wait a few frames */
		udelay(2000);

		printf("sleep out\n");
		run_command("spi 1 16 7011",0);

		printf("display on\n");
		run_command("spi 1 16 7029",0);

		SESSION_DEPENDED_PRINTF_ENABLE()
	}

}

/*
 * disable the backlight and LCD controller
 */
void lcd_disable(void)
{
	#ifdef DEBUG
	printf("[LCD] lcd_disable() called\n");
	#endif

	struct dispc *dispc_base 	= (struct dispc*)DISPC_BASE;

	/* deactivate Backlight PWM */
	board_set_backlight_pwm(0);

	// Disable the LCD Display
	dispc_base->control		= 0x00018068;		// TFTDATALINES | !GOLCD | !TFT | !LCDENABLE
}

void lcd_panel_disable(void)
{
}

void lcd_ctrl_init (void *lcdbase)
{
	#ifdef DEBUG
	printf("[LCD] lcd_ctrl_init() called\n");
	#endif

	uint pixelbytes = 0;

	struct dss *dss_base		= (struct dss*)DSS_BASE;
	struct dispc *dispc_base 	= (struct dispc*)DISPC_BASE;

#ifdef CONFIG_DHCOM
	/* Load Values from dh_board_settings into omap_panel struct */
	lcd_read_dh_board_settings(&lcd);
#endif

	// set bytes per pixel
	pixelbytes = BYTES_PER_PIXEL;

	/* set line length */
	lcd_line_length = (lcd.timings.x_res * pixelbytes);

	/* Set base-address of frame-buffer */
	lcd.fb_address1 = (unsigned int)lcdbase;
	lcd.fb_address2 = (unsigned int)lcdbase;

	// Setup LCD Interface IO and Clocks
	*((uint*) 0x48310034)			= 0xBFFFFFFF;	// GPIO1: GPIO_OE
	*((uint*) 0x48310094)			= 0x40000000;	// GPIO1: GPIO_SETDATAOUT
	*((uint*) 0x48004D44)			= 0x0001B00C;	// CM: Clock_Control_Reg_CM: CM_CLKSEL2_PLL
	*((uint*) 0x48004E40)			= 0x00001006;	// CM: DSS_CM: CM_CLKSEL_DSS
	*((uint*) 0x48004D00)			= 0x00370037;	// CM: Clock_Control_Reg_CM: CM_CLKEN_PLL

	udelay(2000);	// Wait a bit to let the DSS clocks stabilize and lock

	// Display Sub System (DSS) Initialization
	dss_base->sysconfig				= 0x00000001;	// Enable Auto-Idle
	dss_base->control				= 0x00000078;	// Settings for various clocks (was 0x00000078)
	dss_base->irqstatus				= 0x00000000;

	// Perform Softreset of Display Subsystem
	dss_base->sysconfig				= 0x00000003;	// set softreset
	while(dss_base->sysconfig & 0x000000020);		// wait until softreset is done

	dss_base->sdi_control			= 0x00000000;
	dss_base->pll_control			= 0x00000000;

	// Display Controller (DISPC) and Graphics Overlay (GFX) Initialization
	dispc_base->sysconfig			= 0x00002015;	// SMART_STANDBY | NO_IDLE | ENWAKEUP | AUTOIDLE
	dispc_base->config				= 0x00000004;	// LOADMODE = FRAME_ONLY
	dispc_base->default_color[0]	= 0x0000FF00;	// LCD Default Color (24-bit)
	dispc_base->default_color[1]	= 0x00FF0000;	// DIG Default Color (24-bit)
	dispc_base->trans_color[0]		= 0x00000000;
	dispc_base->trans_color[1]		= 0x00000000;


	// Set LCD Parameters (Polarity Bits, Size, Timings)
	omapdss_set_pol_freq(lcd.config, lcd.acbi, lcd.acb);
	omapdss_set_lcd_timings(&lcd.timings, lcd.is_tft);	// Set the panel timings and size

	dispc_base->gfx_ba[0]			= lcd.fb_address1;		// Frame buffer address
	dispc_base->gfx_ba[1]			= lcd.fb_address2;		// Double buffer address (same)
	dispc_base->gfx_position		= 0x00000000;			// GFX Window Position on Screen
	dispc_base->gfx_size			= dispc_base->size_lcd;	// Size of GFX Window (same as LCD Size)

	/* RGB16 configuration: R5 G6 B5 */
	if (pixelbytes == 2) { dispc_base->gfx_attributes = 0x0000008d/*0x000040Ad*/;} // GFXBURSTSIZE = 16x32bit | GFXFORMAT = RGB16 | GFX_ENABLE;
	/* RGB24 configuration: R8 G8 B8 */
	if (pixelbytes == 3) { dispc_base->gfx_attributes = 0x00000093;} // GFXBURSTSIZE = 16x32bit | GFXFORMAT = RGB24 | GFX_ENABLE;

	//dispc_base->gfx_fifo_threshold	= 0x03ff03c0;			// Not sure how to customize this...

	dispc_base->gfx_row_inc			= 1; // Pixel to jump for new row
	dispc_base->gfx_pixel_inc		= 1;

	dispc_base->gfx_window_skip		= 0x00000000;			// Don't skip windows
	dispc_base->gfx_table_ba		= 0x00000000;			// Not using a graphics table

	dispc_base->gfx_preload = 0x00000060;

	udelay(2000);

	#ifdef DEBUG
	printf("[LCD] lcd_ctrl_init() finished\n");
	#endif
}

/*
 * Reinit frame-buffer and lcd settings
 *
 * If bmp format has changed e.g. from RGB24 to RGB16, you have to run this.
 */
void lcd_reinit_to_new_bmpformat(uchar *bmp_header)
{
	int index;
	uchar *fb;
	bmp_image_t *bmp= (bmp_image_t *)bmp_header;

	/* get pointer to frame-buffer */
	fb   = (uchar *) lcd_base;

	if (fb != bmp_header )
	{
		#ifdef DEBUG
		printf("[LCD] Copying bmp header... \n");
		#endif

		for (index = 0; index < bmp->header.data_offset; index++)
		{
				fb[index] = bmp_header[index];
		}

	}

	/* adjust framebuffer and lcd settings to RGB xx (bmp-header data required) */
	lcd_disable();
	lcd_ctrl_init(lcd_base);
	lcd_enable();

	/* update line length */
	lcd_line_length = (panel_info.vl_col * panel_info.vl_bpix) / 8;

}

ulong calc_fbsize(void)
{
	return (gd->dh_board_settings.wYResolution * gd->dh_board_settings.wXResolution * BYTES_PER_PIXEL );
}


