/*
 * (C) Copyright 2006-2008
 * Texas Instruments, <www.ti.com>
 * Richard Woodruff <r-woodruff2@ti.com>
 * Syed Mohammed Khasim <x0khasim@ti.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef _OMAP3_H_
#define _OMAP3_H_

/* Stuff on L3 Interconnect */
#define SMX_APE_BASE			0x68000000

/* GPMC */
#define OMAP34XX_GPMC_BASE		0x6E000000

/* SMS */
#define OMAP34XX_SMS_BASE		0x6C000000

/* SDRC */
#define OMAP34XX_SDRC_BASE		0x6D000000

/*
 * L4 Peripherals - L4 Wakeup and L4 Core now
 */
#define OMAP34XX_CORE_L4_IO_BASE	0x48000000
#define OMAP34XX_WAKEUP_L4_IO_BASE	0x48300000
#define OMAP34XX_ID_L4_IO_BASE		0x4830A200
#define OMAP34XX_L4_PER				0x49000000
#define OMAP34XX_L4_IO_BASE			OMAP34XX_CORE_L4_IO_BASE

/* DMA4/SDMA */
#define OMAP34XX_DMA4_BASE          0x48056000

/* CONTROL */
#define OMAP34XX_CTRL_BASE			(OMAP34XX_L4_IO_BASE + 0x2000)

/* UART */
#define OMAP34XX_UART1				(OMAP34XX_L4_IO_BASE + 0x6a000)
#define OMAP34XX_UART2				(OMAP34XX_L4_IO_BASE + 0x6c000)
#define OMAP34XX_UART3				(OMAP34XX_L4_PER + 0x20000)

/* General Purpose Timers */
#define OMAP34XX_GPT1			0x48318000
#define OMAP34XX_GPT2			0x49032000
#define OMAP34XX_GPT3			0x49034000
#define OMAP34XX_GPT4			0x49036000
#define OMAP34XX_GPT5			0x49038000
#define OMAP34XX_GPT6			0x4903A000
#define OMAP34XX_GPT7			0x4903C000
#define OMAP34XX_GPT8			0x4903E000
#define OMAP34XX_GPT9			0x49040000
#define OMAP34XX_GPT10			0x48086000
#define OMAP34XX_GPT11			0x48088000
#define OMAP34XX_GPT12			0x48304000

/* WatchDog Timers (1 secure, 3 GP) */
#define WD1_BASE				0x4830C000
#define WD2_BASE				0x48314000
#define WD3_BASE				0x49030000

/* 32KTIMER */
#define SYNC_32KTIMER_BASE		0x48320000

#ifndef __ASSEMBLY__

struct s32ktimer {
	unsigned char res[0x10];
	unsigned int s32k_cr;		/* 0x10 */
};

#endif /* __ASSEMBLY__ */

/*
 * OMAP3 GPIO registers
 */

/* Base addresses of 4K bytes*/
#define OMAP34XX_GPIO1_BASE			0x48310000
#define OMAP34XX_GPIO2_BASE			0x49050000
#define OMAP34XX_GPIO3_BASE			0x49052000
#define OMAP34XX_GPIO4_BASE			0x49054000
#define OMAP34XX_GPIO5_BASE			0x49056000
#define OMAP34XX_GPIO6_BASE			0x49058000

#if defined(OMAP34XX_GPIO1_BASE) || defined(OMAP34XX_GPIO2_BASE) || defined(OMAP34XX_GPIO3_BASE) || defined(OMAP34XX_GPIO4_BASE) || defined(OMAP34XX_GPIO5_BASE) || defined(OMAP34XX_GPIO6_BASE)
/*Register-Mapping Offsets Summery GPIO1 to GPIO6*/
#define OFFSET_GPIO_REVISION		0x00000000
#define OFFSET_GPIO_SYSCONFIG		0x00000010
#define OFFSET_GPIO_SYSSTATUS		0x00000014
#define OFFSET_GPIO_IRQSTATUS1		0x00000018
#define OFFSET_GPIO_IRQENABLE1		0x0000001C
#define OFFSET_GPIO_WAKEUPENABLE 	0x00000020
#define OFFSET_GPIO_IRQSTATUS2		0x00000028
#define OFFSET_GPIO_IRQENABLE2		0x0000002C
#define OFFSET_GPIO_CTRL			0x00000030
#define OFFSET_GPIO_OE				0x00000034
#define OFFSET_GPIO_DATAIN			0x00000038
#define OFFSET_GPIO_DATAOUT			0x0000003C
#define OFFSET_GPIO_LEVELDETECT0	0x00000040
#define OFFSET_GPIO_LEVELDETECT1	0x00000044
#define OFFSET_GPIO_RISINGDETECT	0x00000048
#define OFFSET_GPIO_FALLINGDETECT	0x0000004C
#define OFFSET_GPIO_DEBOUNCENABLE	0x00000050
#define OFFSET_GPIO_DEBOUNCINGTIME	0x00000054
#define OFFSET_GPIO_CLEARIRQENABLE1	0x00000060
#define OFFSET_GPIO_SETIRQENABLE1	0x00000064
#define OFFSET_GPIO_CLEARIRQENABLE2	0x00000070
#define OFFSET_GPIO_SETIRQENABLE2	0x00000074
#define OFFSET_GPIO_CLEARWKUENA		0x00000080
#define OFFSET_GPIO_SETWKUENA		0x00000084
#define OFFSET_GPIO_CLEARDATAOUT	0x00000090
#define OFFSET_GPIO_SETDATAOUT		0x00000094
#endif

/* Clock Control Registers */
#define OMAP34XX_PRMC_CCR_CM_BASE	0x48004D00
#ifdef OMAP34XX_PRMC_CCR_CM_BASE
#define OFFSET_CM_CLKEN_PLL			0x00000000
#define OFFSET_CM_CLKEN2_PLL		0x00000004
#define OFFSET_CM_IDLEST_CKGEN		0x00000020
#define OFFSET_CM_IDLEST2_CKGEN		0x00000024
#define OFFSET_CM_AUTOIDLE_PLL		0x00000030
#define OFFSET_CM_AUTOIDLE2_PLL		0x00000034
#define OFFSET_CM_CLKSEL1_PLL		0x00000040
#define OFFSET_CM_CLKSEL2_PLL		0x00000044
#define OFFSET_CM_CLKSEL3_PLL		0x00000048
#define OFFSET_CM_CLKSEL4_PLL		0x0000004C
#define OFFSET_CM_CLKSEL5_PLL		0x00000050
#define OFFSET_CM_CLKOUT_CTRL		0x00000070

#define MUX_VAL_CCR(OFFSET,VALUE)\
	writel((VALUE), OMAP34XX_PRMC_CCR_CM_BASE + (OFFSET));
#endif



/* DSS_Clock_Management */
#define OMAP34xx_PRMC_DSS_CM_BASE	0x48004E00
#ifdef OMAP34xx_PRMC_DSS_CM_BASE
#define OFFSET_CM_FCLKEN_DSS		0x00000000
#define OFFSET_CM_ICLKEN_DSS		0x00000010
#define OFFSET_CM_IDLEST_DSS		0x00000020
#define OFFSET_CM_AUTOIDLE_DSS		0x00000030
#define OFFSET_CM_CLKSEL_DSS		0x00000040
#define OFFSET_CM_SLEEPDEP_DSS		0x00000044
#define OFFSET_CM_CLKSTCTRL_DSS		0x00000048
#define OFFSET_CM_CLKSTST_DSS		0x0000004C

#define MUX_VAL_DSS_CM(OFFSET,VALUE)\
	writel((VALUE), OMAP34xx_PRMC_DSS_CM_BASE + (OFFSET));
#endif

/*
 * Display Subsystem
 */
#define OMAP34XX_DISP_SUB_BASE		0x48050000
#ifdef OMAP34XX_DISP_SUB_BASE
#define OFFSET_DSS_REVISIONNUMBER	0x00000000
#define OFFSET_DSS_SYSCONFIG		0x00000010
#define OFFSET_DSS_SYSSTATUS		0x00000014
#define OFFSET_DSS_IRQSTAUS			0x00000018
#define OFFSET_DSS_CONTROL			0x00000040
#define OFFSET_DSS_SDI_CONTROL		0x00000044
#define OFFSET_DSS_PLL_CONTROL		0x00000048
#define OFFSET_DSS_SDI_STATUS		0x0000005C

#define MUX_VAL_DSS(OFFSET, VALUE)\
	writel((VALUE), OMAP34XX_DISP_SUB_BASE + (OFFSET));
#endif


/* Video Encoder - Display Subsystem */
#define OMAP34XX_DISP_SUB_VENC_BASE	0x48050C00 // 256 bytes
#ifdef OMAP34XX_DISP_SUB_VENC_BASE
#define OFFSET_VENC_REV_ID			0x00000000
#define OFFSET_VENC_STATUS			0x00000004
#define OFFSET_VENC_F_CONTROL		0x00000008
#define OFFSET_VENC_VIDOUT_CTRL		0x00000010
#define OFFSET_VENC_SYNC_CTRL		0x00000014
#define OFFSET_VENC_LLEN			0x0000001C
#define OFFSET_VENC_FLENS			0x00000020
#define OFFSET_VENC_HFLTR_CTRL		0x00000024
#define OFFSET_VENC_CC_CARR_WSS		0x00000028
#define OFFSET_VENC_C_PHASE			0x0000002C
#define OFFSET_VENC_GAIN_U			0x00000030
#define OFFSET_VENC_GAIN_V			0x00000034
#define OFFSET_VENC_GAIN_Y			0x00000038
#define OFFSET_VENC_BLACK_LEVEL		0x0000003C
#define OFFSET_VENC_BLANK_LEVEL		0x00000040
#define OFFSET_VENC_X_COLOR			0x00000044
#define OFFSET_VENC_M_CONTROL		0x00000048
#define OFFSET_VENC_BSTAMP_WSS_DATA	0x0000004C
#define OFFSET_VENC_S_CARR			0x00000050
#define OFFSET_VENC_LINE21			0x00000054
#define OFFSET_VENC_LN_SEL			0x00000058
#define OFFSET_VENC_L21_WC_CTL		0x0000005C
#define OFFSET_VENC_HTRIG_VTRIG		0x00000060
#define OFFSET_VENC_SAVID_EAVID		0x00000064
#define OFFSET_VENC_FLEN_FAL		0x00000068
#define OFFSET_VENC_LAL_PHASE_RESET	0x0000006C
#define OFFSET_VENC_HS_INT_START_STOP_X				0x00000070
#define OFFSET_VENC_HS_EXT_START_STOP_X				0x00000074
#define OFFSET_VENC_VS_INT_START_X					0x00000078
#define OFFSET_VENC_VS_INT_STOP_X_VS_INT_START_Y	0x0000007C
#define OFFSET_VENC_VS_INT_STOP_Y_VS_EXT_START_X	0x00000080
#define OFFSET_VENC_VS_EXT_STOP_X_VS_EXT_START_Y	0x00000084
#define OFFSET_VENC_VS_EXT_STOP_Y					0x00000088
#define OFFSET_VENC_AVID_START_STOP_X				0x00000090
#define OFFSET_VENC_AVID_START_STOP_Y				0x00000094
#define OFFSET_VENC_FID_INT_START_X_FID_INT_START_Y	0x000000A0
#define OFFSET_VENC_FID_INT_OFFSET_Y_FID_EXT_STRT_X 0x000000A4
#define OFFSET_VENC_FID_EXT_STRT_Y_FID_EXT_OFFSET_Y	0x000000A8
#define OFFSET_VENC_TVDETGP_INT_START_STOP_X		0x000000B0
#define OFFSET_VENC_TVDETGP_INT_START_STOP_Y		0x000000B4
#define OFFSET_VENC_GEN_CTRL						0x000000B8
#define OFFSET_VENC_OUTPUT_CONTROL					0x000000C4
#define OFFSET_VENC_OUTPUT_TEST						0x000000C8

#define MUX_VAL_VENC(OFFSET, VALUE)\
	writel((VALUE), OMAP34XX_DISP_SUB_VENC_BASE + (OFFSET));
#endif

/*
 * Display Controller VID1 Register
 */
#define OMAP34XX_DISP_VID1_BASE			0x48050400
#ifdef OMAP34XX_DISP_VID1_BASE
#define OFFSET_DISPC_REVISION			0x00000000
#define OFFSET_DISPC_SYSCONFIG			0x00000010
#define OFFSET_DISPC_SYSSTATUS			0x00000014
#define OFFSET_DISPC_IRQSTATUS			0x00000018
#define OFFSET_DISPC_IRQENABLE			0x0000001C
#define OFFSET_DISPC_CONTROL			0x00000040
#define OFFSET_DISPC_CONFIG				0x00000044
#define OFFSET_DISPC_DEFAULT_COLOR_0	0x0000004C
#define OFFSET_DISPC_DEFAULT_COLOR_1	0x00000050
#define OFFSET_DISPC_TRANS_COLOR_0		0x00000054
#define OFFSET_DISPC_TRANS_COLOR_1		0x00000058
#define OFFSET_DISPC_LINE_STATUS		0x0000005C
#define OFFSET_DISPC_LINE_NUMBER		0x00000060
#define OFFSET_DISPC_TIMING_H			0x00000064
#define OFFSET_DISPC_TIMING_V			0x00000068
#define OFFSET_DISPC_POL_FREQ			0x0000006C
#define OFFSET_DISPC_DIVISOR			0x00000070
#define OFFSET_DISPC_GLOBAL_ALPHA		0x00000074
#define OFFSET_DISPC_SIZE_DIG			0x00000078
#define OFFSET_DISPC_SIZE_LCD			0x0000007C

#define OFFSET_DISPC_VID1_BAj			0x000000BC
#define OFFSET_DISPC_VID1_POSITION		0x000000C4
#define OFFSET_DISPC_VID1_SIZE			0x000000C8
#define OFFSET_DISPC_VID1_ATTRIBUTES 	0x000000CC
#define OFFSET_DISPC_VID1_FIFO_THRESHD	0x000000D0
#define OFFSET_DISPC_VID1_FIFO_SIZE_STA	0x000000D4
#define OFFSET_DISPC_VID1_ROW_INC		0x000000D8
#define OFFSET_DISPC_VID1_PIXEL_INC		0x000000DC
#define OFFSET_DISPC_VID1_FIR			0x000000E0
#define OFFSET_DISPC_VID1_PICTURE_SIZE	0x000000E4
#define OFFSET_DISPC_VID1_ACCUI			0x000000E8
#define OFFSET_DISPC_VID1_FIR_COEF_Hi	0x000000F0
#define OFFSET_DISPC_VID1_FIR_COEF_HVi	0x000000F4
#define OFFSET_DISPC_VID1_CONV_COEF0	0x00000130
#define OFFSET_DISPC_VID1_CONV_COEF1	0x00000134
#define OFFSET_DISPC_VID1_CONV_COEF2	0x00000138
#define OFFSET_DISPC_VID1_CONV_COEF3	0x0000013C
#define OFFSET_DISPC_VID1_CONV_COEF4	0x00000140
#define OFFSET_DISPC_VID1_FIR_COEF_Vi	0x000001E0
#define OFFSET_DISPC_VID1_PRELOAD		0x00000230
#define MUX_VAL_DISPC(OFFSET, VALUE)\
	writel((VALUE), OMAP34XX_DISP_VID1_BASE + (OFFSET));
#endif



#ifndef __ASSEMBLY__
struct gpio {
	unsigned char res1[0x34];
	unsigned int oe;		/* 0x34 */
	unsigned int datain;		/* 0x38 */
	unsigned char res2[0x54];
	unsigned int cleardataout;	/* 0x90 */
	unsigned int setdataout;	/* 0x94 */
};
#endif /* __ASSEMBLY__ */

#define GPIO0				(0x1 << 0)
#define GPIO1				(0x1 << 1)
#define GPIO2				(0x1 << 2)
#define GPIO3				(0x1 << 3)
#define GPIO4				(0x1 << 4)
#define GPIO5				(0x1 << 5)
#define GPIO6				(0x1 << 6)
#define GPIO7				(0x1 << 7)
#define GPIO8				(0x1 << 8)
#define GPIO9				(0x1 << 9)
#define GPIO10				(0x1 << 10)
#define GPIO11				(0x1 << 11)
#define GPIO12				(0x1 << 12)
#define GPIO13				(0x1 << 13)
#define GPIO14				(0x1 << 14)
#define GPIO15				(0x1 << 15)
#define GPIO16				(0x1 << 16)
#define GPIO17				(0x1 << 17)
#define GPIO18				(0x1 << 18)
#define GPIO19				(0x1 << 19)
#define GPIO20				(0x1 << 20)
#define GPIO21				(0x1 << 21)
#define GPIO22				(0x1 << 22)
#define GPIO23				(0x1 << 23)
#define GPIO24				(0x1 << 24)
#define GPIO25				(0x1 << 25)
#define GPIO26				(0x1 << 26)
#define GPIO27				(0x1 << 27)
#define GPIO28				(0x1 << 28)
#define GPIO29				(0x1 << 29)
#define GPIO30				(0x1 << 30)
#define GPIO31				(0x1 << 31)

/* base address for indirect vectors (internal boot mode) */
#define SRAM_OFFSET0			0x40000000
#define SRAM_OFFSET1			0x00200000
#define SRAM_OFFSET2			0x0000F800
#define SRAM_VECT_CODE			(SRAM_OFFSET0 | SRAM_OFFSET1 | SRAM_OFFSET2)

#define OMAP3_PUBLIC_SRAM_BASE		0x40208000 /* Works for GP & EMU */
#define OMAP3_PUBLIC_SRAM_END		0x40210000

#define LOW_LEVEL_SRAM_STACK		0x4020FFFC

/* scratch area - accessible on both EMU and GP */
#define OMAP3_PUBLIC_SRAM_SCRATCH_AREA	OMAP3_PUBLIC_SRAM_BASE

#define DEBUG_LED1			149	/* gpio */
#define DEBUG_LED2			150	/* gpio */

#define XDR_POP			5	/* package on package part */
#define SDR_DISCRETE	4	/* 128M memory SDR module */
#define DDR_STACKED		3	/* stacked part on 2422 */
#define DDR_COMBO		2	/* combo part on cpu daughter card */
#define DDR_DISCRETE	1	/* 2x16 parts on daughter card */

#define DDR_100		100		/* type found on most mem d-boards */
#define DDR_111		111		/* some combo parts */
#define DDR_133		133		/* most combo, some mem d-boards */
#define DDR_165		165		/* future parts */

/*
 * 343x real hardware:
 *  ES1     = rev 0
 *
 *  ES2 onwards, the value maps to contents of IDCODE register [31:28].
 *
 * Note : CPU_3XX_ES20 is used in cache.S.  Please review before changing.
 */
#define CPU_3XX_ES10		0
#define CPU_3XX_ES20		1
#define CPU_3XX_ES21		2
#define CPU_3XX_ES30		3
#define CPU_3XX_ES31		4
#define CPU_3XX_MAX_REV		(CPU_3XX_ES31 + 1)

#define CPU_3XX_ID_SHIFT	28

#define WIDTH_8BIT		0x0000
#define WIDTH_16BIT		0x1000	/* bit pos for 16 bit in gpmc */

/*
 * Hawkeye values
 */
#define HAWKEYE_OMAP34XX	0xb7ae
#define HAWKEYE_AM35XX		0xb868
#define HAWKEYE_OMAP36XX	0xb891

#define HAWKEYE_SHIFT		12

/*
 * Define CPU families
 */
#define CPU_OMAP34XX		0x3400	/* OMAP34xx/OMAP35 devices */
#define CPU_AM35XX			0x3500	/* AM35xx devices          */
#define CPU_OMAP36XX		0x3600	/* OMAP36xx devices        */

/*
 * Define CPUs
 */
#define OMAP3430		0x3430
#define OMAP3503		0x3503
#define OMAP3515		0x3515
#define OMAP3525		0x3525
#define OMAP3530		0x3530

#define AM3505			0x3505
#define AM3517			0x3517

#define OMAP3630		0x3630
#define OMAP3730		0x3730
/*
 * Control status register values corresponding to cpu variants
 */
#define CTRL_OMAP3503		0x5c00
#define CTRL_OMAP3515		0x1c00
#define CTRL_OMAP3525		0x4c00
#define CTRL_OMAP3530		0x0c00

#define CTRL_AM3505			0x5c00
#define CTRL_AM3517			0x1c00

#define CTRL_OMAP3730		0x0c00

/*
 * ROM code API related flags
 */
#define OMAP3_GP_ROMCODE_API_L2_INVAL		1
#define OMAP3_GP_ROMCODE_API_WRITE_ACR		3

/*
 * EMU device PPA HAL related flags
 */
#define OMAP3_EMU_HAL_API_L2_INVAL		40
#define OMAP3_EMU_HAL_API_WRITE_ACR		42

#define OMAP3_EMU_HAL_START_HAL_CRITICAL	4

#endif
