/*
 * DHCOM specific Code added
 * 2012-01-19 LZ
 *
 * (C) Copyright 2002
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef	__ASM_GBL_DATA_H
#define __ASM_GBL_DATA_H
/*
 * The following data structure is placed in some memory wich is
 * available very early after boot (like DPRAM on MPC8xx/MPC82xx, or
 * some locked parts of the data cache) to allow for a minimum set of
 * global variables during system initialization (until we have set
 * up the memory controller so that we can use RAM).
 *
 * Keep it *SMALL* and remember to set CONFIG_SYS_GBL_DATA_SIZE > sizeof(gd_t)
 */

typedef	struct	global_data {
	bd_t		*bd;
	unsigned long	flags;
	unsigned long	baudrate;
	unsigned long	have_console;	/* serial_init() was called */
	unsigned long	env_addr;		/* Address  of Environment struct */
	unsigned long	env_valid;		/* Checksum of Environment valid? */
	unsigned long	fb_base;		/* base address of frame buffer */
#ifdef CONFIG_VFD
	unsigned char	vfd_type;		/* display type */
#endif
#if 0
	unsigned long	cpu_clk;		/* CPU clock in Hz!		*/
	unsigned long	bus_clk;
	phys_size_t	ram_size;			/* RAM size */
	unsigned long	reset_status;	/* reset status register at boot */
#endif
#if !(defined(CONFIG_SYS_ICACHE_OFF) && defined(CONFIG_SYS_DCACHE_OFF))
	unsigned long	tlb_addr;
#endif
#ifdef CONFIG_DHCOM
    settingsinfo_t  dh_board_settings;  /* DH settings block */
    unsigned long   dh_ulFlags; 	    /* Flags used for DH custom features */
    unsigned long   dh_ulSDRAMSize;     /* SDRAM size */
#endif
	void		**jt;				/* jump table */
} gd_t;

/*
 * Global Data Flags
 */
#define	GD_FLG_RELOC	0x00001		/* Code was relocated to RAM		*/
#define	GD_FLG_DEVINIT	0x00002		/* Devices have been initialized	*/
#define	GD_FLG_SILENT	0x00004		/* Silent mode				*/
#define	GD_FLG_POSTFAIL	0x00008		/* Critical POST test failed		*/
#define	GD_FLG_POSTSTOP	0x00010		/* POST seqeunce aborted		*/
#define	GD_FLG_LOGINIT	0x00020		/* Log Buffer has been initialized	*/
#define GD_FLG_DISABLE_CONSOLE	0x00040		/* Disable console (in & out)	 */

#define DECLARE_GLOBAL_DATA_PTR     register volatile gd_t *gd asm ("r8") /* pointer to global data is kept in Register R8 */

/*
 * DH defines
 */

#define DISABLE_PRINTF_FLAG		 		0x01   	// 0 = printf is activated, 1 = printf is deactivated
#define UPDATE_ACTIVE_FLAG	     		0x02   	// 0 = update isn't active, 1 = update is active
#define BOOT_AFTER_UPDATE_FLAG	 		0x04   	// 0 = Don't boot OS after update, 1 = Boot OS after update
#define NO_OSIMAGE_FOUND_FLAG    		0x08   	// 0 = OS image available, 1 = OS image not available
#define TRY_TO_CREATE_BBT_FLAG   		0x10
#define SESSION_PRINTF_DISABLED_FLAG	0x20 	// 0 = printf for this session is enabled, 1 = printf this session is disabled

// If we have a flash memory without bad blocks, the BBT wouldn't be created on the first start of bootloader.
// Reason: If we don't have bad blocks, the BBT content in main area is 0xFF. The same we have on a erased flash block.
//         On creating BBT, the flash write algorithm see's that the old and new flash content would be the same. So the
//         flash block of BBT wouldn't be updated. But the spare area content of the old and the new block is different.
//         So it is necessary to update the block anyway. The flag TRY_TO_CREATE_BBT_FLAG is used to create the BBT also
//         if the block content is equal.

#define CLEAR_DH_GD_FLAG(f)     gd->dh_ulFlags &= (~f)
#define SET_DH_GD_FLAG(f)       gd->dh_ulFlags |= (f)

#define SESSION_DEPENDED_PRINTF_ENABLE() \
if((gd->dh_ulFlags & SESSION_PRINTF_DISABLED_FLAG) == 0) \
{														\
    CLEAR_DH_GD_FLAG(DISABLE_PRINTF_FLAG);				\
}

#define DISABLE_PRINTF() SET_DH_GD_FLAG(DISABLE_PRINTF_FLAG);

#endif /* __ASM_GBL_DATA_H */
