/*
 * dh_am3517.h - Default configuration for DH_AM3517-Modul.
 *
 * Author: Ludwig Zenz <lzenz@dh-electronics.de>
 *
 * Based on omap3_evm_config.h and am2517evm.h
 *
 * Copyright (C) 2011-2012 DH electronics GmbH
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __CONFIG_H
#define __CONFIG_H

//#define DEBUG
//#define DEBUG_BOOTTIME
/*-------------------------------------------------------------------
 * High Level Configuration Options
 */
#define CPU_TYPE_STRING			"TI Sitara AM35x"	/* Necessary to display board-info */

#define CONFIG_DHCOM			1		/* This is a DHCOM Module */

#define CONFIG_ARMCORTEXA8		1		/* This is an ARM V7 CPU core */
#define CONFIG_OMAP				1		/* in a TI OMAP core */
#define CONFIG_AM3517			1		/* which is a AM3517 */
#define CONFIG_DH_AM3517		1		/* working with DH_AM3517 */

#define CONFIG_EMIF4			1		/* The chip has EMIF4 controller - init the emif4 module for DDR access */

#include <asm/arch/cpu.h>				/* get chip and board defs */
#include <asm/arch/omap3.h>

/*-------------------------------------------------------------------------
 * DH - Defines
 */
#define UBOOT_DH_VERSION 	"1.1.0.3" 	/* DH - Version of U-Boot e.g. 1.4.0.1 */
// #define CONFIG_IDENT_STRING " DHCOM AM35x"
/*-------------------------------------------------------------------------
 * Defines for NAND-Flash organization
 */
#define DH_NAND_FLASH
#ifdef DH_NAND_FLASH
	#define FLASH_TYPE_STRING           "NAND"
	#define CONFIG_CMD_NAND								/* NAND support	*/

	// #define NAND_BASE				0x00000000  	/* already defined in cpu.h */
	#define CONFIG_SYS_NAND_BASE		NAND_BASE		/* physical address to access nand */

	/* Configure General Purpose Memory Controller */
	#define CONFIG_NAND_OMAP_GPMC				 		/* Enable configure GPMC for NAND */
	#define GPMC_NAND_ECC_LP_x8_LAYOUT		 			/* Large Page x8 NAND device Layout */
	#define CONFIG_SYS_NAND_USE_FLASH_BBT				/* Use a Flash based BBT */

	#define CONFIG_OMAP3_DMA_PREFETCH					/* Use Prefetch engine to speed up nand */
	#define CONFIG_OMAP3_DMA							/* Use DMA to speed up nand */

	/* Position and size of U-Boot in NAND-Flash */
	#define CONFIG_SYS_MONITOR_BASE		0x00080000 		/* Start address of the U-Boot image in NAND-Flash */
	#define CONFIG_SYS_MONITOR_LEN		(512 << 10)	/* Reserve 512k (4 blocks each 64 pages) for the U-Boot image in flash */

	/* Location, offset and size of the U-Boot environment in NAND-Flash */
	#define CONFIG_ENV_IS_IN_NAND		1		 		/* specifies that environment is in nand */

	#define CONFIG_ENV_OFFSET			0x00100000		/* U-Boot environment starts at this offset */
	#define CONFIG_ENV_SIZE				(128 << 10)		/* 1*128 KiB sector (2¹⁰ == 1024)*/
	#define CONFIG_ENV_RANGE			(256 << 10)		/* 2*128 KiB sector (2¹⁰ == 1024)*/

	#define CONFIG_ENV_OFFSET_REDUND	CONFIG_ENV_OFFSET + (2 * CONFIG_ENV_SIZE)	/* U-Boot environment starts at this offset */
	#define CONFIG_ENV_SIZE_REDUND		CONFIG_ENV_SIZE								/* the same size as default environment */

	//#define CONFIG_SKIP_RELOCATE_UBOOT				/* does not work - no idea why */


#else /* DH_SPI_FLASH */
#define CONFIG_SPI_FLASH_NUMONYX 			1

#define FLASH_TYPE_STRING                   "SPI"
#define CONFIG_SYS_NO_FLASH                 1
#define CONFIG_CMD_SPI                      1
#define CONFIG_CMD_SF                       1

#define SPI_FLASH_PAGE_SIZE                 0x100 // 256 Byte
#define SPI_FLASH_SECTOR_SIZE               0x10000 // 64 kByte
#define SPI_FLASH_SIZE_MB                   4 		// 4MByte
#define MAX_SPI_BYTES		                32
#define CONFIG_SPI_FLASH_CS	                0

 //   #define CSPI1_BASE_ADDR                     0x43FA4000
  //  #define SPI_FLASH_FREQ                      16600000//1100000
//	#define CONFIG_FSL_SF		                1
#endif

/*---------------------------------------------------------------------------
 * Physical Memory Map SDRAM
 */
#define CONFIG_NR_DRAM_BANKS	1 //2				/* not be populated */
#define PHYS_SDRAM_1			OMAP34XX_SDRC_CS0 	/* 0x80000000 - start of SDRAM main address space - EMIF4*/
#define PHYS_SDRAM_1_SIZE		(128 << 20)			/* 256 MB - this define is only used for help-output! */

// DHCOM cortex A8 has only one CS
//#define PHYS_SDRAM_2		OMAP34XX_SDRC_CS1	/* 0xA0000000 */
//#define PHYS_SDRAM_2_SIZE	(128 << 20)			/* 128 MB - this define is only used for help-output! */

#define CONFIG_SYS_LOAD_ADDR		0x80410000 		/* default load address (SDRAM) - if calling e.g. bootm or bootwince without arguments */

#ifdef CONFIG_ENV_OFFSET_REDUND						/* redundant environment needs three-times malloc space */

#define CONFIG_SYS_MALLOC_LEN		((CONFIG_ENV_SIZE*3)+(128 << 10)+(4 << 20)+ (5 << 20)) 	/* Size of malloc() pool - should be greater than 4 MB for unzip routines */
#else
#define CONFIG_SYS_MALLOC_LEN		(CONFIG_ENV_SIZE + (128 << 10)) 	/* Size of malloc() pool - should be greater than 4 MB for unzip routines */
#endif
#define CONFIG_SYS_GBL_DATA_SIZE	128									/* bytes reserved for initial data */



/*---------------------------------------------------------------------------
 * Clock Defines
 */
#define V_OSCK			26000000		/* Clock output from T2 (26 MHz) */
#define V_SCLK			(V_OSCK >> 1)	/* System-Clock is running with the half of the oscillator-frequency */


#undef CONFIG_USE_IRQ					/* no support for IRQs */
#define CONFIG_MISC_INIT_R

#define CONFIG_CMDLINE_TAG			1	/* enable passing of ATAGs */
#define CONFIG_SETUP_MEMORY_TAGS	1
#define CONFIG_INITRD_TAG			1
#define CONFIG_REVISION_TAG			1


#define BOARD_LATE_INIT					/* Execute board_late_init() */

/*
 * DDR2-RAM Transfer Buffer for dh_board_settings
 *
 * the x-loader copies the dh_board settings block to this ddr2 RAM address
 */
#define CONFIG_SETTINGSBIN_TRANSFER_BUFFER  0x803E0000

/*
 * Default settings
 */
#define DEFAULT_SETTINGS_BLOCK_LENGTH       0x2C        /* 44 Byte */
#define DEFAULT_SETTINGS_DISPLAY_ID         0x01
#define DEFAULT_SETTINGS_Y_RESOLUTION       272
#define DEFAULT_SETTINGS_X_RESOLUTION       480
#define DEFAULT_SETTINGS_LCD_CONFIG_FLAGS   0x03E7      /* includes GPIO G for BE enable */

#define DEFAULT_SETTINGS_PIXEL_CLOCK        11100
#define DEFAULT_SETTINGS_V_PULSE_WIDTH      11
#define DEFAULT_SETTINGS_H_PULSE_WIDTH      42
#define DEFAULT_SETTINGS_H_BACK_PORCH       2
#define DEFAULT_SETTINGS_H_FRONT_PORCH      3
#define DEFAULT_SETTINGS_V_BACK_PORCH       2
#define DEFAULT_SETTINGS_V_FRONT_PORCH      3
#define DEFAULT_SETTINGS_AC_BIAS_TRANS      0
#define DEFAULT_SETTINGS_AC_BIAS_FREQ       0
#define DEFAULT_SETTINGS_DATALINES          24

#define DEFAULT_SETTINGS_GPIO_DIRECTION     0x1FF       /* 1 = Input, 0 = Output --> bits 0 - 8 = GPIO A - I */
#define DEFAULT_SETTINGS_GPIO_STATE         0x055       /* 0 = Low, 1 = High --> bits 0 - 8 = GPIO A - I */

#define DEFAULT_SETTINGS_HW_CONFIG_FLAGS    0x027E     	/* SILENT_MODE = off , UPDATE_DEV = all , USE_DA_EEPROM_SETTINGS = on */

#define DEFAULT_BACKLIGHT_PWM				57  		/* DHCOM GPIO_PWM - this is fix and not to change with settings.bin functionality */

/*
 * DHCOM Standard GPIO to omap3/am35x gpio numbers mapping
 */

#define DHCOM_GPIO_A 		116
#define DHCOM_GPIO_B 		117
#define DHCOM_GPIO_C 		118
#define DHCOM_GPIO_D 		119
#define DHCOM_GPIO_E 		125
#define DHCOM_GPIO_F 		140
#define DHCOM_GPIO_G 		141
#define DHCOM_GPIO_H 		155
#define DHCOM_GPIO_I 		152

/* additional GPIOx (extended DHCOM Standard) */

#define DHCOM_GPIO_J  		96
#define DHCOM_GPIO_K  		94
#define DHCOM_GPIO_L 		58
#define DHCOM_GPIO_M 		97
#define DHCOM_GPIO_N 		98
#define DHCOM_GPIO_O 		95
#define DHCOM_GPIO_P 		106
#define DHCOM_GPIO_Q 		105
#define DHCOM_GPIO_R 		104
#define DHCOM_GPIO_S 		103
#define DHCOM_GPIO_T 		102
#define DHCOM_GPIO_U 		101
#define DHCOM_GPIO_V 		100
#define DHCOM_GPIO_W 		99

/* further GPIOs */

#define DHCOM_ETH_VIO_GPIO 	146

/*
 * Hardware drivers
 */

/*-----------------------------------------------------------------------------
 * NS16550 Configuration - Universal asynchronous receiver/transmitter (UART)
 */
#define V_NS16550_CLK			48000000	/* 48MHz (APLL96/2) */

#define CONFIG_SYS_NS16550
#define CONFIG_SYS_NS16550_SERIAL
#define CONFIG_SYS_NS16550_REG_SIZE	(-4)
#define CONFIG_SYS_NS16550_CLK		V_NS16550_CLK


/*-----------------------------------------------------------------------------
 * select serial console configuration
 */
#define CONFIG_CONS_INDEX		1
#define CONFIG_SYS_NS16550_COM1 OMAP34XX_UART1
#define CONFIG_SERIAL1			1	/* UART1 on DHCOM Cortex A8 */

#define CONFIG_ENV_OVERWRITE 		/* allow to overwrite serial and ethaddr */
#define CONFIG_BAUDRATE				38400
#define CONFIG_SYS_BAUDRATE_TABLE	{4800, 9600, 19200, 38400, 57600, 115200}

/*-----------------------------------------------------------------------------
 * USB
 * Enable CONFIG_MUSB_HCD for Host functionalities MSC, keyboard
 * Enable CONFIG_MUSB_UDD for Device functionalities.
 */
#define CONFIG_USB_AM35X		1
//#define CONFIG_MUSB_HCD			1		/* USB OTG support */
#define CONFIG_USB_EHCI 		1 			/* USB Host support - High Speed interface (Enhanced Host Controller Interface) */
//#define CONFIG_MUSB_UDC			1

#ifdef CONFIG_USB_AM35X

#if defined(CONFIG_MUSB_HCD) | defined(CONFIG_USB_EHCI)

	#define CONFIG_CMD_USB
	//#define DHCOM_MUSB_OTG_VBUS_EN	DHCOM_GPIO_B


	#define CONFIG_USB_EHCI_OMAP
	#define CONFIG_SYS_USB_EHCI_MAX_ROOT_PORTS 3

	// #define CONFIG_EHCI_DCACHE*/ /* leave it disabled for now
	// #define CONFIG_OMAP_EHCI_PHY1_RESET_GPIO       156

	#define CONFIG_USB_STORAGE
	#define CONFIG_CMD_STORAGE
	// #define CONFIG_CMD_FAT /* some lines later already defined */

	#ifdef CONFIG_USB_KEYBOARD
		#define CONFIG_SYS_USB_EVENT_POLL
		#define CONFIG_PREBOOT "usb start"
	#endif /* CONFIG_USB_KEYBOARD */

#endif /* CONFIG_MUSB_HCD */

#ifdef CONFIG_MUSB_UDC
/* USB device configuration */
#define CONFIG_USB_DEVICE		1
#define CONFIG_USB_TTY			1
#define CONFIG_SYS_CONSOLE_IS_IN_ENV	1
/* Change these to suit your needs */
#define CONFIG_USBD_VENDORID		0x0451
#define CONFIG_USBD_PRODUCTID		0x5678
#define CONFIG_USBD_MANUFACTURER	"DH electronics GmbH" //"Texas Instruments"
#define CONFIG_USBD_PRODUCT_NAME	"DHCOM Cortex A8 Modul"
#endif /* CONFIG_MUSB_UDC */

#endif /* CONFIG_USB_AM35X */

/*-----------------------------------------------------------------------------
 * commands to include
 */

#define CONFIG_SYS_NO_FLASH			/* neccessary to exclude commands for flash-handling */

#include <config_cmd_default.h> 	/* Include file with defines of default commands that always needed */
//#include <config_cmd_all.h>		/* Include file with alphabetical list of all possible commands. - You have to undef not needed commands.*/

/* Include support for File-Systems */
#undef CONFIG_CMD_EXT2				/* EXT2 Support	*/
#define CONFIG_CMD_FAT				/* FAT support */
#undef CONFIG_CMD_JFFS2				/* JFFS2 Support */
//#define CONFIG_CMD_UBIFS			/* UBIFS Support */
#define CONFIG_CMD_CACHE			/* icache, dcache */
#define CONFIG_CMD_I2C				/* I2C serial bus support */
#define CONFIG_CMD_SPI				/* SPI bus support */
#define CONFIG_CMD_MII				/* MII utility commands */
#define CONFIG_CMD_NET				/* Network support	*/
#define CONFIG_CMD_PING				/* send ICMP ECHO_REQUEST to network host */
#define CONFIG_CMD_MMC				/* MMC support */
#define CONFIG_CMD_MMU				/* Switch ON or OFF MMU */
#define CONFIG_CMD_BMP				/* BMP support - requires Display support */
#define CONFIG_CMD_DHCP				/* Include dhcp command */
#define CONFIG_CMD_UNZIP			/* to unzip from memory to memory */
#define CONFIG_CMD_SETTINGS_INFO    /* DH: print dh_board_settings block */
#define CONFIG_CMD_DHCOM_UPDATE 	/* DH: Update functionality via MMC */
#define CONFIG_CMD_WINCE			/* DH: WinCE command - MMU support has to be included */
#define CONFIG_CMD_TESTROUTINES		/* DH: Include test-routines of DH */
#define CONFIG_CMD_BOARD_INFO		/* DH: Include boardinfo cmd for DHCOM Testbench */
#define CONFIG_CMD_GPIO_INFO		/* DH: display info about DHCOM gpio states */
#define CONFIG_CMD_DISPLAY_INFO		/* DH: display lcd settings */
#define CONFIG_CMD_ERASEENV			/* Enable erase environment command */
// #define CONFIG_CMD_LCD_DEBUG		/* DH: debug commandos to test the lcd interface */

/* undef not needed commands */
#undef CONFIG_CMD_FLASH				/* flinfo, erase, protect - We are working with NAND-Flash and not with Flash. See CONFIG_CMD_NAND */
#undef CONFIG_CMD_FPGA				/* FPGA configuration Support is not needed	*/
#undef CONFIG_CMD_IMI
#undef CONFIG_CMD_IMLS				/* imls - list all images found in flash */
#undef CONFIG_CMD_ITEST				/* not include integer test */
#undef CONFIG_CMD_LOADB				/* loadb			*/
#undef CONFIG_CMD_LOADS				/* loads			*/
#undef CONFIG_CMD_MISC				/* Misc functions like sleep etc*/
#undef CONFIG_CMD_SETGETDCR			/* DCR support on 4xx		*/
#undef CONFIG_CMD_XIMG				/* Load part of Multi Image	*/




/*-------------------------------------------------------------------------------
 * Display CPU, Board and GPIO information
 */
#define CONFIG_DISPLAY_CPUINFO		/* Display CPU information		*/
#define CONFIG_DISPLAY_BOARDINFO	/* Display Board information	*/

/*-------------------------------------------------------------------------------
 * I2C Support
 */
#define CONFIG_HARD_I2C				1
#define CONFIG_SYS_I2C_SPEED		100000
#define CONFIG_SYS_I2C_SLAVE		0		/* CPU node is master - clear slave address to 0 */
#define CONFIG_DRIVER_OMAP34XX_I2C	1

#define	CONFIG_I2C_MULTI_BUS
//#define CONFIG_SYS_I2C_NOPROBES		{{1,0x4B}}

/*-------------------------------------------------------------------------------
 * SPI Support
 */
#define CONFIG_OMAP3_SPI

/*
 * Debug - Commands
 */
#ifdef DEBUG
#define CMD_MEM_DEBUG
#define CONFIG_MTD_DEBUG
#define CONFIG_MTD_DEBUG_VERBOSE 2
#endif

/*
 * Flash Update defines (Command do_update())
 */
#define UPDATE_DHUPDATE_INI_SDRAM_ADDRESS           "80410000"  /* should equal os base */
#define UPDATE_BMP_SDRAM_ADDRESS                    "80510000"

/*
 * Boot WinCE defines (Command do_bootwince())
 */
#define BOOTCE_NK_GZ_IMAGE_SDRAM_ADDRESS            "80410000"
#define BOOTCE_NK_GZ_SDRAM_BUFFER_ADDRESS           0x89410000
#define BOOTCE_NK_GZ_SDRAM_BUFFER_SIZE              0x05000000 // 64MByte on 64MByte SDRAM
#define BOOTCE_NK_BIN_SDRAM_BUFFER_ADDRESS          0x88000000
#define BOOTCE_NK_BIN_SDRAM_BUFFER_SIZE             0x09000000 // 128MByte on 64MByte SDRAM
#define BOOTCE_ARGUMENTS_SDRAM_ADDRESS              0x80400000



/*----------------------------------------------------------------------------------
 * MMC Configs
 * */
#ifdef CONFIG_CMD_MMC
    #define CONFIG_MMC                      1
	#define CONFIG_OMAP3_MMC
    #define CONFIG_DOS_PARTITION            1
#ifndef CONFIG_CMD_FAT
    #define CONFIG_CMD_FAT                  1
#endif
#endif

/*----------------------------------------------------------------------------------
 * Memory Management Unit
 */

#define MMU_PAGE_TABLE_BASE_ADDRESS         0x87FFC000 	/* base address of mmu translation table */

/*----------------------------------------------------------------------------------
 * ToDo: Board NAND Info.
 */
#define CONFIG_SYS_MAX_NAND_DEVICE	1			/* Max number of NAND devices */
#define CONFIG_SYS_64BIT_VSPRINTF				/* needed for nand_util.c - Makes vsprintf (and
												   all *printf functions) support printing
												   of 64bit values by using the L quantifier*/

#define CONFIG_JFFS2_NAND						/* For a default partition on a NAND device */
#define CONFIG_JFFS2_DEV			"nand0"		/* nand device jffs2 lives on */

#define CONFIG_JFFS2_PART_OFFSET	0x680000	/*?? start of jffs2 partition */
#define CONFIG_JFFS2_PART_SIZE		0xf980000	/*?? size of jffs2 part */

/*
 *  Environment information
 */
#define CONFIG_BOOTFILE		uImage

#ifdef DH_NAND_FLASH
    #define DH_FLASH_DEV_ENV                \
        "splashimage=00180000\0"            \
        "settingsflashaddr=005c0000\0"

#else /* DH_SPI_FLASH */
    #define DH_FLASH_DEV_ENV                \
        "splashimage=00100000\0"            \
        "settingsflashaddr=00180000\0"
#endif /* DH_NAND_FLASH */


#define CONFIG_EXTRA_ENV_SETTINGS 							\
	"splashpos=m,m\0"										/* if the bmp is smaller as display center it */\
	"kernel=uImage\0"                       				\
	"linux_root=root=/dev/mmcblk0p2 rootwait\0"				\
    "linux_nfs_path=/tftpboot/nfsroot\0"                    \
    "linux_console=console=ttyO0,38400,8n1 earlyprintk\0"	\
    "otgmode=otg_mode=host\0"										/*setup usb otg modus */\
    \
    /* linux framebuffer preferences */\
    "linux_fb=vram=32M omapfb.vram=0:32M\0"		\
    "linux_framebuffer=vram=32M omapfb.vram=0:32M\0"		\
    "linux_fb_depth=24\0"\
    "linux_fbcon_rotate=0\0"                            	\
    \
    /* boodcmd for wince */\
    "nandbootwince=update auto; bootwince 00600000\0"  		\
    \
    /* set bootargs for linux */\
	"setbootargs=setenv bootargs ${linux_console} ${linux_root} ${linux_fb} fb_depth=${linux_fb_depth} fbcon=rotate:${linux_fbcon_rotate} ${otgmode} pwm_pol=${pwm_pol} ${linux_add_param}\0"\
	"setbootargsnfs=setenv bootargs ${linux_console} root=/dev/nfs rootwait nfsroot=${serverip}:${linux_nfs_path} ip=dhcp ${linux_fb} fb_depth=${linux_fb_depth} fbcon=rotate:${linux_fbcon_rotate} ${otgmode} pwm_pol=${pwm_pol} ${linux_add_param}\0" \
	\
	/* bootcmds for linux for booting from nand-flash */\
    "nandbootlinux=update auto; run setbootargs; nand read 0x80410000 0x00600000 0x00400000; bootm\0"\
    "nandbootlinuxnfs=run setbootargsnfs; nand read 0x80410000 0x00600000 0x00400000; bootm\0" \
    \
    "save_ENV_to_flash=setenv ENVinFlash 1; saveenv; saveenv\0"          /*saveenv twice because of redundant version */    \
    \
    /* nandflash offsets and partitionsizes*/\
    "winceimagesize=00000000\0"                                          \
    "winceimagetype=no_wince\0"                                          \
    "rambufferaddr=81000000\0"                                           /* if to high use 0x80610000*/\
    "nandflashOSaddr=00600000\0"                                         \
    "bootloaderflashaddr=00080000\0"                                     \
    "wincepartitionsize=01D80000\0"                                      \
    "linuxkernelpartitionsize=00380000\0"                                \
	"bootcmdwince=dhcp 88000000 nk.gz; bootwince 88000000 debug\0" \
	"bootcmdeboot=dhcp 88000000 ebootnd.bin; bootwince 88000000 debug\0" \
    DH_FLASH_DEV_ENV


#define CONFIG_BOOTCOMMAND \
    "run nandbootlinux"

#define CONFIG_AUTO_COMPLETE	1

/*----------------------------------------------------------------------------------
 * Configure the u-boot console
 */
#define CONFIG_SYS_PROMPT				"DH-AM35x # "

#define CONFIG_SYS_LONGHELP							/* undef to save memory - only minimal help */
#define CONFIG_SYS_HUSH_PARSER						/* use "hush" command parser */
#define CONFIG_SYS_PROMPT_HUSH_PS2		"> "

#define CONFIG_CMDLINE_EDITING          1			/* Enable editing and History functions for interactive
													   commandline input operations */

#define CONFIG_BOOTDELAY  0							/* Seconds to wait before booting the OS */
#define CONFIG_DONT_SHOW_STOP_AUTOBOOT
#define CONFIG_ZERO_BOOTDELAY_CHECK
#define CONFIG_SYS_CONSOLE_INFO_QUIET				/* Suppress display of console information at boot */

#define CONFIG_SYS_CBSIZE				512			/* Console I/O Buffer Size */
#define CONFIG_SYS_CONSOLE_ENTER_CMD	0x7F		/* enter the console only with "DEL" key */

#define CONFIG_SYS_PBSIZE				(CONFIG_SYS_CBSIZE + sizeof(CONFIG_SYS_PROMPT) + 16)	/* Print Buffer Size */
#define CONFIG_SYS_MAXARGS				16			/* max number of command args */

/*----------------------------------------------------------------------------------
 * Miscellaneous configurable options
 */

	/* mem-test works on */
	#define CONFIG_SYS_MEMTEST_START	(OMAP34XX_SDRC_CS0 + 0x00000000)  	/* start with beginning of ddr2 */
	#define CONFIG_SYS_MEMTEST_END		(OMAP34XX_SDRC_CS0 + 0x07000000) 	/* stop before u-boot stack */

	/* nandflash-test works on */
	#define CONFIG_SYS_NANDTEST_START	NAND_BASE + 0x02a00000				/* Attention: Test works in area of splash-image! Bigger ones will be destroyed */
	#define CONFIG_SYS_NANDTEST_LENGTH	0x00200000							/* use 1 block for nand-flash test */

/*
 * AM3517 has 12 GP timers, they can be driven by the system clock
 * (12/13/16.8/19.2/38.4MHz) or by 32KHz clock. We use 13MHz (V_SCLK).
 * This rate is divided by a local divisor.
 */
#define CONFIG_SYS_TIMERBASE		OMAP34XX_GPT2
#define CONFIG_SYS_PTV				2	/* Divisor: 2^(PTV+1) => 8 */
#define CONFIG_SYS_HZ				1000

/*-----------------------------------------------------------------------
 * Stack sizes
 *
 * The stack sizes are set up in start.S using the settings below
 */
#define CONFIG_STACKSIZE	(128 << 10)	/* regular stack 128 KiB */
#ifdef CONFIG_USE_IRQ
#define CONFIG_STACKSIZE_IRQ	(4 << 10)	/* IRQ stack 4 KiB */
#define CONFIG_STACKSIZE_FIQ	(4 << 10)	/* FIQ stack 4 KiB */
#endif



/* **** PISMO SUPPORT *** */

/* Configure the PISMO */
#define PISMO1_NAND_SIZE		GPMC_SIZE_128M	/* Defines how big is the mapping via GPMC is */

#define CONFIG_SYS_MAX_FLASH_SECT	520	/* max number of sectors on one chip */
#define CONFIG_SYS_MAX_FLASH_BANKS	1	/* max number of flash banks */



/*-----------------------------------------------------------------------
 * CFI FLASH driver setup
 */
/* timeout values are in ticks */
#define CONFIG_SYS_FLASH_ERASE_TOUT	(100 * CONFIG_SYS_HZ)
#define CONFIG_SYS_FLASH_WRITE_TOUT	(100 * CONFIG_SYS_HZ)

/* Flash banks JFFS2 should use */
#define CONFIG_SYS_MAX_MTD_BANKS	(CONFIG_SYS_MAX_FLASH_BANKS + \
					CONFIG_SYS_MAX_NAND_DEVICE)
#define CONFIG_SYS_JFFS2_MEM_NAND
/* use flash_info[2] */
#define CONFIG_SYS_JFFS2_FIRST_BANK	CONFIG_SYS_MAX_FLASH_BANKS
#define CONFIG_SYS_JFFS2_NUM_BANKS	1


/*-------------------------------------------------------------------------
 * ethernet support for AM3517
 */

#if defined(CONFIG_CMD_NET)
	#define CONFIG_DRIVER_TI_EMAC			/* Configure TI-emac driver (davinci-emac) */
	#define CONFIG_MII						/* Configure MII (media independent interface) */
	#define CONFIG_DRIVER_TI_EMAC_USE_RMII 	/* DHCOM ethernet PHY is connect in RMII (reduced MII) way */

	//#define CONFIG_BOOTP_DNS				/* If a DHCP client requests the DNS serverip from a DHCP server, it is possible that more */
	//#define CONFIG_BOOTP_DNS2				/* than one DNS serverip is offered to the client. If CONFIG_BOOTP_DNS2 is enabled, the secondary DNS */
											/* serverip will be stored in the additional environment variable "dnsip2". The first DNS serverip is always */
											/* stored in the variable "dnsip", when CONFIG_BOOTP_DNS is defined. */

	//#define CONFIG_BOOTP_SEND_HOSTNAME	/* Some DHCP servers are capable to do a dynamic update of a DNS server. To do this, they */
											/* need the hostname of the DHCP requester.	If CONFIG_BOOTP_SEND_HOSTNAME is defined, the content */
											/* of the "hostname" environment variable is passed as option 12 to the DHCP server. */
	#define CONFIG_NET_RETRY_COUNT 2		/* This variable defines the number of retries for network operations like ARP, RARP, TFTP, or BOOTP */
											/* before giving up the operation. If not defined, a default value of 5 is used. */

	#define CONFIG_NET_MULTI				/* multiple ethernet interfaces */

	#define CONFIG_NETMASK                      255.255.255.0
	#define CONFIG_SERVERIP                     10.64.1.34
	#define CONFIG_GATEWAYIP                    10.64.1.34
	#define CONFIG_IPADDR                       10.64.1.231
	#define CONFIG_ETHADDR                      00:00:00:00:00:00		// replace MAC-address on first boot with rtc unique id
#endif


/*-------------------------------------------------------------------------
 * LCD - definitions
 */

#define CONFIG_AM35XX_LCD	/* Enable Display Support */

#ifdef  CONFIG_AM35XX_LCD
	#define CONFIG_FRAMEBUFFER_BASE             0x80004000
//  #define CONFIG_RGB24_FRAMEBUFFER
	#define CONFIG_LCD
	#undef 	LCD_TEST_PATTERN
	#define CONFIG_SPLASH_SCREEN                1
//	#define CONFIG_VIDEO_BMP_GZIP				1
	#define CONFIG_SPLASH_SCREEN_ALIGN
	#define CONFIG_SYS_VIDEO_LOGO_MAX_SIZE		400000
	#define CONFIG_SYS_WHITE_ON_BLACK			/* To get a black screen if no splashimage is available */
#endif

#endif /* __CONFIG_H */
