//------------------------------------------------------------------------------
//
//  Define:  OAL_ARGS_xxx
//
//  This constant are used to identify/verify argument structure and its
//  version in memory.
//
#define OAL_ARGS_SIGNATURE      "ARGS"
#define OAL_ARGS_VERSION        1
#define BSP_ARGS_VERSION        1

//------------------------------------------------------------------------
// LCD/DVI
//------------------------------------------------------------------------

typedef enum OMAP_LCD_DVI_RES {
    OMAP_LCD_DEFAULT=0,
    OMAP_DVI_640W_480H,
    OMAP_DVI_640W_480H_72HZ,
    OMAP_DVI_800W_480H,
    OMAP_DVI_800W_600H,
    OMAP_DVI_800W_600H_56HZ,
    OMAP_DVI_1024W_768H,
    OMAP_DVI_1280W_720H,
    OMAP_RES_INVALID=8
}OMAP_LCD_DVI_RES;

//-----------------------------------------------------------------------------
//
//  Type: DDK_CLOCK_SIGNAL
//
//  Clock signal name for querying/setting clock configuration.
//
//-----------------------------------------------------------------------------
typedef enum
{
    DDK_CLOCK_SIGNAL_MCUPLL     = 0,
    DDK_CLOCK_SIGNAL_USBPLL     = 1,
    DDK_CLOCK_SIGNAL_ARM        = 2,
    DDK_CLOCK_SIGNAL_AHB        = 3,
    DDK_CLOCK_SIGNAL_IPG        = 4,
    DDK_CLOCK_SIGNAL_USBCLK     = 5,
    
    DDK_CLOCK_SIGNAL_END        = 6,

    DDK_CLOCK_SIGNAL_START_AHB_CLK      = 9,

    DDK_CLOCK_SIGNAL_AHB_ATA    = 10,
    DDK_CLOCK_SIGNAL_AHB_BROM   = 11,
    DDK_CLOCK_SIGNAL_AHB_CSI    = 12,
    DDK_CLOCK_SIGNAL_AHB_EMI    = 13,
    DDK_CLOCK_SIGNAL_AHB_ESAI   = 14,
    DDK_CLOCK_SIGNAL_AHB_ESDHC1 = 15,
    DDK_CLOCK_SIGNAL_AHB_ESDHC2 = 16,
    DDK_CLOCK_SIGNAL_AHB_FEC    = 17,
    DDK_CLOCK_SIGNAL_AHB_LCDC   = 18,
    DDK_CLOCK_SIGNAL_AHB_RTIC   = 19,
    DDK_CLOCK_SIGNAL_AHB_SDMA   = 20,
    DDK_CLOCK_SIGNAL_AHB_SLCDC  = 21,
    DDK_CLOCK_SIGNAL_AHB_USBOTG = 22,

    DDK_CLOCK_SIGNAL_START_PER_CLK = 29,

    DDK_CLOCK_SIGNAL_PER_CSI    = 30,
    DDK_CLOCK_SIGNAL_PER_EPIT   = 31, 
    DDK_CLOCK_SIGNAL_PER_ESAI   = 32, 
    DDK_CLOCK_SIGNAL_PER_ESDHC1 = 33, 
    DDK_CLOCK_SIGNAL_PER_ESDHC2 = 34, 
    DDK_CLOCK_SIGNAL_PER_GPT    = 35, 
    DDK_CLOCK_SIGNAL_PER_I2C    = 36, 
    DDK_CLOCK_SIGNAL_PER_LCDC   = 37,
    DDK_CLOCK_SIGNAL_PER_NFC    = 38,
    DDK_CLOCK_SIGNAL_PER_OWIRE  = 39,
    DDK_CLOCK_SIGNAL_PER_PWM    = 40,
    DDK_CLOCK_SIGNAL_PER_SIM1   = 41,
    DDK_CLOCK_SIGNAL_PER_SIM2   = 42,
    DDK_CLOCK_SIGNAL_PER_SSI1   = 43,
    DDK_CLOCK_SIGNAL_PER_SSI2   = 44,
    DDK_CLOCK_SIGNAL_PER_UART   = 45,

    DDK_CLOCK_SIGNAL_ENUM_END   = 46,

} DDK_CLOCK_SIGNAL;

//------------------------------------------------------------------------------
//
//  Enum:  OAL_KITL_FLAGS
//
//  This enumeration specifies optional flags passed as part of OAL_KITL_ARGS
//  structure.
//  
typedef enum {
    OAL_KITL_FLAGS_ENABLED  = 0x0001,   // Kitl enable
    OAL_KITL_FLAGS_PASSIVE  = 0x0002,   // Kitl passive mode
    OAL_KITL_FLAGS_DHCP     = 0x0004,   // DHCP enable
    OAL_KITL_FLAGS_VMINI    = 0x0008,   // VMINI enable
    OAL_KITL_FLAGS_POLL     = 0x0010,   // Use polling (no interrupt)
    OAL_KITL_FLAGS_EXTNAME  = 0x0020    // Extend device name
} OAL_KITL_FLAGS;

typedef enum _INTERFACE_TYPE {
  InterfaceTypeUndefined   = -1,
  Internal                 = 0,
  Isa                      = 1,
  Eisa                     = 2,
  MicroChannel             = 3,
  TurboChannel             = 4,
  PCIBus                   = 5,
  VMEBus                   = 6,
  NuBus                    = 7,
  PCMCIABus                = 8,
  CBus                     = 9,
  MPIBus                   = 10,
  MPSABus                  = 11,
  ProcessorInternal        = 12,
  InternalPowerBus         = 13,
  PNPISABus                = 14,
  PNPBus                   = 15,
  Vmcs                     = 16,
  MaximumInterfaceType     = 17 
} INTERFACE_TYPE, *PINTERFACE_TYPE;


typedef struct _DEVICE_LOCATION {
    unsigned long   IfcType;                  // One of the INTERFACE_TYPE enum, typically PCIBus
    unsigned long   BusNumber;                // Bus number, typically the PCI bus number the device resides on
    unsigned long   LogicalLoc;               // Defines logical location of device.  See above for PCIbus example.
    void            *PhysicalLoc;             // Reserved for future use
    unsigned long   Pin;                      // PCIbus IRQ pin
} DEVICE_LOCATION, *PDEVICE_LOCATION;

//------------------------------------------------------------------------------
//
//  Type:  OAL_KITL_ARGS
//
//  This type is used to pass parameters to OALKitlInit function. When
//  another KITL transport protocol is added this structure can change by 
//  adding new structure in union.
//
typedef struct {
    unsigned long flags;
    DEVICE_LOCATION devLoc;             // KITL device location
    union {
        struct {                        // Serial class parameters
            unsigned long baudRate;
            unsigned long dataBits;
            unsigned long stopBits;
            unsigned long parity;
        };         
        struct {                        // Ether class parameters
            u16 mac[3];
            unsigned long ipAddress;
            unsigned long ipMask;
            unsigned long ipRoute;
        };
    };
} OAL_KITL_ARGS;

//------------------------------------------------------------------------------
//
//  Type:  OAL_ARG_HEADER
//
//  This type define arguments header. It should be used at start of argument
//  structure to identify it and verify version.
//
typedef struct {
    unsigned long   signature;
    u16             oalVersion;
    u16             bspVersion;
} OAL_ARGS_HEADER;

//------------------------------------------------------------------------------
//
//  Type:  BSP_ARGS
//
//  Shared argument structure used by the bootloader and the OS image.
//
typedef struct {
    /*OAL_ARGS_HEADER     header;
    u8                  deviceId[16];   // Device identification
    OAL_KITL_ARGS       kitl;
    unsigned long       clockFreq[DDK_CLOCK_SIGNAL_ENUM_END];
    unsigned long       updateMode;
    unsigned long       SDRAMSize;*/
    
    OAL_ARGS_HEADER header;
    unsigned long  updateMode;           // Should IPL run in update mode?
    unsigned long  coldBoot;             // Cold boot (erase registry)?
    unsigned long deviceID;             // Unique ID for development platform
    unsigned long imageLaunch;          // Image launch address
    OAL_KITL_ARGS kitl;                 // KITL parameters
    unsigned long oalFlags;             // OAL flags
    u16 mac[3];					        // mac address for the ethernet
    OMAP_LCD_DVI_RES  dispRes;                     // display resolution
    unsigned long ECCtype;    
    unsigned long opp_mode; 
    unsigned long SDRAMSize;  
    settingsinfo_t dh_board_settings;   // DH settings block
} BSP_ARGS;

/* End of File */
